PROGRAM BUILD_LAYER_MODEL
  !----------------------------------------------------------------------
  !---- Program designed to build a closed quad surface mesh from several
  !---- given sets of points representing surface topologies
  !----------------------------------------------------------------------
  !---- Shared modules
  !----------------------------------------------------------------------
  USE MESH_APPS
  USE IO_FILES
  USE MESH_FACES
  USE ISO_C_BINDING
  !----------------------------------------------------------------------
  IMPLICIT NONE
  !----------------------------------------------------------------------
  !---- Shared variable
  !----------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: dataFile, arg, cParam

  LOGICAL(C_BOOL)                  :: smoothBorder

  INTEGER(C_INT)                    :: nArg

  REAL(C_DOUBLE)                  :: param
  !----------------------------------------------------------------------
  !---- Get arguments
  !----------------------------------------------------------------------
  nArg = COMMAND_ARGUMENT_COUNT()
  IF ((nArg /= 1) .AND. (nArg /= 3)) THEN
     WRITE(6,'(a)') 'You did not provide the right number of arguments'
     WRITE(6,'(a)') 'Please provide:'
     WRITE(6,'(a)') '      - A configuration file'
     WRITE(6,'(a)') '      - A "-smoothBorder" tag (optional)'
     WRITE(6,'(a)') '      - A "parameter of smoothness [0;1] (optional)'
     STOP
  ENDIF
  !
  CALL GETARG(1,dataFile)
  !
  IF (nArg == 3) CALL GETARG(2,arg)
  !
  smoothBorder = .FALSE.
  IF (nArg == 3) THEN
     SELECT CASE (TRIM(arg))
     CASE('-smoothBorder')
        smoothBorder = .TRUE.
        CALL GETARG(3,cParam)
        CALL CHAR_TO_DOUBLE(cParam,param)
     CASE DEFAULT
        WRITE(6,'(a)') 'Unknown argument in command line'
        WRITE(6,'(a)') 'Maybe you meant "-smoothBorder"'
     END SELECT
  ENDIF
  !----------------------------------------------------------------------
  !---- Build model
  !----------------------------------------------------------------------
  checkForFaces = .FALSE.
  CALL BUILD_LAYER_MODEL_SUB(dataFile, smoothBorder, param)
  !----------------------------------------------------------------------
  WRITE(6,'(a)') "I finished building layer model"
  !----------------------------------------------------------------------
END PROGRAM BUILD_LAYER_MODEL
