PROGRAM CONVERT_ABSCISSA
  !------------------------------------------------------------------------
  !---- Program for scaling mesh node coordinates by a given factor
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE MATH_PHYS_CONSTANTS
  USE IO_FILES
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variable
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: datafile, cMask, absIn, absOut

  INTEGER(C_INT)                   :: nArg, nColumns, mask, nUnit, nlines, &
                                      iLine, iColumn, iRead
  
  REAL(C_DOUBLE)                   :: scaling

  REAL(C_DOUBLE), DIMENSION(:,:), &
                      ALLOCATABLE  :: fileData
  !----------------------------------------------------------------------
  !---- Get arguments
  !----------------------------------------------------------------------
  nArg = COMMAND_ARGUMENT_COUNT()
  IF (nArg /= 4) THEN
     WRITE(6,'(a)') 'You did not provide the right number of arguments'
     WRITE(6,'(a)') 'Please provide:'
     WRITE(6,'(a)') '      - A data file'
     WRITE(6,'(a)') '      - The columns position of the abscissa to change'
     WRITE(6,'(a)') '      - The current unit (freq - frequency, wl - wavelength, en - energy)'
     WRITE(6,'(a)') '      - The destination unit (freq, wl, en)'
     STOP
  ENDIF
  !
  CALL GETARG(1,datafile)
  CALL GETARG(2,cMask)
  CALL GETARG(3,absIn)
  CALL GETARG(4,absOut)
  !
  CALL CHAR_TO_INT(cMask, mask)
  !------------------------------------------------------------------------
  !---- Read data
  !------------------------------------------------------------------------
  CALL GET_SIZE_OF_FILE(datafile, nLines, nColumns)
  ALLOCATE(fileData(nLines,nColumns))
  !
  CALL GET_NEW_FILE_UNIT(nUnit)
  OPEN(UNIT=nUnit, FILE=TRIM(datafile))
  !
  DO iLine=1,nLines
        READ(nUnit,*) fileData(iLine,:)
  ENDDO
  CLOSE(nUnit)
  !------------------------------------------------------------------------
  !---- Convert to frequency
  !------------------------------------------------------------------------
  SELECT CASE(absIn)
     !
  CASE('wl')
        fileData(:,mask) = c0/fileData(:,mask)
  CASE('en')
        fileData(:,mask) = fileData(:,mask)/(2.0*Pi*hb)
  CASE DEFAULT
     ! 
  END SELECT
  !------------------------------------------------------------------------
  !---- Convert to wanted unit and write
  !------------------------------------------------------------------------
  SELECT CASE(absOut)
     !
  CASE('wl')
        fileData(:,mask) = c0/fileData(:,mask)
  CASE('en')
        fileData(:,mask) = 2.0*Pi*hb*fileData(:,mask)
  CASE DEFAULT
     ! 
  END SELECT
  !
  CALL REMOVE_FILE(dataFile)
  OPEN(UNIT=nUnit, FILE=TRIM(dataFile))
  !
  DO iLine=1,nLines
        WRITE(nUnit,*) fileData(iLine,:)
  ENDDO
  CLOSE(nUnit)
  !
  WRITE(6,*) 'Successfully converted: ', TRIM(dataFile), ' from ', TRIM(absIn), ' to ', TRIM(absOut)
  !------------------------------------------------------------------------
  !----
  !------------------------------------------------------------------------
END PROGRAM CONVERT_ABSCISSA
