MODULE BASES

  !====================================================================
  ! Computation of global arrays associated to basis functions 
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_INITIALIZATION(Bases_Type)
    !------------------------------------------------------------------
    !---- Initialisazation of global arrays associated to basis expansions
    !---- on the reference tetrahedron Tref
    !
    !---- IN
    !---- Bases_Type : Type of polynomial base
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE BASES_LAGRANGE
    USE MESH_DATA
    USE REFERENCE_CELLS
    USE MESH_ELEMENTS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(LEN=1, KIND=C_CHAR), INTENT(IN) :: Bases_Type
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Initialize base
    !------------------------------------------------------------------
    SELECT CASE (Bases_Type)
    CASE ('L') ! Lagrange basis functions
       IF (Mesh%nCellTet > 0) THEN
          CALL BASES_LAGRANGE_DOF_TET
          CALL BASES_LAGRANGE_PERM_TET
          CALL BASES_LAGRANGE_MATRICES_TET
       ENDIF
       !
       IF (Mesh%nCellHex > 0) THEN
          CALL BASES_LAGRANGE_DOF_HEX
          CALL BASES_LAGRANGE_PERM_HEX
          CALL BASES_LAGRANGE_MATRICES_HEX
       ENDIF
       !
       IF (Mesh%nFacHyb > 0) THEN
          ALLOCATE(mSurfHyb (Mesh%nFacHyb))
          ALLOCATE(mSurfHybF(Mesh%nFacHyb))
       ENDIF
       !
       ORDER_TO_NDOFV => ORDER_TO_NDOFV_LAGRANGE
       ORDER_TO_NDOFS => ORDER_TO_NDOFS_LAGRANGE
    CASE DEFAULT
       WRITE(6, '(a)') 'Unknwon  basis function type'
       WRITE(6, '(a)') 'Received basis function type: ',Bases_Type
       CALL TILT
    END SELECT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_INITIALIZATION

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE BASES
