MODULE BASES_LAGRANGE_DG_MATRICES

  !====================================================================
  ! DG matrices integration with Lagrange basis
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE BASES_LAGRANGE_ORDER_TO_NDOF
  USE BASES_LAGRANGE_POLYNOMIALS
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_MATRICES_TET
    !------------------------------------------------------------------
    !---- Computation of elementary matrices on the reference tetrahedron
    !
    !---- IN
    !
    !---- OUT
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    CHARACTER(LEN=3, KIND=C_CHAR)  :: cellType, faceType
    
    INTEGER(C_INT)                 :: maxOrder, nDofVMax, i

    REAL(C_DOUBLE)                 :: coeffMass, coeffStiff, coeffSurf, vol
    REAL(C_DOUBLE), DIMENSION(3)   :: dirX, dirY, dirZ
    REAL(C_DOUBLE), DIMENSION(3,4) :: vtk
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- A few coefficients and initializations
    !------------------------------------------------------------------
    !---- Vertices of the reference tetrahedron
    vtk(1:3,1) = Tref_Vert1
    vtk(1:3,2) = Tref_Vert2
    vtk(1:3,3) = Tref_Vert3
    vtk(1:3,4) = Tref_Vert4
    !
    dirX(:) = (/ 1.0, 0.0, 0.0 /)
    dirY(:) = (/ 0.0, 1.0, 0.0 /)
    dirZ(:) = (/ 0.0, 0.0, 1.0 /)    
    !
    !---- Determine maximal polynomial order and # of dof
    maxOrder   = 4
    cellType   = 'TET'
    faceType   = 'TRI'
    coeffMass  = 1.0
    coeffStiff = 1.0
    coeffSurf  = 1.0    
    vol        = Tref_vol
    CALL ORDER_TO_NDOFV_LAGRANGE(maxOrder, cellType, nDofVMax)
    !------------------------------------------------------------------
    !---- Allocate all matrices
    !------------------------------------------------------------------
    CALL BASES_LAGRANGE_MASS_MATRIX        (maxOrder, nDofVMax, vtk, cellType, &
         coeffMass, massTet)
    CALL BASES_LAGRANGE_INVERSE_MASS_MATRIX(maxOrder, nDofVMax,      cellType, &
         massTet, iMassTet)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vtk, cellType, &
         dirX, vol, coeffStiff, rdxTet)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vtk, cellType, &
         dirY, vol, coeffStiff, rdyTet)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vtk, cellType, &
         dirZ, vol, coeffStiff, rdzTet)
    CALL BASES_LAGRANGE_SURFACE_MATRIX     (maxOrder, nDofVMax, vtk, cellType, &
         faceType,  coeffSurf, mSurfTet)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_MATRICES_TET
  
  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_MATRICES_HEX
    !------------------------------------------------------------------
    !---- Computation of elementary matrices on the reference hexahedron
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    CHARACTER(LEN=3, KIND=C_CHAR)  :: cellType, faceType
    
    INTEGER(C_INT)                 :: maxOrder, nDofVMax, i

    REAL(C_DOUBLE)                 :: coeffMass, coeffStiff, coeffSurf, vol
    REAL(C_DOUBLE), DIMENSION(3)   :: dirX, dirY, dirZ
    REAL(C_DOUBLE), DIMENSION(3,8) :: vhk
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- A few coefficients and initializations
    !------------------------------------------------------------------
    !---- Vertices of the reference hexahedron
    vhk(1:3,1) = Href_Vert1
    vhk(1:3,2) = Href_Vert2
    vhk(1:3,3) = Href_Vert3
    vhk(1:3,4) = Href_Vert4
    vhk(1:3,5) = Href_Vert5
    vhk(1:3,6) = Href_Vert6
    vhk(1:3,7) = Href_Vert7
    vhk(1:3,8) = Href_Vert8
    !
    dirX(:) = (/ 1.0, 0.0, 0.0 /)
    dirY(:) = (/ 0.0, 1.0, 0.0 /)
    dirZ(:) = (/ 0.0, 0.0, 1.0 /)    
    !
    !---- Determine maximal polynomial order and # of dof
    maxOrder   = 4
    cellType   = 'HEX'
    faceType   = 'QUA'
    coeffMass  = 1.0/8.0
    coeffStiff = 1.0/4.0
    coeffSurf  = 1.0/4.0
    vol        = Href_vol
    CALL ORDER_TO_NDOFV_LAGRANGE(maxOrder, cellType, nDofVMax)    
    !------------------------------------------------------------------
    !---- Allocate all matrices
    !------------------------------------------------------------------
    CALL BASES_LAGRANGE_MASS_MATRIX        (maxOrder, nDofVMax, vhk, cellType, &
         coeffMass, massHex)
    CALL BASES_LAGRANGE_INVERSE_MASS_MATRIX(maxOrder, nDofVMax,      cellType, &
         massHex, iMassHex)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vhk, cellType, &
         dirX, vol, coeffStiff, rdxHex)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vhk, cellType, &
         dirY, vol, coeffStiff, rdyHex)
    CALL BASES_LAGRANGE_STIFFNESS_MATRIX   (maxOrder, nDofVMax, vhk, cellType, &
         dirZ, vol, coeffStiff, rdzHex)
    CALL BASES_LAGRANGE_SURFACE_MATRIX     (maxOrder, nDofVMax, vhk, cellType, &
         faceType, coeffSurf, mSurfHex)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_MATRICES_HEX

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_MASS_MATRIX(maxOrder, nDofVMax, verts, &
       cellType, coeffMass, massMatrix)
    !------------------------------------------------------------------
    !---- Compute DG mass matrix
    !
    !---- IN
    !---- maxOrder   : maximal polynomial order
    !---- nDofVMax   : maximal # of dofs
    !---- verts      : vertices
    !---- cellType   : type of cell
    !---- coeffMass  : multiplicative coefficient for mass matrix
    !
    !---- OUT
    !---- massMatrix : output mass matrix
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE QUADRATURES_3D
    USE MATRICES_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=*),        INTENT(INOUT) :: cellType
    
    INTEGER(C_INT),                       INTENT(INOUT) :: maxOrder, nDofVMax

    REAL(C_DOUBLE),                       INTENT(IN)    :: coeffMass
    REAL(C_DOUBLE),       DIMENSION(:,:), INTENT(INOUT) :: verts

    TYPE(matrix_sq_real), DIMENSION(:), ALLOCATABLE     :: massMatrix
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    INTEGER(C_INT)                                      :: deg, iDeg, nDofV, i, j, k

    REAL(C_DOUBLE)                                      :: val
    
    REAL(C_DOUBLE), DIMENSION(:), ALLOCATABLE           :: phi
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Mass matrix computation
    !------------------------------------------------------------------
    ALLOCATE(massMatrix(0:maxOrder))
    !
    DO deg=0,maxOrder
       CALL ORDER_TO_NDOFV_LAGRANGE(deg, cellType, nDofV)
       ALLOCATE(massMatrix(deg)%tab(nDofV,nDofV))
       massMatrix(deg)%ndof = nDofV
       massMatrix(deg)%tab  = 0.d0
       iDeg = 2*deg
       CALL BUILD_QUADRATURE_3D(cellType, iDeg)
       !
       DO j=1,nDofV ! Loop on dofs
          DO k=1,nDofV ! Loop on dofs
             val = 0.0d0
             !
             DO i=1,nq3d(iDeg) ! Integration
                CALL BASES_LAGRANGE_POLY(cellType, verts, qx3d(i), qy3d(i), qz3d(i), deg, phi)
                val = val + phi(j)*phi(k)*qw3d(i)*coeffMass
             ENDDO
             !
             massMatrix(deg)%tab(j,k) = val
          ENDDO
       ENDDO
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_MASS_MATRIX

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_INVERSE_MASS_MATRIX(maxOrder, nDofVMax, &
       cellType, massMatrix, iMassMatrix)
    !------------------------------------------------------------------
    !---- Compute inverse DG mass matrix
    !
    !---- IN
    !---- maxOrder    : maximal polynomial order
    !---- nDofVMax    : maximal # of dofs
    !---- cellType    : type of cell
    !---- massMatrix  : matrix to invert
    !
    !---- OUT
    !---- iMassMatrix : inverse mass matrix
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE MATRICES_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=*),       INTENT(INOUT) :: cellType
    
    INTEGER(C_INT),                      INTENT(INOUT) :: maxOrder, nDofVMax

    TYPE(matrix_sq_real), DIMENSION(:), ALLOCATABLE    :: massMatrix    
    TYPE(matrix_sq_real), DIMENSION(:), ALLOCATABLE    :: iMassMatrix    
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    CHARACTER(LEN=2, KIND=C_CHAR)                      :: solver
    
    INTEGER(C_INT)                                     :: deg, nDofV

    REAL(C_DOUBLE)                                     :: error
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Inverse mass matrix
    !------------------------------------------------------------------
    solver = 'CK'    
    ALLOCATE(iMassMatrix(0:maxOrder))
    !
    DO deg=0,maxOrder
       CALL ORDER_TO_NDOFV_LAGRANGE(deg, cellType, nDofV)       
       ALLOCATE(iMassMatrix(deg)%tab(nDofV,nDofV))
       iMassMatrix(deg)%ndof = nDofV
       iMassMatrix(deg)%tab  = 0.d0
       !
       CALL INVERSE_MATRIX(solver, nDofV, massMatrix(deg)%tab, iMassMatrix(deg)%tab, error)
       !               
       !---- Stop if error > 1.e-2
       IF (error > 1.0e-2) THEN
          WRITE(6, '(a12)') 'Error in routine BASES_LAGRANGE '
          WRITE(6, '(a46)') 'The inverse mass matrix is not satisfactory '
          WRITE(6, '(a34)') '||M_T.(M_T)^(-1) - Id|| > 1.E-2 '
          WRITE(6, '(a1)')  ''
          CALL TILT
       ENDIF
       !              
       !---- Warn if error > 1.e-6
       IF (error > 1.0e-6) THEN
          WRITE(6, '(a12)') 'Warning in BASES_LAGRANGE '
          WRITE(6, '(a50)') 'The inverse mass matrix may not be satisfactory '
          WRITE(6, '(a34)') '||M_T.(M_T)^(-1) - Id|| > 1.E-6 '
          WRITE(6, '(a1)')  ''
       ENDIF
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_INVERSE_MASS_MATRIX

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_STIFFNESS_MATRIX(maxOrder, nDofVMax, verts, &
       cellType, dir, cellVolume, coeffStiff, stiffMatrix)
    !------------------------------------------------------------------
    !---- Compute DG stiffness matrix
    !
    !---- IN
    !---- maxOrder    : maximal polynomial order
    !---- nDofVMax    : maximal # of dofs
    !---- verts       : vertices
    !---- cellType    : type of cell
    !---- dir         : direction in which derivative is done
    !---- cellVolume  : volume of reference cell
    !---- coeffStiff  : multiplicative coefficient for stiffness matrix
    !
    !---- OUT
    !---- stiffMatrix : output stiffness matrix
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE QUADRATURES_3D
    USE MATRICES_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=*),        INTENT(INOUT) :: cellType
    
    INTEGER(C_INT),                       INTENT(INOUT) :: maxOrder, nDofVMax

    REAL(C_DOUBLE),                       INTENT(IN)    :: cellVolume, coeffStiff
    REAL(C_DOUBLE),       DIMENSION(3),   INTENT(IN)    :: dir
    REAL(C_DOUBLE),       DIMENSION(:,:), INTENT(INOUT) :: verts

    TYPE(matrix_sq_real), DIMENSION(:), ALLOCATABLE     :: stiffMatrix
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    INTEGER(C_INT)                                      :: deg, nDofV, i, j, k, iDeg

    REAL(C_DOUBLE)                                      :: val, h, hInv, x0, y0, z0, x, y, z
    REAL(C_DOUBLE), DIMENSION(:),   ALLOCATABLE         :: phi, phiPlus, phiMinus    
    REAL(C_DOUBLE), DIMENSION(:,:), ALLOCATABLE         :: phiTot, deltaPhi
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Stiffness matrix computation
    !------------------------------------------------------------------
    h    = 1.0e-7
    hInv = 1.0/h    
    ALLOCATE(stiffMatrix(0:maxOrder))
    !
    DO deg=0,maxOrder
       CALL ORDER_TO_NDOFV_LAGRANGE(deg, cellType, nDofV)
       ALLOCATE(stiffMatrix(deg)%tab(nDofV,nDofV))
       stiffMatrix(deg)%ndof = nDofV
       stiffMatrix(deg)%tab  = 0.0
       !
       iDeg = MAX(2*deg-1,0)
       CALL BUILD_QUADRATURE_3D(cellType, iDeg)       
       !
       ALLOCATE(phiTot  (nDofV, nq3d(iDeg)))
       ALLOCATE(deltaPhi(nDofV, nq3d(iDeg)))
       !
       DO i=1,nq3d(iDeg)
          x0 = qx3d(i)
          y0 = qy3d(i)
          z0 = qz3d(i)
          !
          !---- Value of lagrange polynomial
          CALL BASES_LAGRANGE_POLY(cellType, verts, x0, y0, z0, deg, phi)
          phiTot(:,i) = phi(:)
          !
          !---- Spatial derivative
          x = x0 + dir(1)*h
          y = y0 + dir(2)*h
          z = z0 + dir(3)*h
          CALL BASES_LAGRANGE_POLY(cellType, verts, x, y, z, deg, phiPlus)
          x = x0 - dir(1)*h
          y = y0 - dir(2)*h
          z = z0 - dir(3)*h
          CALL BASES_LAGRANGE_POLY(cellType, verts, x, y, z, deg, phiMinus)
          deltaPhi(:,i) = 0.5*(phiPlus(:) - phiMinus(:))*hInv
       ENDDO
       !
       DO j=1,nDofV
          DO k=1,nDofV
             stiffMatrix(deg)%tab(k,j) = SUM(phiTot(j,:)*deltaPhi(k,:)*qw3d(:))*cellVolume*coeffStiff
          ENDDO
       ENDDO
       !
       DEALLOCATE(phiTot, deltaPhi)
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_STIFFNESS_MATRIX

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_SURFACE_MATRIX(maxOrder, nDofVMax, verts, &
       cellType, faceType, coeffSurf, surfMatrix)
    !------------------------------------------------------------------
    !---- Compute DG surface matrix
    !
    !---- IN
    !---- maxOrder   : maximal polynomial order
    !---- nDofVMax   : maximal # of dofs
    !---- verts      : vertices
    !---- cellType   : type of cell
    !---- faceType   : type of face
    !---- coeffSurf  : multiplicative coefficient for surface matrix
    !
    !---- OUT
    !---- surfMatrix : output surface matrix
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE QUADRATURES_2D
    USE MATRICES_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=*),           INTENT(INOUT) :: cellType, faceType
    
    INTEGER(C_INT),                          INTENT(INOUT) :: maxOrder, nDofVMax

    REAL(C_DOUBLE),                          INTENT(IN)    :: coeffSurf
    REAL(C_DOUBLE),         DIMENSION(:,:),  INTENT(INOUT) :: verts

    TYPE(matrix_rect_real), DIMENSION(:,:), ALLOCATABLE    :: surfMatrix
    !------------------------------------------------------------------
    !---- Local variable
    !------------------------------------------------------------------
    INTEGER(C_INT)                                         :: deg1, deg2, &
         nDofV1, nDofV2, nDofS1, nDofS2, i, j, k, iDeg

    REAL(C_DOUBLE)                                         :: val, zero, specVal
    
    REAL(C_DOUBLE), DIMENSION(:), ALLOCATABLE              :: phi, psi
    REAL(C_DOUBLE), DIMENSION(nDofVMax, nDofVMax)          :: matrix
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Surface matrix computation
    !------------------------------------------------------------------
    IF (faceType == 'TRI') specVal = 0.0
    IF (faceType == 'QUA') specVal =-1.0
    !
    ALLOCATE(surfMatrix(0:maxOrder,0:maxOrder))
    !
    DO deg1=0,maxOrder
       CALL ORDER_TO_NDOFV_LAGRANGE(deg1, cellType, nDofV1)
       CALL ORDER_TO_NDOFS_LAGRANGE(deg1, faceType, nDofS1)
       !
       DO deg2=0,maxOrder
          CALL ORDER_TO_NDOFV_LAGRANGE(deg2, cellType, nDofV2)
          CALL ORDER_TO_NDOFS_LAGRANGE(deg2, faceType, nDofS2)
          !
          surfMatrix(deg1,deg2)%nDof1 = nDofS1
          surfMatrix(deg1,deg2)%nDof2 = nDofS2
          ALLOCATE(surfMatrix(deg1,deg2)%tab(nDofS1, nDofS2))
          surfMatrix(deg1,deg2)%tab = 0.0
          !
          iDeg = deg1+deg2
          CALL BUILD_QUADRATURE_2D(faceType, iDeg)
          matrix(:,:) = 0.0
          !
          DO j=1,nDofV1
             DO k=1,nDofV2
                val = 0.0
                !
                DO i=1,nq2d(iDeg)
                   CALL BASES_LAGRANGE_POLY(cellType, verts, qx2d(i), qy2d(i), specVal, deg1, phi)
                   CALL BASES_LAGRANGE_POLY(cellType, verts, qx2d(i), qy2d(i), specVal, deg2, psi)
                   !
                   val = val + phi(j)*psi(k)*qw2d(i)*coeffSurf
                ENDDO
                !
                matrix(j,k) = val
             ENDDO
          ENDDO
          !
          !---- Elimination of the null rows and columns 
          CALL REDUCTION(matrix(1:nDofV1,1:nDofV2), surfMatrix(deg1,deg2)%tab, &
               nDofV1, nDofV2, nDofS1, nDofS2)
       ENDDO
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_SURFACE_MATRIX

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE BASES_LAGRANGE_DG_MATRICES
