MODULE BASES_LAGRANGE_DOF

  !====================================================================
  ! Dof computation with Lagrange basis
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE BASES_LAGRANGE_ORDER_TO_NDOF
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_DOF_TET
    !------------------------------------------------------------------
    !---- Computate dof coordinates on reference tetrahedron
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofV

    REAL(C_DOUBLE) :: q2, q3, q4, q6
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    q2 = 1.d0/2.d0
    q3 = 1.d0/3.d0
    q4 = 1.d0/4.d0
    q6 = 1.d0/6.d0
    !
    ALLOCATE(xDofTet(0:4))
    ALLOCATE(yDofTet(0:4))
    ALLOCATE(zDofTet(0:4))
    !
    DO deg=0,4
       CALL ORDER_TO_NDOFV_LAGRANGE(deg,'TET1',nDofV)
       !
       xDofTet(deg)%ndof = nDofV
       yDofTet(deg)%ndof = nDofV
       zDofTet(deg)%ndof = nDofV
       !
       ALLOCATE(xDofTet(deg)%tab(nDofV))
       ALLOCATE(yDofTet(deg)%tab(nDofV))
       ALLOCATE(zDofTet(deg)%tab(nDofV))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill dof coordinates
    !------------------------------------------------------------------
    !---- Zero order
    xDofTet(0)%tab = (/ 0.25d0 /)
    yDofTet(0)%tab = (/ 0.25d0 /)
    zDofTet(0)%tab = (/ 0.25d0 /)    
    !---- First order
    xDofTet(1)%tab = (/ 0.d0, 1.d0, 0.d0, 0.d0 /)
    yDofTet(1)%tab = (/ 0.d0, 0.d0, 1.d0, 0.d0 /)
    zDofTet(1)%tab = (/ 0.d0, 0.d0, 0.d0, 1.d0 /)
    !---- Second order
    xDofTet(2)%tab = (/ 0.d0, 2.d0, 0.d0, 0.d0, &
         1.d0, 1.d0, 0.d0, 1.d0, 0.d0, 0.d0 /)*q2
    yDofTet(2)%tab = (/ 0.d0, 0.d0, 2.d0, 0.d0, &
         0.d0, 1.d0, 1.d0, 0.d0, 0.d0, 1.d0 /)*q2
    zDofTet(2)%tab = (/ 0.d0, 0.d0, 0.d0, 2.d0, &
         0.d0, 0.d0, 0.d0, 1.d0, 1.d0, 1.d0 /)*q2
    !---- Third order
    xDofTet(3)%tab = (/ 0.d0, 3.d0, 0.d0, 0.d0, &
         1.d0, 2.d0, 2.d0, 1.d0, 0.d0, 0.d0,    &
         1.d0, 1.d0, 2.d0, 0.d0, 0.d0, 1.d0,    &
         0.d0, 0.d0, 0.d0, 1.d0 /)*q3
    yDofTet(3)%tab = (/ 0.d0, 0.d0, 3.d0, 0.d0, &
         0.d0, 0.d0, 1.d0, 2.d0, 2.d0, 1.d0,    &
         1.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,    &
         2.d0, 1.d0, 1.d0, 1.d0 /)*q3
    zDofTet(3)%tab = (/ 0.d0, 0.d0, 0.d0, 3.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,    &
         0.d0, 2.d0, 1.d0, 1.d0, 2.d0, 1.d0,    &
         1.d0, 2.d0, 1.d0, 1.d0 /)*q3
    !---- Fourth order
    xDofTet(4)%tab = (/ 0.d0, 4.d0, 0.d0, 0.d0, &
         1.d0, 2.d0, 3.d0, 3.d0, 2.d0, 1.d0,    &
         0.d0, 0.d0, 0.d0, 1.d0, 2.d0, 1.d0,    &
         1.d0, 2.d0, 3.d0, 0.d0, 0.d0, 0.d0,    &
         1.d0, 2.d0, 1.d0, 0.d0, 0.d0, 0.d0,    &
         0.d0, 0.d0, 0.d0, 2.d0, 1.d0, 1.d0, 1.d0 /)*q4
    yDofTet(4)%tab = (/ 0.d0, 0.d0, 4.d0, 0.d0, &
         0.d0, 0.d0, 0.d0, 1.d0, 2.d0, 3.d0,    &
         3.d0, 2.d0, 1.d0, 1.d0, 1.d0, 2.d0,    &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,    &
         0.d0, 0.d0, 0.d0, 3.d0, 2.d0, 1.d0,    &
         1.d0, 2.d0, 1.d0, 1.d0, 2.d0, 1.d0, 1.d0  /)*q4
    zDofTet(4)%tab = (/ 0.d0, 0.d0, 0.d0, 4.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,    &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0,    &
         3.d0, 2.d0, 1.d0, 1.d0, 2.d0, 3.d0,    &
         1.d0, 1.d0, 2.d0, 1.d0, 2.d0, 3.d0,    &
         1.d0, 1.d0, 2.d0, 1.d0, 1.d0, 2.d0, 1.d0 /)*q4
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_DOF_TET

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_DOF_TRI
    !------------------------------------------------------------------
    !---- Computate dof coordinates on reference triangle
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofS

    REAL(C_DOUBLE) :: q3
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    q3 = 1.0/3.0
    !
    ALLOCATE(xDofTri(0:2))
    ALLOCATE(yDofTri(0:2))
    !
    DO deg=0,2
       CALL ORDER_TO_NDOFV_LAGRANGE(deg,'TRI1',nDofS)
       !
       xDofTri(deg)%ndof = nDofS
       yDofTri(deg)%ndof = nDofS
       !
       ALLOCATE(xDofTri(deg)%tab(nDofS))
       ALLOCATE(yDofTri(deg)%tab(nDofS))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill dof coordinates
    !------------------------------------------------------------------
    !---- Zero order
    xDofTri(0)%tab = (/ 1.0 /)*q3
    yDofTri(0)%tab = (/ 1.0 /)*q3
    !---- First order
    xDofTri(1)%tab = (/ 0.0, 1.0, 0.0 /)
    yDofTri(1)%tab = (/ 0.0, 0.0, 1.0 /)
    !---- Second order
    xDofTri(2)%tab = (/ 0.0, 1.0, 0.0, 0.5, 0.5, 0.0 /)
    yDofTri(2)%tab = (/ 0.0, 0.0, 1.0, 0.0, 0.5, 0.5 /)
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_DOF_TRI

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_DOF_QUAD
    !------------------------------------------------------------------
    !---- Computate dof coordinates on reference quadrangle
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofS
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    ALLOCATE(xDofQua(0:2))
    ALLOCATE(yDofQua(0:2))
    !
    DO deg=0,2
       CALL ORDER_TO_NDOFS_LAGRANGE(deg,'QUA1',nDofS)
       !
       xDofQua(deg)%ndof = nDofS
       yDofQua(deg)%ndof = nDofS
       !
       ALLOCATE(xDofQua(deg)%tab(nDofS))
       ALLOCATE(yDofQua(deg)%tab(nDofS))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill dof coordinates
    !------------------------------------------------------------------
    !---- Zero order
    xDofQua(0)%tab = (/ 0.0 /)
    yDofQua(0)%tab = (/ 0.0 /)
    !---- First order
    xDofQua(1)%tab = (/-1.0, 1.0, 1.0,-1.0 /)
    yDofQua(1)%tab = (/-1.0,-1.0, 1.0, 1.0 /)
    !---- Second order
    xDofQua(2)%tab = (/-1.0, 1.0, 1.0,-1.0, 0.0, 1.0, 0.0,-1.0, 0.0 /)
    yDofQua(2)%tab = (/-1.0,-1.0, 1.0, 1.0,-1.0, 0.0, 1.0, 0.0, 0.0 /)
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_DOF_QUAD

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_DOF_HEX
    !------------------------------------------------------------------
    !---- Computate dof coordinates on reference tetrahedron
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofV

    REAL(C_DOUBLE) :: q2, q3
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    q2 = 1.d0/2.d0
    q3 = 1.d0/3.d0
    !
    ALLOCATE(xDofHex(0:4))
    ALLOCATE(yDofHex(0:4))
    ALLOCATE(zDofHex(0:4))
    !
    DO deg=0,4
       CALL ORDER_TO_NDOFV_LAGRANGE(deg,'HEX1',nDofV)
       !
       xDofHex(deg)%ndof = nDofV
       yDofHex(deg)%ndof = nDofV
       zDofHex(deg)%ndof = nDofV
       !
       ALLOCATE(xDofHex(deg)%tab(nDofV))
       ALLOCATE(yDofHex(deg)%tab(nDofV))
       ALLOCATE(zDofHex(deg)%tab(nDofV))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill dof coordinates
    !------------------------------------------------------------------
    !---- Zero order
    xDofHex(0)%tab = (/ 0.d0 /)
    yDofHex(0)%tab = (/ 0.d0 /)
    zDofHex(0)%tab = (/ 0.d0 /)
    !---- First order
    xDofHex(1)%tab = (/ -1.d0, 1.d0, 1.d0, -1.d0, &
         -1.d0, 1.d0, 1.d0, -1.d0 /)
    yDofHex(1)%tab = (/ -1.d0, -1.d0, 1.d0, 1.d0, &
         -1.d0, -1.d0, 1.d0, 1.d0 /)
    zDofHex(1)%tab = (/ -1.d0, -1.d0, -1.d0, -1.d0, &
         1.d0, 1.d0, 1.d0, 1.d0 /)
    !---- Second order
    xDofHex(2)%tab = (/  -1.d0, 1.d0, 1.d0, -1.d0, -1.d0, 1.d0, 1.d0, -1.d0,    &
         0.d0, 1.d0, 0d0, -1.d0, 0.d0, 0.d0, 1.d0, 0.d0, -1.d0, &
         0.d0, -1.d0, -1.d0, -1.d0, 1.d0, 1.d0, 1.d0, 0.d0,     &
         0.d0, 0.d0 /)
    yDofHex(2)%tab = (/ -1.d0, -1.d0, 1.d0, 1.d0, -1.d0, -1.d0, 1.d0, 1.d0, -1.d0,&
         0.d0, 1.d0, 0.d0, 0.d0, -1.d0, 0.d0, 1.d0, 0.d0,          &
         0.d0, -1.d0, 1.d0, 0.d0, -1.d0, 1.d0, 0.d0, -1.d0, 1.d0,  &
         0.d0 /)
    zDofHex(2)%tab = (/ -1.d0, -1.d0, -1.d0, -1.d0, 1.d0, 1.d0, 1.d0, 1.d0, -1.d0,&
         -1.d0, -1.d0, -1.d0, -1.d0, 1.d0, 1.d0, 1.d0, 1.d0, 1.d0, &
         0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0 /)
    !---- Third order
    xDofHex(3)%tab = (/ -3.d0, 3.d0, 3.d0, -3.d0, -3.d0, 3.d0, 3.d0, -3.d0,  &
         -1.d0, 1.d0, 3.d0, 3.d0, 1.d0, -1.d0,     &
         -3.d0, -3.d0, -1.d0, 1.d0, 1.d0, -1.d0,   &
         -3.d0, 3.d0, 3.d0, -3.d0, -1.d0, 1.d0,    &
         3.d0, 3.d0, 1.d0, -1.d0, -3.d0, -3.d0,    &
         -1.d0, 1.d0, 1.d0, -1.d0, -3.d0, 3.d0,    &
         3.d0, -3.d0, -1.d0, 1.d0, 3.d0, 3.d0,     &
         1.d0, -1.d0, -3.d0, -3.d0, -1.d0, 1.d0,   &
         1.d0, -1.d0, -1.d0, 1.d0, 3.d0, 3.d0,     &
         1.d0, -1.d0, -3.d0, -3.d0, -1.d0, 1.d0,   &
         1.d0, -1.d0 /)*q3
    yDofHex(3)%tab = (/ -3.d0, -3.d0, 3.d0, 3.d0, -3.d0, -3.d0, 3.d0, 3.d0,   &
         -3.d0, -3.d0, -1.d0, 1.d0, 3.d0, 3.d0,   &
         1.d0, -1.d0, -1.d0, -1.d0, 1.d0, 1.d0,   &
         -3.d0, -3.d0, 3d0, 3d0, -3.d0, -3.d0,    &
         -1.d0, 1.d0, 3d0, 3d0, 1.d0, -1.d0,      &
         -1.d0, -1.d0, 1.d0, 1.d0, -3.d0, -3.d0,  &
         3.d0, 3d0, -3.d0, -3.d0, -1.d0, 1.d0,    &
         3.d0, 3d0, 1.d0, -1.d0, -1.d0, -1.d0,    &
         1.d0, 1.d0, -3.d0, -3.d0, -1.d0, 1.d0,   &
         3.d0, 3d0, 1.d0, -1.d0, -1.d0, -1.d0,    &
         1.d0, 1.d0 /)*q3
    zDofHex(3)%tab = (/  -3.d0, -3.d0, -3.d0, -3.d0, 3d0, 3d0, 3d0, 3d0, &
         -3.d0,-3.d0,-3.d0,-3.d0,-3.d0,-3.d0,     &
         -3.d0,-3.d0,-3.d0,-3.d0,-3.d0,-3.d0,     &
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,     &
         -1.d0,-1.d0,-1.d0,-1.d0,-1.d0,-1.d0,     &
         -1.d0,-1.d0,-1.d0,-1.d0,1.d0,1.d0,       &
         1.d0,1.d0,1.d0,1.d0,1.d0,1.d0,           &
         1.d0,1.d0,1.d0,1.d0,1.d0,1.d0,           &
         1.d0,1.d0,3d0,3d0,3d0,3d0,               &
         3.d0,3d0,3d0,3d0,3d0,3d0,                &
         3.d0,3d0 /)*q3
    !---- Fourth order
    xDofHex(4)%tab = (/ -2.d0,    2.d0,    2.d0,    -2.d0,    -2.d0,&
         2.d0,    2.d0,    -2.d0,    &
         -1.d0,    0.d0,    1.d0,    2.d0,    2.d0,    2.d0,    &
         1.d0,    0.d0,    -1.d0,    -2.d0,    -2.d0,    -2.d0, &
         -1.d0,    0.d0,    1.d0,    1.d0,    1.d0,    0.d0,    &
         -1.d0,    -1.d0,    0.d0,  &
         -2.d0,    2.d0,    2.d0,    -2.d0,   &
         -1.d0,    0.d0,    1.d0,    2.d0,    2.d0,    2.d0,    &
         1.d0,    0.d0,    -1.d0,    -2.d0,    -2.d0,    -2.d0, &
         -1.d0,    0.d0,    1.d0,    1.d0,    1.d0,    0.d0,    &
         -1.d0,    -1.d0,    0.d0,    &
         -2.d0,    2.d0,    2.d0,    -2.d0,  &
         -1.d0,    0.d0,    1.d0,    2.d0,    2.d0,    2.d0,    &
         1.d0,    0.d0,    -1.d0,    -2.d0,    -2.d0,    -2.d0, &
         -1.d0,    0.d0,    1.d0,    1.d0,    1.d0,    0.d0,    &
         -1.d0,    -1.d0,    0.d0,    &
         -2.d0,    2.d0,    2.d0,    -2.d0,  &
         -1.d0,    0.d0,    1.d0,    2.d0,    2.d0,    2.d0,    &
         1.d0,    0.d0,    -1.d0,    -2.d0,    -2.d0,    -2.d0, &
         -1.d0,    0.d0,    1.d0,    1.d0,    1.d0,    0.d0,    &
         -1.d0,    -1.d0,    0.d0,   &
         -1.d0,    0.d0,    1.d0,    2.d0,    2.d0,    2.d0,    &
         1.d0,    0.d0,    -1.d0,    -2.d0,    -2.d0,    -2.d0, &
         -1.d0,    0.d0,    1.d0,    1.d0,    1.d0,    0.d0,    &
         -1.d0,    -1.d0,    0.d0/)*q2
    yDofHex(4)%tab = (/ -2.d0,   -2.d0,   2.d0,   2.d0,   -2.d0,  &
         -2.d0,   2.d0,   2.d0,&
         -2.d0,   -2.d0,   -2.d0,   -1.d0,   0.d0,   1.d0,  &
         2.d0,   2.d0,   2.d0,   1.d0,   0.d0,   -1.d0,     &
         -1.d0,   -1.d0,   -1.d0,   0.d0,   1.d0,   1.d0,   &
         1.d0,   0.d0,   0.d0,  &
         -2.d0,   -2.d0,   2.d0,   2.d0, &
         -2.d0,   -2.d0,   -2.d0,   -1.d0,   0.d0,   1.d0, &
         2.d0,   2.d0,   2.d0,   1.d0,   0.d0,   -1.d0,    &
         -1.d0,   -1.d0,   -1.d0,   0.d0,   1.d0,   1.d0,  &
         1.d0,   0.d0,   0.d0,  &
         -2.d0,   -2.d0,   2.d0,   2.d0,   &
         -2.d0,   -2.d0,   -2.d0,   -1.d0,   0.d0,   1.d0, &
         2.d0,   2.d0,   2.d0,   1.d0,   0.d0,   -1.d0,    &
         -1.d0,   -1.d0,   -1.d0,   0.d0,   1.d0,   1.d0,  &
         1.d0,   0.d0,   0.d0,  &
         -2.d0,   -2.d0,   2.d0,   2.d0,   &
         -2.d0,   -2.d0,   -2.d0,   -1.d0,   0.d0,   1.d0, &
         2.d0,   2.d0,   2.d0,   1.d0,   0.d0,   -1.d0,    &
         -1.d0,   -1.d0,   -1.d0,   0.d0,   1.d0,   1.d0,  &
         1.d0,   0.d0,   0.d0,  &
         -2.d0,   -2.d0,   -2.d0,   -1.d0,   0.d0,   1.d0, &
         2.d0,   2.d0,   2.d0,   1.d0,   0.d0,   -1.d0,    &
         -1.d0,   -1.d0,   -1.d0,   0.d0,   1.d0,   1.d0,  &
         1.d0,   0.d0,   0.d0  /)*q2
    zDofHex(4)%tab = (/ -2.d0,   -2.d0,   -2.d0,   -2.d0, &
         2.d0,   2.d0,   2.d0,   2.d0, &
         -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0, &
         -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0, &
         -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0,   -2.d0, &
         -2.d0,   -2.d0,   -2.d0,  &
         -1.d0,   -1.d0,   -1.d0,   -1.d0,   &
         -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0, &
         -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0, &
         -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0,   -1.d0, &
         -1.d0,   -1.d0,   -1.d0,  &
         0.d0,   0.d0,   0.d0,   0.d0,   &
         0.d0,   0.d0,   0.d0,   0.d0,   0.d0,   0.d0, &
         0.d0,   0.d0,   0.d0,   0.d0,   0.d0,   0.d0, &
         0.d0,   0.d0,   0.d0,   0.d0,   0.d0,   0.d0, &
         0.d0,   0.d0,   0.d0,  &
         1.d0,   1.d0,   1.d0,   1.d0,   &
         1.d0,   1.d0,   1.d0,   1.d0,   1.d0,   1.d0, &
         1.d0,   1.d0,   1.d0,   1.d0,   1.d0,   1.d0, &
         1.d0,   1.d0,   1.d0,   1.d0,   1.d0,   1.d0, &
         1.d0,   1.d0,   1.d0,  &
         2.d0,   2.d0,   2.d0,   2.d0,   2.d0,   2.d0, &
         2.d0,   2.d0,   2.d0,   2.d0,   2.d0,   2.d0, &
         2.d0,   2.d0,   2.d0,   2.d0,   2.d0,   2.d0, &
         2.d0,   2.d0,   2.d0 /)*q2
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_DOF_HEX

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE BASES_LAGRANGE_DOF


