MODULE BASES_LAGRANGE_ORDER_TO_NDOF

  !====================================================================
  ! Order to # of dof conversion for Lagrange basis
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================
  
  SUBROUTINE ORDER_TO_NDOFV_LAGRANGE(order, cellType, nDofV)
    !------------------------------------------------------------------
    !---- Number of volumic degrees of freedom for a given cell type
    !---- and polynomial order
    !
    !---- IN
    !---- order    : polynomial order
    !---- cellType : type of cell
    !
    !---- OUT
    !---- nDofV    : number of volumic degrees of freedom
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(LEN=*, KIND=C_CHAR), INTENT(IN)  :: cellType

    INTEGER(C_INT),                INTENT(IN)  :: order
    INTEGER(C_INT),                INTENT(OUT) :: nDofV
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Compute ndofv
    !------------------------------------------------------------------
    SELECT CASE(cellType)
    CASE('TET', 'TET1', 'TET2', 'TET3', 'TET4')
       nDofV = FLOOR((order+1)*(order+2)*(order+3)/6.0)
    CASE('HEX', 'HEX1', 'HEX2')
       nDofV = (order+1)**3
    END SELECT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE ORDER_TO_NDOFV_LAGRANGE

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE ORDER_TO_NDOFS_LAGRANGE(order, faceType, nDofS)
    !------------------------------------------------------------------
    !---- Number of surfacic degrees of freedom for a given face type
    !---- and polynomial order
    !
    !---- IN
    !---- order    : polynomial order
    !---- faceType : type of face
    !
    !---- OUT
    !---- nDofS    : number of surfacic degrees of freedom
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(LEN=*, KIND=C_CHAR), INTENT(IN)  :: faceType

    INTEGER(C_INT),                INTENT(IN)  :: order
    INTEGER(C_INT),                INTENT(OUT) :: nDofS
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Compute ndofs
    !------------------------------------------------------------------
    SELECT CASE(TRIM(faceType))
    CASE('TRI','TRI1', 'TRI2')
       nDofS = FLOOR(0.5*(order+1)*(order+2))
    CASE('QUA','QUA1', 'QUA2')
       nDofS = (order+1)**2
    END SELECT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE ORDER_TO_NDOFS_LAGRANGE

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE BASES_LAGRANGE_ORDER_TO_NDOF


