MODULE BASES_LAGRANGE_PERMUTATIONS

  !====================================================================
  ! Dof permutations for Lagrange basis
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE BASES_LAGRANGE_ORDER_TO_NDOF
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_PERM_TET
    !------------------------------------------------------------------
    !---- Computation of the dof permutations from a tri to a tet
    !
    !---- IN
    !
    !---- OUT
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofS
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    ALLOCATE(iPermDofTet(0:4))
    !
    DO deg=0,4
       CALL ORDER_TO_NDOFS_LAGRANGE(deg,'TRI1',nDofS)
       iPermDofTet(deg)%ndof = nDofS
       ALLOCATE(iPermDofTet(deg)%tab(nDofS,24))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill permutations
    !------------------------------------------------------------------
    !---- Zero order
    iPermDofTet(0)%tab = RESHAPE( (/ &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1/), &
         (/1,24/) )      
    !---- First order
    iPermDofTet(1)%tab = RESHAPE( (/ &
         1,2,3, &
         1,2,4, &
         1,3,2, &
         1,3,4, &
         1,4,2, &
         1,4,3, &
         2,1,3, &
         2,1,4, &
         2,3,1, &
         2,3,4, &
         2,4,1, &
         2,4,3, &
         3,1,2, &
         3,1,4, &
         3,2,1, &
         3,2,4, &
         3,4,1, &
         3,4,2, &
         4,1,2, &
         4,1,3, &
         4,2,1, &
         4,2,3, &
         4,3,1, &
         4,3,2/), &
         (/3,24/) )   
    !---- Second order
    iPermDofTet(2)%tab = RESHAPE( (/ &
         1,2,3, 5,6,7,  &   
         1,2,4, 5,8,9,  &
         1,3,2, 7,6,5,  &  
         1,3,4, 7,10,9, &
         1,4,2, 9,8,5,  &
         1,4,3, 9,10,7, &
         2,1,3, 5,7,6,  &
         2,1,4, 5,9,8,  &
         2,3,1, 6,7,5,  &
         2,3,4, 6,10,8, &
         2,4,1, 8,9,5,  &
         2,4,3, 8,10,6, &
         3,1,2, 7,5,6,  &
         3,1,4, 7,9,10, &
         3,2,1, 6,5,7,  &
         3,2,4, 6,8,10, &
         3,4,1, 10,9,7, &
         3,4,2, 10,8,6, &
         4,1,2, 9,5,8,  &
         4,1,3, 9,7,10, &
         4,2,1, 8,5,9,  &
         4,2,3, 8,6,10, &
         4,3,1, 10,7,9, &
         4,3,2, 10,6,8/), &
         (/6,24/) )   
    !---- Third order
    iPermDofTet(3)%tab = RESHAPE( (/ &
         1,2,3, 5,6,7,8,9,10,     11, &
         1,2,4, 5,6,13,12,15,14,  16, &
         1,3,2, 10,9,8,7,6,5,     11, &
         1,3,4, 10,9,17,18,15,14, 19, &
         1,4,2, 14,15,12,13,6,5,  16, &
         1,4,3, 14,15,18,17,9,10, 19, &
         2,1,3, 6,5,10,9,8,7,     11, &
         2,1,4, 6,5,14,15,12,13,  16, &
         2,3,1, 7,8,9,10,5,6,     11, &
         2,3,4, 7,8,17,18,12,13,  20, &
         2,4,1, 13,12,15,14,5,6,  16, &
         2,4,3, 13,12,18,17,8,7,  20, &
         3,1,2, 9,10,5,6,7,8,     11, &
         3,1,4, 9,10,14,15,18,17, 19, &
         3,2,1, 8,7,6,5,10,9,     11, &
         3,2,4, 8,7,13,12,18,17,  20, &
         3,4,1, 17,18,15,14,10,9, 19, &
         3,4,2, 17,18,12,13,7,8,  20, &
         4,1,2, 15,14,5,6,13,12,  16, &
         4,1,3, 15,14,10,9,17,18, 19, &
         4,2,1, 12,13,6,5,14,15,  16, &
         4,2,3, 12,13,7,8,17,18,  20, &
         4,3,1, 18,17,9,10,14,15, 19, &
         4,3,2, 18,17,8,7,13,12,  20/), &
         (/10,24/) )
    !---- Fourth order
    iPermDofTet(4)%tab = RESHAPE( (/ &
         1,2,3, 5,6,7,8,9,10,11,12,13,      14,15,16,   &
         1,2,4, 5,6,7,19,18,17,22,21,20,    23,24,25,   &
         1,3,2, 13,12,11,10,9,8,7,6,5,      14,16,15,   &
         1,3,4, 13,12,11,26,27,28,22,21,20, 29,30,31,   &
         1,4,2, 20,21,22,17,18,19,7,6,5,    23,25,24,   &
         1,4,3, 20,21,22,28,27,26,11,12,13, 29,31,30,   &
         2,1,3, 7,6,5,13,12,11,10,9,8,      15,14,16,   &
         2,1,4, 7,6,5,20,21,22,17,18,19,    24,23,25,   &
         2,3,1, 8,9,10,11,12,13,5,6,7,      15,16,14,   &
         2,3,4, 8,9,10,26,27,28,17,18,19,   32,33,34,   &
         2,4,1, 19,18,17,22,21,20,5,6,7,    24,25,23,   &
         2,4,3, 19,18,17,28,27,26,10,9,8,   32,34,33,   &
         3,1,2, 11,12,13,5,6,7,8,9,10,      16,14,15,   &
         3,1,4, 11,12,13,20,21,22,28,27,26, 30,29,31,   &
         3,2,1, 10,9,8,7,6,5,13,12,11,      16,15,14,   &
         3,2,4, 10,9,8,19,18,17,28,27,26,   33,32,34,   &
         3,4,1, 26,27,28,22,21,20,13,12,11, 30,31,29,   &
         3,4,2, 26,27,28,17,18,19,8,9,10,   33,34,32,   &
         4,1,2, 22,21,20,5,6,7,19,18,17,    25,23,24,   &
         4,1,3, 22,21,20,13,12,11,26,27,28, 31,29,30,   &
         4,2,1, 17,18,19,7,6,5,20,21,22,    25,24,23,   &
         4,2,3, 17,18,19,8,9,10,26,27,28,   34,32,33,   &
         4,3,1, 28,27,26,11,12,13,20,21,22, 31,30,29,   &
         4,3,2, 28,27,26,10,9,8,19,18,17,   34,33,32/), &
         (/15,24/) )
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_PERM_TET

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE BASES_LAGRANGE_PERM_HEX
    !------------------------------------------------------------------
    !---- Computation of the dof permutations from a quad to an hex
    !
    !---- IN
    !
    !---- OUT
    !    
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: deg, nDofS
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Allocate arrays
    !------------------------------------------------------------------
    ALLOCATE(iPermDofHex(0:4))
    !
    DO deg=0,4
       CALL ORDER_TO_NDOFS_LAGRANGE(deg,'QUA1',nDofS)
       iPermDofHex(deg)%ndof = nDofS
       ALLOCATE(iPermDofHex(deg)%tab(nDofS,48))
    ENDDO
    !------------------------------------------------------------------
    !---- Fill permutations
    !------------------------------------------------------------------
    !---- Zero order
    iPermDofHex(0)%tab = RESHAPE( (/ &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1, &
         1/), &
         (/1,48/) )      
    !---- First order
    iPermDofHex(1)%tab = RESHAPE( (/ &
         1, 2, 3, 4, &
         1, 2, 6, 5, &
         1, 4, 3, 2, &
         1, 4, 8, 5, &
         1, 5, 6, 2, &
         1, 5, 8, 4, &
         2, 1, 4, 3, &
         2, 1, 5, 6, &
         2, 3, 4, 1, &
         2, 3, 7, 6, &
         2, 6, 5, 1, &
         2, 6, 7, 3, &
         3, 2, 1, 4, &
         3, 2, 6, 7, &
         3, 4, 1, 2, &
         3, 4, 8, 7, &
         3, 7, 6, 2, &
         3, 7, 8, 4, &
         4, 1, 2, 3, &
         4, 1, 5, 8, &
         4, 3, 2, 1, &
         4, 3, 7, 8, &
         4, 8, 5, 1, &
         4, 8, 7, 3, &
         5, 1, 2, 6, &
         5, 1, 4, 8, &
         5, 6, 2, 1, &
         5, 6, 7, 8, &
         5, 8, 4, 1, &
         5, 8, 7, 6, &
         6, 2, 1, 5, &
         6, 2, 3, 7, &
         6, 5, 1, 2, &
         6, 5, 8, 7, &
         6, 7, 3, 2, &
         6, 7, 8, 5, &
         7, 3, 2, 6, &
         7, 3, 4, 8, &
         7, 6, 2, 3, &
         7, 6, 5, 8, &
         7, 8, 4, 3, &
         7, 8, 5, 6, &
         8, 4, 1, 5, &
         8, 4, 3, 7, &
         8, 5, 1, 4, &
         8, 5, 6, 7, &
         8, 7, 3, 4, &
         8, 7, 6, 5/), &
         (/4,48/) )   
    !---- Second order
    iPermDofHex(2)%tab = RESHAPE( (/ &
         1, 2, 3, 4,  9, 10, 11, 12, 13, &
         1, 2, 6, 5,  9, 22, 14, 19, 25, &
         1, 4, 3, 2, 12, 11, 10,  9, 13, &
         1, 4, 8, 5, 12, 20, 17, 19, 21, &
         1, 5, 6, 2, 19, 14, 22,  9, 25, &
         1, 5, 8, 4, 19, 17, 20, 12, 21, &
         2, 1, 4, 3,  9, 12, 11, 10, 13, &
         2, 1, 5, 6,  9, 19, 14, 22, 25, &
         2, 3, 4, 1, 10, 11, 12,  9, 13, &
         2, 3, 7, 6, 10, 23, 15, 22, 24, &
         2, 6, 5, 1, 22, 14, 19,  9, 25, &
         2, 6, 7, 3, 22, 15, 23, 10, 24, &
         3, 2, 1, 4, 10,  9, 12, 11, 13, &
         3, 2, 6, 7, 10, 22, 15, 23, 24, &
         3, 4, 1, 2, 11, 12,  9, 10, 13, &
         3, 4, 8, 7, 11, 20, 16, 23, 26, &
         3, 7, 6, 2, 23, 15, 22, 10, 24, &
         3, 7, 8, 4, 23, 16, 20, 11, 26, &
         4, 1, 2, 3, 12,  9, 10, 11, 13, &
         4, 1, 5, 8, 12, 19, 17, 20, 21, &
         4, 3, 2, 1, 11, 10,  9, 12, 13, &
         4, 3, 7, 8, 11, 23, 16, 20, 26, &
         4, 8, 5, 1, 20, 17, 19, 12, 21, &
         4, 8, 7, 3, 20, 16, 23, 11, 26, &
         5, 1, 2, 6, 19,  9, 22, 14, 25, &
         5, 1, 4, 8, 19, 12, 20, 17, 21, &
         5, 6, 2, 1, 14, 22,  9, 19, 25, &
         5, 6, 7, 8, 14, 15, 16, 17, 18, &
         5, 8, 4, 1, 17, 20, 12, 19, 21, &
         5, 8, 7, 6, 17, 16, 15, 14, 18, &
         6, 2, 1, 5, 22,  9, 19, 14, 25, &
         6, 2, 3, 7, 22, 10, 23, 15, 24, &
         6, 5, 1, 2, 14, 19,  9, 22, 25, &
         6, 5, 8, 7, 14, 17, 16, 15, 18, &
         6, 7, 3, 2, 15, 23, 10, 22, 24, &
         6, 7, 8, 5, 15, 16, 17, 14, 18, &
         7, 3, 2, 6, 23, 10, 22, 15, 24, &
         7, 3, 4, 8, 23, 11, 20, 16, 26, &
         7, 6, 2, 3, 15, 22, 10, 23, 24, &
         7, 6, 5, 8, 15, 14, 17, 16, 18, &
         7, 8, 4, 3, 16, 20, 11, 23, 26, &
         7, 8, 5, 6, 16, 17, 14, 15, 18, &
         8, 4, 1, 5, 20, 12, 19, 17, 21, &
         8, 4, 3, 7, 20, 11, 23, 16, 26, &
         8, 5, 1, 4, 17, 19, 12, 20, 21, &
         8, 5, 6, 7, 17, 14, 15, 16, 18, &
         8, 7, 3, 4, 16, 23, 11, 20, 26, &
         8, 7, 6, 5, 16, 15, 14, 17, 18/), &
         (/9,48/) )   
    !---- Third order
    iPermDofHex(3)%tab = RESHAPE( (/ &
      	 1, 2, 3, 4,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, &
      	 1, 2, 6, 5,  9, 10, 22, 38, 54, 53, 37, 21, 25, 26, 42, 41, &
      	 1, 4, 3, 2, 16, 15, 14, 13, 12, 11, 10,  9, 17, 20, 19, 18, &
      	 1, 4, 8, 5, 16, 15, 24, 40, 59, 60, 37, 21, 32, 31, 47, 48, &
      	 1, 5, 6, 2, 21, 37, 53, 54, 38, 22, 10,  9, 25, 41, 42, 26, &
      	 1, 5, 8, 4, 21, 37, 60, 59, 40, 24, 15, 16, 32, 48, 47, 31, &
      	 2, 1, 4, 3, 10,  9, 16, 15, 14, 13, 12, 11, 18, 17, 20, 19, &
      	 2, 1, 5, 6, 10,  9, 21, 37, 53, 54, 38, 22, 26, 25, 41, 42, &
      	 2, 3, 4, 1, 11, 12, 13, 14, 15, 16,  9, 10, 18, 19, 20, 17, &
      	 2, 3, 7, 6, 11, 12, 23, 39, 56, 55, 38, 22, 27, 28, 44, 43, &
      	 2, 6, 5, 1, 22, 38, 54, 53, 37, 21,  9, 10, 26, 42, 41, 25, & 
      	 2, 6, 7, 3, 22, 38, 55, 56, 39, 23, 12, 11, 27, 43, 44, 28, &
      	 3, 2, 1, 4, 12, 11, 10,  9, 16, 15, 14, 13, 19, 18, 17, 20, &
       	 3, 2, 6, 7, 12, 11, 22, 38, 55, 56, 39, 23, 28, 27, 43, 44, &
      	 3, 4, 1, 2, 13, 14, 15, 16,  9, 10, 11, 12, 19, 20, 17, 18, &
      	 3, 4, 8, 7, 13, 14, 24, 40, 58, 57, 39, 23, 29, 30, 46, 45, &
      	 3, 7, 6, 2, 23, 39, 56, 55, 38, 22, 11, 12, 28, 44, 43, 27, & 
      	 3, 7, 8, 4, 23, 39, 57, 58, 40, 24, 14, 13, 29, 45, 46, 30, &
      	 4, 1, 2, 3, 15, 16,  9, 10, 11, 12, 13, 14, 20, 17, 18, 19, &
       	 4, 1, 5, 8, 15, 16, 21, 37, 60, 59, 40, 24, 31, 32, 48, 47, & 
      	 4, 3, 2, 1, 14, 13, 12, 11, 10,  9, 16, 15, 20, 19, 18, 17, &
      	 4, 3, 7, 8, 14, 13, 23, 39, 57, 58, 40, 24, 30, 29, 45, 46, &
      	 4, 8, 5, 1, 24, 40, 59, 60, 37, 21, 16, 15, 31, 47, 48, 32, &
      	 4, 8, 7, 3, 24, 40, 58, 57, 39, 23, 13, 14, 30, 46, 45, 29, & 
      	 5, 1, 2, 6, 37, 21,  9, 10, 22, 38, 54, 53, 41, 25, 26, 42, &
      	 5, 1, 4, 8, 37, 21, 16, 15, 24, 40, 59, 60, 48, 32, 31, 47, &
      	 5, 6, 2, 1, 53, 54, 38, 22, 10,  9, 21, 37, 41, 42, 26, 25, & 
      	 5, 6, 7, 8, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, &
      	 5, 8, 4, 1, 60, 59, 40, 24, 15, 16, 21, 37, 48, 47, 31, 32, &
      	 5, 8, 7, 6, 60, 59, 58, 57, 56, 55, 54, 53, 61, 64, 63, 62, &
      	 6, 2, 1, 5, 38, 22, 10,  9, 37, 21, 53, 54, 42, 26, 25, 41, &
      	 6, 2, 3, 7, 38, 22, 11, 12, 23, 39, 56, 55, 43, 27, 28, 44, &
      	 6, 5, 1, 2, 54, 53, 37, 21,  9, 10, 22, 38, 42, 41, 25, 26, &
      	 6, 5, 8, 7, 54, 53, 60, 59, 58, 57, 56, 55, 62, 61, 64, 63, &
      	 6, 7, 3, 2, 55, 56, 39, 23, 12, 11, 22, 38, 43, 44, 28, 27, &
      	 6, 7, 8, 5, 55, 56, 57, 58, 59, 60, 53, 54, 62, 63, 64, 61, &
      	 7, 3, 2, 6, 39, 23, 12, 11, 22, 38, 55, 56, 44, 28, 27, 43, &
      	 7, 3, 4, 8, 39, 23, 13, 14, 24, 40, 58, 57, 45, 29, 30, 46, &
      	 7, 6, 2, 3, 56, 55, 38, 22, 11, 12, 23, 39, 44, 43, 27, 28, &
      	 7, 6, 5, 8, 56, 55, 54, 53, 60, 59, 58, 57, 63, 62, 61, 64, &
      	 7, 8, 4, 3, 57, 58, 40, 24, 14, 13, 23, 39, 45, 46, 30, 29, &
      	 7, 8, 5, 6, 57, 58, 59, 60, 53, 54, 55, 56, 63, 64, 61, 62, &
      	 8, 4, 1, 5, 40, 24, 15, 16, 21, 37, 60, 59, 47, 31, 32, 48, &
      	 8, 4, 3, 7, 40, 24, 14, 13, 23, 39, 57, 58, 46, 30, 29, 45, &
      	 8, 5, 1, 4, 59, 60, 37, 21, 16, 15, 24, 40, 47, 48, 32, 31, &
      	 8, 5, 6, 7, 59, 60, 53, 54, 55, 56, 57, 58, 64, 61, 62, 63, &
      	 8, 7, 3, 4, 58, 57, 39, 23, 13, 14, 24, 40, 46, 45, 29, 30, &
      	 8, 7, 6, 5, 58, 57, 56, 55, 54, 53, 60, 59, 64, 63, 62, 61/), &
         (/16,48/) )    
    !---- Fourth order
    iPermDofHex(4)%tab = RESHAPE( (/ &
         1, 2, 3, 4, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, &
         22, 23, 24, 25, 26, 27, 28, 29,&
         1, 2, 6, 5, 9, 10, 11, 31, 56, 81, 107, 106, 105, 80, 55, 30, &
         34, 35, 36, 61, 86, 85, 84, 59, 60,&
         1, 4, 3, 2, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, &
         21, 28, 27, 26, 25, 24, 23, 22, 29, &
         1, 4, 8, 5, 20, 19, 18, 33, 58, 83, 114, 115, 116, 80, 55, &
         30, 45, 44, 43, 68, 93, 94, 95, 70, 69,&
         1, 5, 6, 2, 30, 55, 80, 105, 106, 107, 81, 56, 31, 11, 10, &
         9, 34, 59, 84, 85, 86, 61, 36, 35, 60,&
         1, 5, 8, 4, 30, 55, 80, 116, 115, 114, 83, 58, 33, 18, 19, 20, &
         45, 70, 95, 94, 93, 68, 43, 44, 69,&
         2, 1, 4, 3, 11, 10, 9, 20, 19, 18, 17, 16, 15, 14, 13, 12, 23, &
         22, 21, 28, 27, 26, 25, 24, 29, &
         2, 1, 5, 6, 11, 10, 9, 30, 55, 80, 105, 106, 107, 81, 56, 31, &
         36, 35, 34, 59, 84, 85, 86, 61, 60,&
         2, 3, 4, 1, 12, 13, 14, 15, 16, 17, 18, 19, 20, 9, 10, 11, 23, &
         24, 25, 26, 27, 28, 21, 22, 29,&
         2, 3, 7, 6, 12, 13, 14, 32, 57, 82, 110, 109, 108, 81, 56, 31, &
         37, 38, 39, 64, 89, 88, 87, 62, 63,& 
         2, 6, 5, 1, 31, 56, 81, 107, 106, 105, 80, 55, 30, 9, 10, 11, &
         36, 61, 86, 85, 84, 59, 34, 35, 60, &
         2, 6, 7, 3, 31, 56, 81, 108, 109, 110, 82, 57, 32, 14, 13, 12,&
         37, 62, 87, 88, 89, 64, 39, 38, 63,&
         3, 2, 1, 4, 14, 13, 12, 11, 10, 9, 20, 19, 18, 17, 16, 15, 25, &
         24, 23, 22, 21, 28, 27, 26, 29,&
         3, 2, 6, 7, 14, 13, 12, 31, 56, 81, 108, 109, 110, 82, 57, 32,&
         39, 38, 37, 62, 87, 88, 89, 64, 63,&
         3, 4, 1, 2, 15, 16, 17, 18, 19, 20, 9, 10, 11, 12, 13, 14, 25,&
         26, 27, 28, 21, 22, 23, 24, 29,&
         3, 4, 8, 7, 15, 16, 17, 33, 58, 83, 113, 112, 111, 82, 57, 32, &
         40, 41, 42, 67, 92, 91, 90, 65, 66,&
         3, 7, 6, 2, 32, 57, 82, 110, 109, 108, 81, 56, 31, 12, 13, 14, &
         39, 64, 89, 88, 87, 62, 37, 38, 63, &
         3, 7, 8, 4, 32, 57, 82, 111, 112, 113, 83, 58, 33, 17, 16, 15, &
         40, 65, 90, 91, 92, 67, 42, 41, 66,&
         4, 1, 2, 3, 18, 19, 20, 9, 10, 11, 12, 13, 14, 15, 16, 17, 27,&
         28, 21, 22, 23, 24, 25, 26, 29,&
         4, 1, 5, 8, 18, 19, 20, 30, 55, 80, 116, 115, 114, 83, 58, 33,&
         43, 44, 45, 70, 95, 94, 93, 68, 69,& 
         4, 3, 2, 1, 17, 16, 15, 14, 13, 12, 11, 10, 9, 20, 19, 18, 27, &
         26, 25, 24, 23, 22, 21, 28, 29,&
         4, 3, 7, 8, 17, 16, 15, 32, 57, 82, 111, 112, 113, 83, 58, 33, &
         42, 41, 40, 65, 90, 91, 92, 67, 66,&
         4, 8, 5, 1, 33, 58, 83, 114, 115, 116, 80, 55, 30, 20, 19, 18, &
         43, 68, 93, 94, 95, 70, 45, 44, 69, &
         4, 8, 7, 3, 33, 58, 83, 113, 112, 111, 82, 57, 32, 15, 16, 17, &
         42, 67, 92, 91, 90, 65, 40, 41, 66, &
         5, 1, 2, 6, 80, 55, 30, 9, 10, 11, 31, 56, 81, 107, 106, 105, &
         84, 59, 34, 35, 36, 61, 86, 85, 60,&
         5, 1, 4, 8, 80, 55, 30, 20, 19, 18, 33, 58, 83, 114, 115, 116,&
         95, 70, 45, 44, 43, 68, 93, 94, 69,&
         5, 6, 2, 1, 105, 106, 107, 81, 56, 31, 11, 10, 9, 30, 55, 80, &
         84, 85, 86, 61, 36, 35, 34, 59, 60, &
         5, 6, 7, 8, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, &
         115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,&
         5, 8, 4, 1, 116, 115, 114, 83, 58, 33, 18, 19, 20, 30, 55, 80, &
         95, 94, 93, 68, 43, 44, 45, 70, 69,&
         5, 8, 7, 6, 116, 115, 114, 113, 112, 111, 110, 109, 108, 107, &
         106, 105, 117, 124, 123, 122, 121, 120, 119, 118, 125,&
         6, 2, 1, 5, 81, 56, 31, 11, 10, 9, 30, 55, 80, 105, 106, 107, &
         86, 61, 36, 35, 34, 59, 84, 85, 60,&
         6, 2, 3, 7, 81, 56, 31, 12, 13, 14, 32, 57, 82, 110, 109, 108,&
         87, 62, 37, 38, 39, 64, 89, 88, 63, &
         6, 5, 1, 2, 107, 106, 105, 80, 55, 30, 9, 10, 11, 31, 56, 81,&
         86, 85, 84, 59, 34, 35, 36, 61, 60, &
         6, 5, 8, 7, 107, 106, 105, 116, 115, 114, 113, 112, 111, 110,&
         109, 108, 119, 118, 117, 124, 123, 122, 121, 120, 125,&
         6, 7, 3, 2, 108, 109, 110, 82, 57, 32, 14, 13, 12, 31, 56, 81,&
         87, 88, 89, 64, 39, 38, 37, 62, 63,&
         6, 7, 8, 5, 108, 109, 110, 111, 112, 113, 114, 115, 116, 105,&  
         106, 107, 119, 120, 121, 122, 123, 124, 117, 118, 125,&
         7, 3, 2, 6, 82, 57, 32, 14, 13, 12, 31, 56, 81, 108, 109, 110,&
         89, 64, 39, 38, 37, 62, 87, 88, 63,& 
         7, 3, 4, 8, 82, 57, 32, 15, 16, 17, 33, 58, 83, 113, 112, 111,&
         90, 65, 40, 41, 42, 67, 92, 91, 66,&
         7, 6, 2, 3, 110, 109, 108, 81, 56, 31, 12, 13, 14, 32, 57, 82,&
         89, 88, 87, 62, 37, 38, 39, 64, 63,&
         7, 6, 5, 8, 110, 109, 108, 107, 106, 105, 116, 115, 114, 113,&
         112, 111, 121, 120, 119, 118, 117, 124, 123, 122, 125,&
         7, 8, 4, 3, 111, 112, 113, 83, 58, 33, 17, 16, 15, 32, 57, 82,&
         90, 91, 92, 67, 42, 41, 40, 65, 66,&
         7, 8, 5, 6, 111, 112, 113, 114, 115, 116, 105, 106, 107, 108,& 
         109, 110, 121, 122, 123, 124, 117, 118, 119, 120, 125,&
         8, 4, 1, 5, 83, 58, 33, 18, 19, 20, 30, 55, 80, 116, 115,&
         114, 93, 68, 43, 44, 45, 70, 95, 94, 69,&
         8, 4, 3, 7, 83, 58, 33, 17, 16, 15, 32, 57, 82, 111, 112, &
         113, 92, 67, 42, 41, 40, 65, 90, 91, 66, &
         8, 5, 1, 4, 114, 115, 116, 80, 55, 30, 20, 19, 18, 33, 58, 83,&
         93, 94, 95, 70, 45, 44, 43, 68, 69,&
         8, 5, 6, 7, 114, 115, 116, 105, 106, 107, 108, 109, 110, 111,&
         112, 113, 123, 124, 117, 118, 119, 120, 121, 122, 125,&
         8, 7, 3, 4, 113, 112, 111, 82, 57, 32, 15, 16, 17, 33, 58, 83,&
         92, 91, 90, 65, 40, 41, 42, 67, 66,&
         8, 7, 6, 5, 113, 112, 111, 110, 109, 108, 107, 106, 105, 116,&  
         115, 114, 123, 122, 121, 120, 119, 118, 117, 124, 125 /), &
         (/25,48/) )
    !------------------------------------------------------------------
  END SUBROUTINE BASES_LAGRANGE_PERM_HEX

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE BASES_LAGRANGE_PERMUTATIONS


