MODULE DOF_PERMUTATIONS

  !====================================================================
  ! All fixed permutations:
  ! Tetrahedron vertices to triangular   face vertices
  ! Tetrahedron P2 dofs  to triangular   face P2 dofs
  ! Hexahedron  vertices to quadrangular face vertices
  ! Hexahedron  Q2 dofs  to quandragular face Q2 dofs

  ! WARNING : By convention, the HO vertices are stored in INRIA format
  ! Hence, they must be reordered through permutations at reading and
  ! writing everytime it is necessary
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Maximal numbers of vertices read for each kind of element
  !--------------------------------------------------------------------
  INTEGER(C_INT) :: nsTriMax  = 6  ! Quadratic triangle
  INTEGER(C_INT) :: nsQuadMax = 9  ! Quadratic quadrangle
  INTEGER(C_INT) :: nsTetMax  = 35 ! Quartic tetrahedron
  INTEGER(C_INT) :: nsHexMax  = 27 ! Quadratic hexahedron

  !--------------------------------------------------------------------
  ! Maximal numbers of vertices handled by each format
  !--------------------------------------------------------------------
  INTEGER(C_INT) :: nsTriMaxMedit    = 6
  INTEGER(C_INT) :: nsTriMaxInria    = 6
  INTEGER(C_INT) :: nsTriMaxGmsh     = 6
  INTEGER(C_INT) :: nsTriMaxRaw      = 6
  INTEGER(C_INT) :: nsTriMaxEnsight  = 6  
  INTEGER(C_INT) :: nsQuadMaxMedit   = 9
  INTEGER(C_INT) :: nsQuadMaxInria   = 9
  INTEGER(C_INT) :: nsQuadMaxGmsh    = 9
  INTEGER(C_INT) :: nsQuadMaxRaw     = 9
  INTEGER(C_INT) :: nsQuadMaxEnsight = 8
  INTEGER(C_INT) :: nsTetMaxMedit    = 10
  INTEGER(C_INT) :: nsTetMaxInria    = 10
  INTEGER(C_INT) :: nsTetMaxGmsh     = 10
  INTEGER(C_INT) :: nsTetMaxRaw      = 35
  INTEGER(C_INT) :: nsTetMaxEnsight  = 10
  INTEGER(C_INT) :: nsHexMaxMedit    = 27
  INTEGER(C_INT) :: nsHexMaxInria    = 27
  INTEGER(C_INT) :: nsHexMaxGmsh     = 27
  INTEGER(C_INT) :: nsHexMaxRaw      = 27
  INTEGER(C_INT) :: nsHexMaxEnsight  = 20
 
  !--------------------------------------------------------------------
  ! Permutations of the hexahedron vertices on its six quadrangular faces
  ! nvHex(1,k) gives the kth vertex number of face 1 (with k=1..6)
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(6,4), TARGET :: nvHex
  DATA nvHex/            &
       1, 5, 1, 2, 1, 4, &
       2, 6, 5, 6, 2, 3, &
       3, 7, 8, 7, 6, 7, &
       4, 8, 4, 3, 5, 8/

  !--------------------------------------------------------------------
  ! Permutations of the tetrahedron P2 dofs on its four triangular faces
  ! nvTet(1,k) gives the kth dof number of face 1 (with k=1..3 for 
  ! linear tetrahedron and k=1..6 for curved tetrahedron)
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(4,6), TARGET :: nvTet
  DATA nvTet/        &
       1, 2, 3,  4,  &
       2, 1, 4,  3,  &
       3, 4, 1,  2,  &
       5, 5, 10, 10, &
       6, 9, 9,  6,  &
       7, 8, 7,  8/

  !--------------------------------------------------------------------
  ! Permutations of the tetrahedron Pn dofs between formats
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(10) :: nvTetGmshToMedit, &
       nvTetInriaToMedit, nvTetMeditToInria,         &
       nvTetGmshToInria, nvTetInriaToEnsight
  INTEGER(C_INT), DIMENSION(35) :: nvTetIdentity  

  DATA nvTetGmshToMedit    /1, 2, 3, 4, 5, 6, 7, 8,  10, 9/
  DATA nvTetInriaToMedit   /1, 2, 3, 4, 5, 6, 7, 9,  8, 10/
  DATA nvTetMeditToInria   /1, 2, 3, 4, 5, 6, 7, 9,  8, 10/
  DATA nvTetGmshToInria    /1, 2, 3, 4, 5, 6, 7, 10, 8, 9 /
  DATA nvTetInriaToEnsight /1, 2, 3, 4, 5, 6, 7, 9,  8, 10/  
  DATA nvTetIdentity       /1, 2, 3, 4, 5, 6, 7, 8,  9, 10, &
       11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,  &
       24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35/

  !--------------------------------------------------------------------
  ! Permutations of the triangle P2 dofs between formats
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(6) :: nsTriGmshToMedit,  &
       nsTriInriaToMedit, nsTriMeditToInria,         &
       nsTriGmshToInria, nsTriInriaToEnsight,        &
       nsTriIdentity

  DATA nsTriGmshToMedit    /1, 2, 3, 4, 5, 6/
  DATA nsTriInriaToMedit   /1, 2, 3, 4, 5, 6/
  DATA nsTriMeditToInria   /1, 2, 3, 4, 5, 6/
  DATA nsTriGmshToInria    /1, 2, 3, 4, 5, 6/
  DATA nsTriInriaToEnsight /1, 2, 3, 4, 5, 6/
  DATA nsTriIdentity       /1, 2, 3, 4, 5, 6/

  !--------------------------------------------------------------------
  ! Permutations of the quadrangle Q2 dofs between formats
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(9) :: nsQuadGmshToMedit, &
       nsQuadInriaToMedit, nsQuadMeditToInria,       &
       nsQuadGmshToInria,  nsQuadIdentity
  INTEGER(C_INT), DIMENSION(8) :: nsQuadInriaToEnsight

  DATA nsQuadGmshToMedit    /1, 2, 3, 4, 5, 6, 7, 8, 9/
  DATA nsQuadInriaToMedit   /1, 2, 3, 4, 5, 6, 7, 8, 9/
  DATA nsQuadMeditToInria   /1, 2, 3, 4, 5, 6, 7, 8, 9/
  DATA nsQuadGmshToInria    /1, 2, 3, 4, 5, 6, 7, 8, 9/
  DATA nsQuadInriaToEnsight /1, 2, 3, 4, 5, 6, 7, 8/
  DATA nsQuadIdentity       /1, 2, 3, 4, 5, 6, 7, 8, 9/

  !--------------------------------------------------------------------
  ! Permutations of the hexahedron Q2 dofs between formats
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(27) :: nvHexGmshToMedit, &
       nvHexInriaToMedit, nvHexMeditToInria,         &
       nvHexGmshToInria, nvHexIdentity
  INTEGER(C_INT), DIMENSION(20) :: nvHexInriaToEnsight

  DATA nvHexGmshToMedit /                            &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  12, 14,   &
       10, 11, 13, 15, 16, 17, 19, 20, 18, 21, 23,   &
       22, 26, 24, 25, 27/
  DATA nvHexInriaToMedit/                            &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,   &
       12, 19, 22, 23, 20, 14, 15, 16, 17, 13, 21,   &
       25, 18, 24, 26, 27/
  DATA nvHexMeditToInria/                            &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  12, 14,   &
       10, 21, 17, 19, 20, 18, 26, 11, 16, 23, 13,   &
       15, 24, 22, 25, 27/
  DATA nvHexGmshToInria /                            &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  12, 14,   &
       10, 11, 13, 15, 16, 17, 19, 20, 18, 21, 23,   &
       22, 26, 24, 25, 27/
  DATA nvHexInriaToEnsight /                         &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,   &
       12, 14, 15, 16, 17, 19, 22, 23, 20/  
  DATA nvHexIdentity    /                            &
       1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11,   &
       12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,   &
       23, 24, 25, 26, 27/
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE FIND_PERM_NUMBER(faceType, ivF, ivE, perm)
    !----------------------------------------------------------------------
    !---- Identify the permutation that matches the local vertices of a 
    !---- triangle with those of the attached tetrahedron
    !
    !---- IN
    !---- faceType : type of face
    !---- ivF      : tri vertices
    !---- ivE      : tet vertices
    !
    !---- OUT
    !---- perm     : permutation number
    !
    !----------------------------------------------------------------------
    !---- Modules 
    !----------------------------------------------------------------------
    USE ISO_C_BINDING
    !----------------------------------------------------------------------
    IMPLICIT NONE
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    CHARACTER(LEN=4, KIND=C_CHAR), INTENT(IN)  :: faceType

    INTEGER, DIMENSION(:),         INTENT(IN)  :: ivF, ivE
    INTEGER(C_INT),                INTENT(OUT) :: perm
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Find permutation
    !----------------------------------------------------------------------
    SELECT CASE(faceType)
    CASE("TRI1","TRI2")
       CALL FIND_TRI_PERM_NUMBER(ivF,ivE,perm)
    CASE("QUA1","QUA2")
       CALL FIND_QUAD_PERM_NUMBER(ivF,ivE,perm)
    END SELECT
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE FIND_PERM_NUMBER

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE FIND_TRI_PERM_NUMBER(ivF, ivE, perm)
    !----------------------------------------------------------------------
    !---- Identify the permutation that matches the local vertices of a 
    !---- triangle with those of the attached tetrahedron
    !
    !---- IN
    !---- ivF  : tri vertices
    !---- ivE  : tet vertices
    !
    !---- OUT
    !---- perm : permutation number
    !
    !----------------------------------------------------------------------
    !---- Modules 
    !----------------------------------------------------------------------
    USE ISO_C_BINDING
    !----------------------------------------------------------------------
    IMPLICIT NONE
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    INTEGER, DIMENSION(:), INTENT(IN)  :: ivE, ivF
    INTEGER(C_INT),        INTENT(OUT) :: perm
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Find permutation
    !----------------------------------------------------------------------
    perm = 0

    IF (ivF(1) == ivE(1)) THEN
       IF (ivF(2) == ivE(2)) THEN
          IF (ivF(3) == ivE(3)) THEN
             perm = 1                   ! 1 2 3 
          ELSE
             perm = 2                   ! 1 2 4 
          ENDIF
       ELSE
          IF (ivF(2) == ivE(3)) THEN
             IF (ivF(3) == ivE(2)) THEN               
                perm = 3                ! 1 3 2 
             ELSE
                perm = 4                ! 1 3 4 
             ENDIF
          ELSE
             IF (ivF(3) == ivE(2)) THEN               
                perm = 5                ! 1 4 2 
             ELSE
                perm = 6                ! 1 4 3 
             ENDIF
          ENDIF
       ENDIF
    ELSE
       IF ((ivF(1) == ivE(2))) THEN
          IF (ivF(2) == ivE(1)) THEN
             IF (ivF(3) == ivE(3)) THEN 
                perm = 7                ! 2 1 3 
             ELSE
                perm = 8                ! 2 1 4 
             ENDIF
          ELSE
             IF (ivF(2) == ivE(3)) THEN 
                IF (ivF(3) == ivE(1)) THEN                
                   perm = 9             ! 2 3 1 
                ELSE 
                   perm = 10            ! 2 3 4 
                ENDIF
             ELSE
                IF (ivF(3) == ivE(1)) THEN                
                   perm = 11            ! 2 4 1 
                ELSE
                   perm = 12            ! 2 4 3 
                ENDIF
             ENDIF
          ENDIF
       ELSE
          IF (ivF(1) == ivE(3)) THEN 
             IF (ivF(2) == ivE(1)) THEN 
                IF (ivF(3) == ivE(2)) THEN 
                   perm = 13            ! 3 1 2 
                ELSE
                   perm = 14            ! 3 1 4 
                ENDIF
             ELSE
                IF (ivF(2) == ivE(2)) THEN 
                   IF (ivF(3) == ivE(1)) THEN                
                      perm = 15         ! 3 2 1 
                   ELSE
                      perm = 16         ! 3 2 4 
                   ENDIF
                ELSE
                   IF (ivF(3) == ivE(1)) THEN                
                      perm = 17         ! 3 4 1 
                   ELSE
                      perm = 18         ! 3 4 2 
                   ENDIF
                ENDIF
             ENDIF
          ELSE
             IF (ivF(2) == ivE(1)) THEN 
                IF (ivF(3) == ivE(2)) THEN 
                   perm = 19            ! 4 1 2 
                ELSE
                   perm = 20            ! 4 1 3 
                ENDIF
             ELSE
                IF (ivF(2) == ivE(2)) THEN 
                   IF (ivF(3) == ivE(1)) THEN                
                      perm = 21         ! 4 2 1 
                   ELSE
                      perm = 22         ! 4 2 3 
                   ENDIF
                ELSE
                   IF (ivF(3) == ivE(1)) THEN                
                      perm = 23         ! 4 3 1 
                   ELSE
                      perm = 24         ! 4 3 2 
                   ENDIF
                ENDIF
             ENDIF
          ENDIF
       ENDIF
    ENDIF
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE FIND_TRI_PERM_NUMBER

  !====================================================================
  !====================================================================
  !====================================================================

 SUBROUTINE FIND_QUAD_PERM_NUMBER(ivF, ivE, perm)
    !----------------------------------------------------------------------
    !---- Identify the permutation that matches the local vertices of a 
    !---- triangle with those of the attached tetrahedron
    !
    !---- IN
    !---- ivF  : tri vertices
    !---- ivE  : tet vertices
    !
    !---- OUT
    !---- perm : permutation number
    !
    !----------------------------------------------------------------------
    !---- Modules 
    !----------------------------------------------------------------------
    USE ISO_C_BINDING
    !----------------------------------------------------------------------
    IMPLICIT NONE
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    INTEGER, DIMENSION(:), INTENT(IN)  :: ivE, ivF
    INTEGER(C_INT),        INTENT(OUT) :: perm
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Find permutation
    !----------------------------------------------------------------------
    perm = 0
    !
    IF (ivF(1) .EQ. ivE(1)) THEN
       IF (ivF(2) .EQ. ivE(2)) THEN
          IF (ivF(3) .EQ. ivE(3)) THEN
             perm = 1                  ! 1 2 3 4 
          ELSE
             perm = 2                  ! 1 2 5 6 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(4)) THEN
          IF (ivF(3) .EQ. ivE(3)) THEN               
             perm = 3                  ! 1 4 3 2 
          ELSE
             perm = 4                  ! 1 4 8 5 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(5)) THEN
          IF (ivF(3) .EQ. ivE(6)) THEN               
             perm = 5                  ! 1 5 6 2 
          ELSE
             perm = 6                  ! 1 5 8 4 
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(2)) THEN
       IF (ivF(2) .EQ. ivE(1)) THEN
          IF (ivF(3) .EQ. ivE(4)) THEN
             perm = 7                  ! 2 1 4 3
          ELSE
             perm = 8                  ! 2 1 5 6 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(3)) THEN 
          IF (ivF(3) .EQ. ivE(4)) THEN               
             perm = 9                  ! 2 3 4 1
          ELSE
             perm = 10                 ! 2 3 7 6 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(6)) THEN
          IF (ivF(3) .EQ. ivE(5)) THEN               
             perm = 11                 ! 2 6 5 1 
          ELSE
             perm = 12                 ! 2 6 7 3 
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(3)) THEN
       IF (ivF(2) .EQ. ivE(2)) THEN
          IF (ivF(3) .EQ. ivE(1)) THEN
             perm = 13                 ! 3 2 1 4
          ELSE
             perm = 14                 ! 3 2 6 7 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(4)) THEN 
          IF (ivF(3) .EQ. ivE(1)) THEN               
             perm = 15                 ! 3 4 1 2
          ELSE
             perm = 16                 ! 3 4 8 7 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(7)) THEN
          IF (ivF(3) .EQ. ivE(6)) THEN               
             perm = 17                 ! 3 7 6 2 
          ELSE
             perm = 18                 ! 3 7 8 4 
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(4)) THEN
       IF (ivF(2) .EQ. ivE(1)) THEN
          IF (ivF(3) .EQ. ivE(2)) THEN
             perm = 19                 ! 4 1 2 3
          ELSE
             perm = 20                 ! 4 1 5 8 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(3)) THEN 
          IF (ivF(3) .EQ. ivE(2)) THEN               
             perm = 21                 ! 4 3 2 1
          ELSE
             perm = 22                 ! 4 3 7 8 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(8)) THEN
          IF (ivF(3) .EQ. ivE(5)) THEN               
             perm = 23                 ! 4 8 5 1 
          ELSE
             perm = 24                 ! 4 8 7 3 
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(5)) THEN
       IF (ivF(2) .EQ. ivE(1)) THEN
          IF (ivF(3) .EQ. ivE(2)) THEN
             perm = 25                 ! 5 1 2 6
          ELSE
             perm = 26                 ! 5 1 4 8 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(6)) THEN 
          IF (ivF(3) .EQ. ivE(4)) THEN               
             perm = 27                 ! 5 6 2 1
          ELSE
             perm = 28                 ! 5 6 7 8 
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(8)) THEN
          IF (ivF(3) .EQ. ivE(4)) THEN               
             perm = 29                 ! 5 8 4 1
          ELSE
             perm = 30                 ! 5 8 7 6
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(6)) THEN
       IF (ivF(2) .EQ. ivE(2)) THEN
          IF (ivF(3) .EQ. ivE(1)) THEN
             perm = 31                 ! 6 2 1 5
          ELSE
             perm = 32                 ! 6 2 3 7
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(5)) THEN 
          IF (ivF(3) .EQ. ivE(1)) THEN               
             perm = 33                 ! 6 5 1 2
          ELSE
             perm = 34                 ! 6 5 8 7
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(7)) THEN
          IF (ivF(3) .EQ. ivE(3)) THEN               
             perm = 35                 ! 6 7 3 2 
          ELSE
             perm = 36                 ! 6 7 8 5
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(7)) THEN
       IF (ivF(2) .EQ. ivE(3)) THEN
          IF (ivF(3) .EQ. ivE(2)) THEN
             perm = 37                 ! 7 3 2 6
          ELSE
             perm = 38                 ! 7 3 4 8
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(6)) THEN 
          IF (ivF(3) .EQ. ivE(2)) THEN               
             perm = 39                 ! 7 6 2 3
          ELSE
             perm = 40                 ! 7 6 5 8
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(8)) THEN
          IF (ivF(3) .EQ. ivE(4)) THEN               
             perm = 41                 ! 7 8 4 3
          ELSE
             perm = 42                 ! 7 8 5 6
          ENDIF
       ENDIF
       !
    ELSEIF (ivF(1) .EQ. ivE(8)) THEN
       IF (ivF(2) .EQ. ivE(4)) THEN
          IF (ivF(3) .EQ. ivE(1)) THEN
             perm = 43                 ! 8 4 1 5
          ELSE
             perm = 44                 ! 8 4 3 7
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(5)) THEN 
          IF (ivF(3) .EQ. ivE(1)) THEN               
             perm = 45                 ! 8 5 1 4
          ELSE
             perm = 46                 ! 8 5 6 7
          ENDIF
       ELSEIF (ivF(2) .EQ. ivE(7)) THEN
          IF (ivF(3) .EQ. ivE(3)) THEN               
             perm = 47                 ! 8 7 3 4
          ELSE
             perm = 48                 ! 8 7 6 5
          ENDIF
       ENDIF
    ENDIF
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE FIND_QUAD_PERM_NUMBER

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE DOF_PERMUTATIONS
