MODULE MATH_PHYS_CONSTANTS
  
  !========================================================================
  ! Declarations of useful mathematical and physical constants for EM
  !========================================================================

  !------------------------------------------------------------------------
  ! Shared module
  !------------------------------------------------------------------------
  USE ISO_C_BINDING
  !------------------------------------------------------------------------

  !------------------------------------------------------------------------
  ! Shared mathematical constants
  !
  ! Pi              ==> the ratio of a circle's circumference to its diameter
  ! SqrtPi          ==> square root of Pi
  ! SqrtPiOverTwo   ==> square root of (Pi/2)
  ! OverSqrtTwoPi   ==> square root of 1/(2*Pi)
  ! iUnit           ==> the imaginary unit
  !------------------------------------------------------------------------
  REAL(C_DOUBLE),            PARAMETER ::     Pi        = 4.0*ATAN(1.0)
  REAL(C_DOUBLE),            PARAMETER :: SqrtPi        = SQRT(Pi)
  REAL(C_DOUBLE),            PARAMETER :: SqrtPiOverTwo = SQRT(Pi/2.0)
  REAL(C_DOUBLE),            PARAMETER :: OverSqrtTwoPi = 0.39894228
  
  COMPLEX(C_DOUBLE_COMPLEX), PARAMETER :: iUnit = (0.d0, 1.d0)

  !------------------------------------------------------------------------
  ! Shared physical constants
  !
  ! c0     ==> speed of light in vacuum (in m/s)
  ! invC0  ==> inverse of c0
  ! eps0   ==> vaccum permittivity      (in Coulomb/V.mr)
  ! mu0    ==> vacuum permeability      (in V.s/A.m)
  ! z0     ==> impedance of free space  (in Ohm) 
  ! hb     ==> reduced Planck constant  (in eV.s)
  ! coul   ==> elementary charge        (in Coulomb)
  !------------------------------------------------------------------------
  REAL(C_DOUBLE), PARAMETER :: c0    = 2.99792458e+08
  REAL(C_DOUBLE), PARAMETER :: invC0 = 3.33564095e-9
  REAL(C_DOUBLE), PARAMETER :: eps0  = 8.85418781762039e-12
  REAL(C_DOUBLE), PARAMETER :: mu0   = 4.0*Pi*1.0e-07
  REAL(C_DOUBLE), PARAMETER :: z0    = 119.9169832*Pi
  REAL(C_DOUBLE), PARAMETER :: hb    = 6.58211928e-16
  REAL(C_DOUBLE), PARAMETER :: coul  = 1.602176565e-19

  !------------------------------------------------------------------------
  ! Shared constants of other kinds
  !
  ! absoluteEps   ==> reasonable zero-like value for comparison with reals
  !                   WARNING : for length, areas and volumes comparisons, 
  !                   please use mesh-dependant tolerances (see MESH_DATA)
  ! numericalZero ==> a value to compare actual numerical zeros
  !------------------------------------------------------------------------
  REAL(C_DOUBLE), PARAMETER :: absoluteEps   = 1.0e-7
  REAL(C_DOUBLE), PARAMETER :: numericalZero = 1.0e-15
  !------------------------------------------------------------------------

END MODULE MATH_PHYS_CONSTANTS



