MODULE MEDIT_BINARY_MESH

  !========================================================================
  ! Module containing the subroutines related to Medit/Vizir binary mesh format
  !========================================================================

  !------------------------------------------------------------------------
  ! Shared modules
  !------------------------------------------------------------------------
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------

CONTAINS

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE READ_MEDIT_BINARY_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE LIBMESH    
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE MESH_ELEMENTS
    USE DOF_PERMUTATIONS
    USE REFERENCE_CELLS
    USE PARALLEL_STRUCTURES
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    CHARACTER(LEN=1000, KIND=C_CHAR)     :: newMeshname
    CHARACTER(LEN=4,    KIND=C_CHAR)     :: type

    INTEGER(C_INT)                       :: tag, nVert, nCellTetLin, &
         nCellTetQuadr, nCellHexLin, nCellHexQuadr, nFacTriLin,      &
         nFacTriQuadr, nFacQuadLin, nFacQuadQuadr, iVersion,         &
         iDimension, iVert, iCellTetLin, iCellTetQuadr,              &
         iFaceTriLin, iFaceTriQuadr, iCellHexLin, iCellHexQuadr,     &
         iFaceQuadLin, iFaceQuadQuadr, nUnit, strSize
    INTEGER(C_INT), DIMENSION(nvCellMax) :: iBuffer

    REAL(C_DOUBLE), DIMENSION(3)         :: rBuffer
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Open mesh
    !----------------------------------------------------------------------
    newMeshName = TRIM(Mesh%name)//'.meshb'
    strSize     = LEN(TRIM(newMeshName))
    CALL GmfOpenMeshFort(TRIM(newMeshname), strSize, GmfRead, iVersion, &
         iDimension, nUnit)
    !----------------------------------------------------------------------
    !---- Read vertices
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfVertices, nVert)
    CALL GmfGoToKwdFort(nUnit, GmfVertices)
    vertexBlockSize = nVert
    !
    DO iVert=1,nVert
       CALL GmfReadVertexFort(nUnit, GmfVertices, rBuffer(1:3))
       CALL ADD_VERTEX(rBuffer(1:3))
    ENDDO
    !----------------------------------------------------------------------
    !---- Read P1 tetrahedra
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfTetrahedra, nCellTetLin)
    CALL GmfGoToKwdFort(nUnit, GmfTetrahedra)
    type          = 'TET1'
    cellBlockSize = MAX(nCellTetLin, cellBlockSize)
    !
    DO iCellTetLin=1,nCellTetLin
       CALL GmfReadP1TetrahedronFort(nUnit, GmfTetrahedra, &
            iBuffer(1:4), tag)
       CALL ADD_ELEMENT(iBuffer(1:4), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read P2 tetrahedra
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfTetrahedraP2, nCellTetQuadr)
    CALL GmfGoToKwdFort(nUnit, GmfTetrahedraP2)
    type          = 'TET2'
    cellBlockSize = MAX(nCellTetQuadr, cellBlockSize)
    !
    DO iCellTetQuadr=1,nCellTetQuadr
       CALL GmfReadP2TetrahedronFort(nUnit, GmfTetrahedraP2, &
            iBuffer(1:10), tag)
       CALL ADD_ELEMENT(iBuffer(1:10), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read P1 triangles
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfTriangles, nFacTriLin)
    CALL GmfGoToKwdFort(nUnit, GmfTriangles)
    type          = 'TRI1'
    faceBlockSize = MAX(nFacTriLin, faceBlockSize)
    !
    DO iFaceTriLin=1,nFacTriLin
       CALL GmfReadP1TriangleFort(nUnit, GmfTriangles, &
            iBuffer(1:3), tag)
       CALL ADD_ELEMENT(iBuffer(1:3), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read P2 triangles
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfTrianglesP2, nFacTriQuadr)
    CALL GmfGoToKwdFort(nUnit, GmfTrianglesP2)
    type          = 'TRI2'
    faceBlockSize = MAX(nFacTriQuadr, faceBlockSize)
    !
    DO iFaceTriQuadr=1,nFacTriQuadr
       CALL GmfReadP2TriangleFort(nUnit, GmfTrianglesP2, &
            iBuffer(1:6), tag)
       CALL ADD_ELEMENT(iBuffer(1:6), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read Q1 hexahedra
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfHexahedra, nCellHexLin)
    CALL GmfGoToKwdFort(nUnit, GmfHexahedra)
    type          = 'HEX1'
    cellBlockSize = MAX(nCellHexLin, cellBlockSize)
    !
    DO iCellHexLin=1,nCellHexLin
       CALL GmfReadQ1HexahedronFort(nUnit, GmfHexahedra, &
            iBuffer(1:8), tag)
       CALL ADD_ELEMENT(iBuffer(1:8), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read Q2 hexahedra
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfHexahedraQ2, nCellHexQuadr)
    CALL GmfGoToKwdFort(nUnit, GmfHexahedraQ2)
    type          = 'HEX2'
    cellBlockSize = MAX(nCellHexQuadr, cellBlockSize)
    !
    DO iCellHexQuadr=1,nCellHexQuadr
       CALL GmfReadQ2HexahedronFort(nUnit, GmfHexahedraQ2, &
            iBuffer(1:27), tag)
       CALL ADD_ELEMENT(iBuffer(1:27), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read Q1 quadrilaterals
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfQuadrilaterals, nFacQuadLin)
    CALL GmfGoToKwdFort(nUnit, GmfQuadrilaterals)
    type          = 'QUA1'
    faceBlockSize = MAX(nFacQuadLin, faceBlockSize)
    !
    DO iFaceQuadLin=1,nFacQuadLin
       CALL GmfReadQ1QuadrilateralFort(nUnit, GmfQuadrilaterals, &
            iBuffer(1:4), tag)
       CALL ADD_ELEMENT(iBuffer(1:4), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Read Q2 quadrilaterals
    !----------------------------------------------------------------------
    CALL GmfStatKwdFort(nUnit, GmfQuadrilateralsQ2, nFacQuadQuadr)
    CALL GmfGoToKwdFort(nUnit, GmfQuadrilateralsQ2)
    type          = 'QUA2'
    faceBlockSize = MAX(nFacQuadQuadr, faceBlockSize)
    !
    DO iFaceQuadQuadr=1,nFacQuadQuadr
       CALL GmfReadQ2QuadrilateralFort(nUnit, GmfQuadrilateralsQ2, &
            iBuffer(1:9), tag)
       CALL ADD_ELEMENT(iBuffer(1:9), tag, type)
    ENDDO
    !----------------------------------------------------------------------
    !---- Close mesh
    !----------------------------------------------------------------------
    CALL GmfCloseMeshFort(nUnit)
    !----------------------------------------------------------------------
    !---- Build derived elements
    !----------------------------------------------------------------------
    CALL BUILD_DERIVED_FACES
    CALL BUILD_DERIVED_CELLS
    !----------------------------------------------------------------------
    IF (idWrite) WRITE(6,'(a)') 'Finished mesh reading'
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE READ_MEDIT_BINARY_MESH

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE WRITE_MEDIT_BINARY_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE LIBMESH
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE DOF_PERMUTATIONS
    USE MESH_TFSF_FACES
    USE REFERENCE_CELLS
    USE IO_FILES
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=1000)     :: newMeshName

    INTEGER(C_INT)                       :: nUnit, iVert, iFace, iCell, &
         iCellTetLin, iCellTetQuadr, iFaceTriLin, iFaceTriQuadr,        &
         iCellHexLin, iCellHexQuadr, iFaceQuadLin, iFaceQuadQuadr,      &
         iVersion, iDimension, strSize 
    INTEGER(C_INT), DIMENSION(nvTriMax)  :: triPerm
    INTEGER(C_INT), DIMENSION(nvQuadMax) :: quadPerm
    INTEGER(C_INT), DIMENSION(nvTetMax)  :: tetPerm
    INTEGER(C_INT), DIMENSION(nvHexMax)  :: hexPerm   
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Reset cell tags to initial values depending on zone
    !----------------------------------------------------------------------
    CALL PROCESS_TFSF_ZONES_BEFORE_WRITING
    !----------------------------------------------------------------------
    !---- Write header
    !----------------------------------------------------------------------
    newMeshName = TRIM(Mesh%name)//'.meshb'
    iVersion    = 2
    iDimension  = 3
    strSize     = LEN(TRIM(newMeshName))
    CALL GmfOpenMeshFort(TRIM(newMeshName), strSize, GmfWrite, iVersion, &
         iDimension, nUnit)
    !----------------------------------------------------------------------
    !---- Write vertices
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfVertices, Mesh%nVert, 0, 0)
    !
    DO iVert=1,Mesh%nVert
       CALL GmfWriteVertexFort(nUnit, GmfVertices, &
            MeshVert(iVert)%coor(:))
    ENDDO
    !----------------------------------------------------------------------
    !---- Write P1 tetrahedra
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfTetrahedra, Mesh%nCellTetLin, 0, 0)
    !
    DO iCellTetLin=1,Mesh%nCellTetLin
       iCell = MeshCellTetLin(iCellTetLin)%cellNum
       CALL GmfWriteP1TetrahedronFort(nUnit, GmfTetrahedra, &
            MeshCell(iCell)%vertex(:), MeshCell(iCell)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write P2 tetrahedra
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfTetrahedraP2, Mesh%nCellTetQuadr, 0, 0)
    tetPerm = nvTetInriaToMedit
    !
    DO iCellTetQuadr=1,Mesh%nCellTetQuadr
       iCell = MeshCellTetQuadr(iCellTetQuadr)%cellNum
       CALL GmfWriteP2TetrahedronFort(nUnit, GmfTetrahedraP2, &
            MeshCell(iCell)%vertex(tetPerm(:)), MeshCell(iCell)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write P1 triangles
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfTriangles, Mesh%nFacTriLin, 0, 0)
    !
    DO iFaceTriLin=1,Mesh%nFacTriLin
       iFace = MeshFacTriLin(iFaceTriLin)%faceNum
       CALL GmfWriteP1TriangleFort(nUnit, GmfTriangles, &
            MeshFac(iFace)%vertex(:), MeshFac(iFace)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write P2 triangles
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfTrianglesP2, Mesh%nFacTriQuadr, 0, 0)
    triPerm = nsTriInriaToMedit
    !
    DO iFaceTriQuadr=1,Mesh%nFacTriQuadr
       iFace = MeshFacTriQuadr(iFaceTriQuadr)%faceNum
       CALL GmfWriteP2TriangleFort(nUnit, GmfTrianglesP2, &
            MeshFac(iFace)%vertex(triPerm(:)), MeshFac(iFace)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write Q1 hexahedra
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfHexahedra, Mesh%nCellHexLin, 0, 0)
    !
    DO iCellHexLin=1,Mesh%nCellHexLin
       iCell = MeshCellHexLin(iCellHexLin)%cellNum
       CALL GmfWriteQ1HexahedronFort(nUnit, GmfHexahedra, &
            MeshCell(iCell)%vertex(:), MeshCell(iCell)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write Q2 hexahedra
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfHexahedraQ2, Mesh%nCellHexQuadr, 0, 0)
    !
    DO iCellHexQuadr=1,Mesh%nCellHexQuadr
       iCell = MeshCellHexQuadr(iCellHexQuadr)%cellNum
       CALL GmfWriteQ2HexahedronFort(nUnit, GmfHexahedraQ2, &
            MeshCell(iCell)%vertex(:), MeshCell(iCell)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write Q1 quadrilaterals
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfQuadrilaterals, Mesh%nFacQuadLin, 0, 0)
    !
    DO iFaceQuadLin=1,Mesh%nFacQuadLin
       iFace = MeshFacQuadLin(iFaceQuadLin)%faceNum
       CALL GmfWriteQ1QuadrilateralFort(nUnit, GmfQuadrilaterals, &
            MeshFac(iFace)%vertex(1), MeshFac(iFace)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Write Q2 quadrilaterals
    !----------------------------------------------------------------------
    CALL GmfSetKwdFort(nUnit, GmfQuadrilateralsQ2, Mesh%nFacQuadQuadr, 0, 0)
    !
    DO iFaceQuadQuadr=1,Mesh%nFacQuadQuadr
       iFace = MeshFacQuadQuadr(iFaceQuadQuadr)%faceNum
       CALL GmfWriteQ2QuadrilateralFort(nUnit, GmfQuadrilateralsQ2, &
            MeshFac(iFace)%vertex(1), MeshFac(iFace)%log)
    ENDDO
    !----------------------------------------------------------------------
    !---- Close mesh
    !----------------------------------------------------------------------
    CALL GmfCloseMeshFort(nUnit)
    !----------------------------------------------------------------------
    !-- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE WRITE_MEDIT_BINARY_MESH

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE WRITE_MEDIT_BINARY_SOLUTION
    !----------------------------------------------------------------------
    !---- Write the 3D fields in Medit format
    !
    !---- IN
    !
    !---- OUT
    !
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE NUMERICAL_TYPES
    USE MESH_DATA
    USE MESH_VERTICES
    USE IO_FILES
    USE LIBMESH
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    CHARACTER(KIND=C_CHAR, LEN=1000) :: solName

    INTEGER(C_INT)                   :: nUnit, iVersion, iDimension, &
         nLines, nTypes, iElt, iDof, strSize
    INTEGER(C_INT), DIMENSION(1)     :: tabTypes
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Write header
    !----------------------------------------------------------------------
    solName     = TRIM(Mesh%name)//'.solb'
    iVersion    = 2
    iDimension  = 3
    strSize     = LEN(TRIM(solName))
    CALL GmfOpenMeshFort(TRIM(solName), strSize, GmfWrite, iVersion, &
         iDimension, nUnit)
    !----------------------------------------------------------------------
    !---- Write solutions
    !----------------------------------------------------------------------
    nLines      = Mesh%nVert
    nTypes      = 1
    tabTypes(:) = GmfVec
    !
    CALL GmfSetSolKwdFort(nUnit,GmfSolAtVertices,nLines,nTypes,tabTypes(:))
    !
    DO iElt=1,nCellEff
       DO iDof=1,dgEM(iElt)%nDofV
          CALL GmfWriteVectorSolutionFort(nUnit, GmfSolAtVertices, &
               dgEM(iElt)%E%vec(:,iDof))
       ENDDO
    ENDDO
    !----------------------------------------------------------------------
    !---- Close solution file
    !----------------------------------------------------------------------
    CALL GmfCloseMeshFort(nUnit)
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE WRITE_MEDIT_BINARY_SOLUTION

  !========================================================================
  !========================================================================
  !========================================================================

END MODULE MEDIT_BINARY_MESH
