MODULE MEDIT_MESH

  !========================================================================
  ! Module containing the subroutines related to Medit/Vizir mesh format
  !========================================================================

  !------------------------------------------------------------------------
  ! Shared modules
  !------------------------------------------------------------------------
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------

CONTAINS

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE READ_MEDIT_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE IO_FILES
    USE MESH_ELEMENTS
    USE DOF_PERMUTATIONS
    USE REFERENCE_CELLS
    USE PARALLEL_STRUCTURES
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    CHARACTER(LEN=100, KIND=C_CHAR)      :: keyword
    CHARACTER(LEN=4,   KIND=C_CHAR)      :: type

    INTEGER(C_INT)                       :: eltTag, iElt, nElts, nUnit
    INTEGER(C_INT), DIMENSION(nvCellMax) :: iBuffer

    REAL(C_DOUBLE), DIMENSION(3)         :: rBuffer
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Reading and filling Mesh arrays
    !----------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.mesh')
    !
    Readingloop: DO
       READ(nUnit, *) keyword 
       SELECT CASE (trim(keyword))
          !
          !---- Vertices
       CASE('Vertices')
          READ(nUnit,*) nElts
          vertexBlockSize = nElts
          DO iElt=1,nElts
             READ(nUnit,*) rBuffer(1:3), eltTag
             CALL ADD_VERTEX(rBuffer(1:3))
          ENDDO
          !
          !---- Linear tetrahedra
       CASE('Tetrahedra')
          type = 'TET1'
          !
          READ(nUnit,*) nElts
          cellBlockSize = MAX(nElts, cellBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:4), eltTag
             CALL ADD_ELEMENT(iBuffer(1:4), eltTag, type)
          ENDDO
          !
          !---- Quadratic tetrahedra
       CASE('TetrahedraP2')
          type = 'TET2'
          !
          READ(nUnit,*) nElts
          cellBlockSize = MAX(nElts, cellBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*)  iBuffer(1:10), eltTag
             CALL ADD_ELEMENT(iBuffer(1:10), eltTag, type)
          ENDDO
          !
          !---- Linear triangles
       CASE('Triangles')
          type = 'TRI1'
          !
          READ(nUnit,*) nElts
          faceBlockSize = MAX(nElts, faceBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:3), eltTag
             CALL ADD_ELEMENT(iBuffer(1:3), eltTag, type)
          ENDDO
          !
          !---- Quadratic triangles
       CASE('TrianglesP2')
          type = 'TRI2'
          !
          READ(nUnit,*) nElts
          faceBlockSize = MAX(nElts, faceBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:6), eltTag
             CALL ADD_ELEMENT(iBuffer(1:6), eltTag, type)
          ENDDO
          !
          !---- Linear hexahedra
       CASE('Hexahedra')
          type = 'HEX1'
          !
          READ(nUnit,*) nElts
          cellBlockSize = MAX(nElts, cellBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:8), eltTag
             CALL ADD_ELEMENT(iBuffer(1:8), eltTag, type)
          ENDDO
          !
          !---- Quadratic hexahedra
       CASE('HexahedraQ2')
          type = 'HEX2'
          !
          READ(nUnit,*) nElts
          cellBlockSize = MAX(nElts, cellBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:27), eltTag
             CALL ADD_ELEMENT(iBuffer(1:27), eltTag, type)
          ENDDO
          !
          !---- Linear quadrangles
       CASE('Quadrilaterals')
          type = 'QUA1'
          !
          READ(nUnit,*) nElts
          faceBlockSize = MAX(nElts, faceBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:4), eltTag
             CALL ADD_ELEMENT(iBuffer(1:4), eltTag, type)
          ENDDO
          !
          !---- Quadratic quadrangles
       CASE('QuadrilateralsQ2')
          type = 'QUA2'
          !
          READ(nUnit,*) nElts
          faceBlockSize = MAX(nElts, faceBlockSize)
          DO iElt=1,nElts
             READ(nUnit,*) iBuffer(1:9), eltTag
             CALL ADD_ELEMENT(iBuffer(1:9), eltTag, type)
          ENDDO
          !
          !---- Stop reading
       CASE('End')
          EXIT Readingloop
       CASE DEFAULT
          !
       END SELECT
    ENDDO Readingloop
    !
    CLOSE(nUnit)
    !----------------------------------------------------------------------
    IF (idWrite) WRITE(6,'(a)') 'Finished mesh reading'
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE READ_MEDIT_MESH

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE WRITE_MEDIT_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE IO_FILES
    USE DOF_PERMUTATIONS
    USE REFERENCE_CELLS
    USE MESH_TFSF_FACES
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    INTEGER(C_INT)                       :: nUnit, iElt, iCell, iFace
    INTEGER(C_INT), DIMENSION(nvTriMax)  :: triPerm
    INTEGER(C_INT), DIMENSION(nvQuadMax) :: quadPerm
    INTEGER(C_INT), DIMENSION(nvTetMax)  :: tetPerm
    INTEGER(C_INT), DIMENSION(nvHexMax)  :: hexPerm
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Reset cell tags to initial values depending on zone
    !----------------------------------------------------------------------
    CALL PROCESS_TFSF_ZONES_BEFORE_WRITING
    !----------------------------------------------------------------------
    !---- Write mesh
    !----------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.mesh')
    !
    WRITE(nUnit, '(a30)') 'MeshVersionFormatted 1'
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'Dimension 3'
    !
    !---- Vertices
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'Vertices'
    WRITE(nUnit, '(i9)' ) Mesh%nVert
    !
    DO iElt=1,Mesh%nVert
       WRITE(nUnit, '(3e15.7,1x,i0)') MeshVert(iElt)%coor(:), 0
    ENDDO
    !
    !---- Linear tetrahedra
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'Tetrahedra'
    WRITE(nUnit, '(i9)' ) Mesh%nCellTetLin
    !
    DO iElt=1,Mesh%nCellTetLin
       iCell = MeshCellTetLin(iElt)%cellNum
       WRITE(nUnit, '(5I9)') MeshCell(iCell)%vertex(:), MeshCell(iCell)%log
    ENDDO
    !
    !---- Quadratic tetrahedra
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'TetrahedraP2'
    WRITE(nUnit, '(i9)' ) Mesh%nCellTetQuadr
    tetPerm = nvTetInriaToMedit
    !
    DO iElt=1,Mesh%nCellTetQuadr
       iCell = MeshCellTetQuadr(iElt)%cellNum
       WRITE(nUnit, '(11i9)') MeshCell(iCell)%vertex(tetPerm(:)), MeshCell(iCell)%log
    ENDDO
    !
    !---- Linear triangles
    WRITE(nUnit, '(a1)')  ' '
    WRITE(nUnit, '(a30)') 'Triangles'
    WRITE(nUnit, '(i9)')  Mesh%nFacTriLin
    !
    DO iElt=1,Mesh%nFacTriLin
       iFace = MeshFacTriLin(iElt)%faceNum
       WRITE(nUnit, '(4i9)') MeshFac(iFace)%vertex(:), MeshFac(iFace)%log
    ENDDO
    !
    !---- Quadratic triangles
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'TrianglesP2'
    WRITE(nUnit, '(i9)' ) Mesh%nFacTriQuadr
    triPerm = nsTriInriaToMedit
    !
    DO iElt=1,Mesh%nFacTriQuadr
       iFace = MeshFacTriQuadr(iElt)%faceNum
       WRITE(nUnit, '(7i9)') MeshFac(iFace)%vertex(triPerm(:)), MeshFac(iFace)%log
    ENDDO
    !
    !---- Linear hexahedra
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'Hexahedra'
    WRITE(nUnit, '(i9)' ) Mesh%nCellHexLin
    !
    DO iElt=1,Mesh%nCellHexLin
       iCell = MeshCellHexLin(iElt)%cellNum
       WRITE(nUnit, '(9i9)') MeshCell(iCell)%vertex(:), MeshCell(iCell)%log
    ENDDO
    !
    !---- Quadratic hexahedra
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'HexahedraQ2'
    WRITE(nUnit, '(i9)' ) Mesh%nCellHexQuadr
    !
    DO iElt=1,Mesh%nCellHexQuadr
       iCell = MeshCellHexQuadr(iElt)%cellNum
       WRITE(nUnit, '(28i9)') MeshCell(iCell)%vertex(:), MeshCell(iCell)%log
    ENDDO
    !
    !---- Linear quadrangles
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'Quadrilaterals'
    WRITE(nUnit, '(i9)' ) Mesh%nFacQuadLin
    !
    DO iElt=1,Mesh%nFacQuadLin
       iFace = MeshFacQuadLin(iElt)%faceNum
       WRITE(nUnit, '(5i9)') MeshFac(iFace)%vertex(:), MeshFac(iFace)%log
    ENDDO
    !
    !---- Quadratic quadrangles
    WRITE(nUnit, '(a1)' ) ' '
    WRITE(nUnit, '(a30)') 'QuadrilateralsQ2'
    WRITE(nUnit, '(i9)' ) Mesh%nFacQuadQuadr
    !
    DO iElt=1,Mesh%nFacQuadQuadr
       iFace = MeshFacQuadQuadr(iElt)%faceNum
       WRITE(nUnit, '(10i9)') MeshFac(iFace)%vertex(:), MeshFac(iFace)%log
    ENDDO
    !
    !---- End
    WRITE(nUnit, '(a3)') 'End'
    !
    CLOSE(nUnit)
    !----------------------------------------------------------------------
    !-- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE WRITE_MEDIT_MESH

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE READ_MEDIT_SOLUTION
    !------------------------------------------------------------------
    !---- Read the 3D fields in Medit format
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE IO_FILES
    USE MESH_DATA
    USE DOF_PERMUTATIONS
    USE NUMERICAL_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: nVerts, iElt, iDof, nUnit
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Allocate solution fields
    !------------------------------------------------------------------
    CALL ALLOCATE_NUMERICAL_ELEMENTS_FROM_MESH_TYPES
    CALL ALLOCATE_NUMERICAL_FACES_FROM_MESH_TYPES
    !------------------------------------------------------------------
    !---- Read solution
    !------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%solutionName)//'.sol')
    !
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *)
    READ(nUnit, *) nVerts
    !
    IF (nVerts /= Mesh%nVert) THEN
       WRITE(6, '(a)') 'Error in READ_MEDIT_SOLUTION'
       WRITE(6, '(a)') 'The # of vertices read in ',TRIM(Mesh%solutionName)//'.sol &
            is different from that read in the corresponding mesh file'
       CALL TILT
    ENDIF
    !
    READ(nUnit, *) ! Format is read here, to be improved with different reading subroutines
    !
    !---- Cells
    DO iElt=1,Mesh%nCell
       DO iDof=1,dgEM(iElt)%nDofV
          READ(nUnit, *) dgEM(iElt)%E%vec(:,iDof)
          READ(nUnit, *) dgEM(iElt)%H%vec(:,iDof)
       ENDDO
    ENDDO
    !
    !---- Faces
    DO iElt=1,Mesh%nFac
       DO iDof=1,EMf(iElt)%nDofS
          READ(nUnit, *) EMf(iElt)%E%vec(:,iDof)
          READ(nUnit, *) EMf(iElt)%H%vec(:,iDof)
       ENDDO
    ENDDO
    !
    CLOSE(nUnit)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE READ_MEDIT_SOLUTION

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_MEDIT_SOLUTION
    !------------------------------------------------------------------
    !---- Write the 3D fields in Medit format
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE IO_FILES
    USE MESH_DATA
    USE DOF_PERMUTATIONS
    USE NUMERICAL_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)                 :: iDof, iFac, iElt, nUnit
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Write solution
    !------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%solutionName)//'.sol')
    !
    WRITE(nUnit, '(a)' ) 'MeshVersionFormatted'
    WRITE(nUnit, '(a)' ) '1'
    WRITE(nUnit, '(a)' ) ' '
    WRITE(nUnit, '(a)' ) 'Dimension'
    WRITE(nUnit, '(a)' ) '3'
    WRITE(nUnit, '(a)' ) ' '
    WRITE(nUnit, '(a)' ) 'SolAtVertices'
    WRITE(nUnit, '(i7)') Mesh%nVert
    WRITE(nUnit, '(a)' ) '2 2 2'
    !
    !---- Cells
    DO iElt=1,Mesh%nCell
       DO iDof=1,dgEM(iElt)%nDofV
          WRITE(nUnit,'(3e15.7)') dgEM(iElt)%E%vec(:,iDof)
          WRITE(nUnit,'(3e15.7)') dgEM(iElt)%H%vec(:,iDof)
       ENDDO
    ENDDO
    !
    !---- Faces
    DO iFac=1,Mesh%nFacI
       iElt = MeshFacI(iFac)%faceNum
       DO iDof=1,EMf(iElt)%nDofS
          WRITE(nUnit,'(3e15.7)') EMf(iElt)%E%vec(:,iDof)
          WRITE(nUnit,'(3e15.7)') EMf(iElt)%H%vec(:,iDof)
       ENDDO
    ENDDO
    !
    WRITE(nUnit, '(a)') 'End'
    !
    CLOSE(nUnit)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_MEDIT_SOLUTION

  !========================================================================
  !========================================================================
  !========================================================================

END MODULE MEDIT_MESH
