MODULE MESH_FACES_LOGS

  !====================================================================
  ! Everything related to faces logs
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE COMPUTE_LOG_SUM(log1, log2, newLog)
    !------------------------------------------------------------------
    !---- Compute sum of two logs
    !
    !---- IN
    !---- log*   : input logs to sum
    !
    !---- OUT
    !---- newLog : summed log    
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT), INTENT(IN)  :: log1, log2
    INTEGER(C_INT), INTENT(OUT) :: newLog
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)              :: log1_1, log1_10, log1_100, log1_1000, &
         log1_10000, log2_1, log2_10, log2_100, log2_1000, log2_10000,   &
         newLog_1, newLog_10, newLog_100, newLog_1000, newLog_10000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Compute log difference
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log1, 1, log1_1    )
    CALL DECOMPOSE_INTEGER(log1, 2, log1_10   )
    CALL DECOMPOSE_INTEGER(log1, 3, log1_100  )
    CALL DECOMPOSE_INTEGER(log1, 4, log1_1000 )
    CALL DECOMPOSE_INTEGER(log1, 5, log1_10000)
    !
    CALL DECOMPOSE_INTEGER(log2, 1, log2_1    )
    CALL DECOMPOSE_INTEGER(log2, 2, log2_10   )
    CALL DECOMPOSE_INTEGER(log2, 3, log2_100  )
    CALL DECOMPOSE_INTEGER(log2, 4, log2_1000 )
    CALL DECOMPOSE_INTEGER(log2, 5, log2_10000)
    !
    newLog_1     = MAX(log1_1,    log2_1      )
    newLog_10    = MAX(log1_10,   log2_10     )
    newLog_100   = MAX(log1_100,  log2_100    )
    newLog_1000  = MAX(log1_1000, log2_1000   )
    newLog_10000 = MAX(log1_10000,log2_10000  )
    !
    newLog =  10000*newLog_10000 &
         +     1000*newLog_1000  &
         +      100*newLog_100   &
         +       10*newLog_10    &
         +          newLog_1
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE COMPUTE_LOG_SUM

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_LOG_INCLUDED(log, logRef, isIncluded)
    !------------------------------------------------------------------
    !---- Tell if log is included in logRef
    !
    !---- IN
    !---- log        : log to test
    !---- logRef     : reference log
    !
    !---- OUT
    !---- isIncluded : TRUE if log is included in logRef
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAl(C_BOOL), INTENT(OUT) :: isIncluded
    
    INTEGER(C_INT),  INTENT(IN)  :: log, logRef
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: log_1, log_10, log_100, log_1000, &
         log_10000, logRef_1, logRef_10, logRef_100, logRef_1000,     &
         logRef_10000, diff_1, diff_10, diff_100, diff_1000, diff_10000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Compute log difference
    !------------------------------------------------------------------
    isIncluded = .TRUE.
    !
    CALL DECOMPOSE_INTEGER(log,    1, log_1       )
    CALL DECOMPOSE_INTEGER(log,    2, log_10      )
    CALL DECOMPOSE_INTEGER(log,    3, log_100     )
    CALL DECOMPOSE_INTEGER(log,    4, log_1000    )
    CALL DECOMPOSE_INTEGER(log,    5, log_10000   )
    !
    CALL DECOMPOSE_INTEGER(logRef, 1, logRef_1    )
    CALL DECOMPOSE_INTEGER(logRef, 2, logRef_10   )
    CALL DECOMPOSE_INTEGER(logRef, 3, logRef_100  )
    CALL DECOMPOSE_INTEGER(logRef, 4, logRef_1000 )
    CALL DECOMPOSE_INTEGER(logRef, 5, logRef_10000)
    !
    diff_1     = logRef_1     - log_1
    diff_10    = logRef_10    - log_10
    diff_100   = logRef_100   - log_100
    diff_1000  = logRef_1000  - log_1000
    diff_10000 = logRef_10000 - log_10000
    !
    IF (diff_1     < 0) isIncluded = .FALSE.
    IF (diff_10    < 0) isIncluded = .FALSE.
    IF (diff_100   < 0) isIncluded = .FALSE.
    IF (diff_1000  < 0) isIncluded = .FALSE.
    IF (diff_10000 < 0) isIncluded = .FALSE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_LOG_INCLUDED

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_CONFORMAL_BOUNDARY_LOG(log, isConformalBoundaryLog)
    !------------------------------------------------------------------
    !---- Says if log is a conformal boundary log
    !
    !---- IN
    !---- log           : face log
    !
    !---- OUT
    !---- isConformalBoundaryLog : TRUE if log is a conformal boundary log
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAL(C_BOOL), INTENT(OUT) :: isConformalBoundaryLog

    INTEGER(C_INT),  INTENT(IN)  :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: tag1, tag10, tag1000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 1, tag1   )
    CALL DECOMPOSE_INTEGER(log, 2, tag10  )
    CALL DECOMPOSE_INTEGER(log, 4, tag1000)
    isConformalBoundaryLog                   = .FALSE.
    !
    IF (tag1    == 2) isConformalBoundaryLog = .TRUE.
    IF (tag1    == 4) isConformalBoundaryLog = .TRUE.
    IF (tag1    == 6) isConformalBoundaryLog = .TRUE.
    IF (tag1    == 8) isConformalBoundaryLog = .TRUE.
    IF (tag10   == 1) isConformalBoundaryLog = .TRUE.
    IF (tag1000 == 1) isConformalBoundaryLog = .FALSE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_CONFORMAL_BOUNDARY_LOG

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_INTERFACE_LOG(log, isInterfaceLog)
    !------------------------------------------------------------------
    !---- Says if log is an interface log
    !
    !---- IN
    !---- log            : face log
    !
    !---- OUT
    !---- isInterfaceLog : TRUE if log is an interface log
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAL(C_BOOL), INTENT(OUT) :: isInterfaceLog

    INTEGER(C_INT),  INTENT(IN)  :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: tag10000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 5, tag10000)
    isInterfaceLog                    = .FALSE.
    IF (tag10000 /= 0) isInterfaceLog = .TRUE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_INTERFACE_LOG

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_CONFORMAL_INNER_LOG(log, isConformalInnerLog)
    !------------------------------------------------------------------
    !---- Says if log is a conformal inner log
    !
    !---- IN
    !---- log                 : face log
    !
    !---- OUT
    !---- isConformalInnerLog : TRUE if log is a conformal inner log
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAL(C_BOOL), INTENT(OUT) :: isConformalInnerLog

    INTEGER(C_INT),  INTENT(IN)  :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: tag1, tag10, tag1000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 1, tag1   )
    CALL DECOMPOSE_INTEGER(log, 2, tag10  )
    CALL DECOMPOSE_INTEGER(log, 4, tag1000)
    isConformalInnerLog                                                          = .FALSE.
    IF ((tag1 == 1) .AND. (tag10 == 0) .AND. (tag1000 == 0)) isConformalInnerLog = .TRUE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_CONFORMAL_INNER_LOG

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_HYBRID_INNER_LOG(log, isHybridInnerLog)
    !------------------------------------------------------------------
    !---- Says if log is a hybrid inner log
    !
    !---- IN
    !---- log                 : face log
    !
    !---- OUT
    !---- isHybridInnerLog : TRUE if log is a hybrid inner log
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAL(C_BOOL), INTENT(OUT) :: isHybridInnerLog

    INTEGER(C_INT),  INTENT(IN)  :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: tag1, tag10, tag1000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 1, tag1   )
    CALL DECOMPOSE_INTEGER(log, 2, tag10  )
    CALL DECOMPOSE_INTEGER(log, 4, tag1000)
    isHybridInnerLog = .FALSE.
    IF ((tag1 == 1) .AND. (tag10 == 0) .AND. (tag1000 == 1)) &
         isHybridInnerLog = .TRUE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_HYBRID_INNER_LOG

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE IS_HYBRID_ARTIFICIAL_LOG(log, isHybridArtificialLog)
    !------------------------------------------------------------------
    !---- Says if log is a hybrid inner log
    !
    !---- IN
    !---- log                 : face log
    !
    !---- OUT
    !---- isHybridArtificialLog : TRUE if log is a hybrid inner log
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    LOGICAL(C_BOOL), INTENT(OUT) :: isHybridArtificialLog

    INTEGER(C_INT),  INTENT(IN)  :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)               :: tag1, tag10, tag1000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 1, tag1   )
    CALL DECOMPOSE_INTEGER(log, 2, tag10  )
    CALL DECOMPOSE_INTEGER(log, 4, tag1000)
    isHybridArtificialLog = .FALSE.
    IF ((tag1 == 0) .AND. (tag10 == 1) .AND. (tag1000 == 1)) &
         isHybridArtificialLog = .TRUE.
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE IS_HYBRID_ARTIFICIAL_LOG

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE REMOVE_ARTIFICIAL_LOG(log)
    !------------------------------------------------------------------
    !---- Remove the artificial attribute to a face log
    !
    !---- IN
    !---- log : face log
    !
    !---- OUT
    !---- log : modified in the process
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE SPECIAL_FUNCTIONS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT), INTENT(INOUT) :: log
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)                :: tag1, tag10, tag100, tag1000, tag10000
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Sort log
    !------------------------------------------------------------------
    CALL DECOMPOSE_INTEGER(log, 1, tag1    )
    CALL DECOMPOSE_INTEGER(log, 2, tag10   )
    CALL DECOMPOSE_INTEGER(log, 3, tag100  )
    CALL DECOMPOSE_INTEGER(log, 4, tag1000 )
    CALL DECOMPOSE_INTEGER(log, 5, tag10000)
    !
    IF (tag10 == 1) THEN
       tag10 = 0
       !
       log  =  10000*tag10000 &
            +   1000*tag1000  &
            +    100*tag100   &
            +     10*tag10    &
            +        tag1
    ENDIF
    !------------------------------------------------------------------
    !---- Testing for errors 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE REMOVE_ARTIFICIAL_LOG
  
  !====================================================================
  !====================================================================
  !====================================================================

END MODULE MESH_FACES_LOGS
