MODULE MESH_HO_CELLS

  !====================================================================
  ! Module containing the subroutines related to HO cells
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

!   SUBROUTINE COMPUTE_MESH_BOUNDING_BOX
!     !------------------------------------------------------------------
!     !---- Compute the bounding box of Mesh structure
!     !
!     !---- IN
!     !
!     !---- OUT
!     !
!     !------------------------------------------------------------------
!     !---- Shared modules
!     !------------------------------------------------------------------
!     USE MESH_DATA
!     USE MESH_VERTICES
!     USE PARALLEL_STRUCTURES
!     !------------------------------------------------------------------
!     !---- I/O variables
!     !------------------------------------------------------------------
!     !
!     !------------------------------------------------------------------
!     !---- Local variables
!     !------------------------------------------------------------------
!     INTEGER(C_INT)               :: iVert, iCoor

!     REAL(C_DOUBLE)               :: localValue
!     !------------------------------------------------------------------
!     !---- Testing for bad input
!     !------------------------------------------------------------------
!     !
!     !------------------------------------------------------------------
!     !---- Compute bounding box and mesh center
!     !------------------------------------------------------------------
!     Mesh%coorMin(:)    = 1e9
!     Mesh%coorMax(:)    =-1e9
!     Mesh%coorCenter(:) = 0.0
!     !
!     DO iVert=1,Mesh%nVert
!        DO iCoor=1,3
!           localValue             = MeshVert(iVert)%coor(iCoor)
!           Mesh%coorMin(iCoor)    = MIN(Mesh%coorMin(iCoor), localValue) 
!           Mesh%coorMax(iCoor)    = MAX(Mesh%coorMax(iCoor), localValue)
!           !
!           Mesh%coorCenter(iCoor) = Mesh%coorCenter(iCoor) + MeshVert(iVert)%coor(iCoor)
!        ENDDO
!     ENDDO
!     !
!     CALL GLOBAL_OP(Mesh%coorCenter(:), MPI_SUM, globalMesh%coorCenter(:))
!     CALL GLOBAL_OP(Mesh%nVert,         MPI_SUM, globalMesh%nVert)
!     CALL GLOBAL_OP(Mesh%coorMin,       MPI_MIN, globalMesh%coorMin)
!     CALL GLOBAL_OP(Mesh%coorMax,       MPI_MAX, globalMesh%coorMax)    
!     !
!     Mesh%coorCenter(:)    = globalMesh%coorCenter(:)/globalMesh%nVert
!     Mesh%boxSize(:)       = ABS(globalMesh%coorMax(:)-globalMesh%coorMin(:))
!     globalMesh%boxSize(:) = Mesh%boxSize(:)
!     !------------------------------------------------------------------
!     !---- Testing for errors
!     !------------------------------------------------------------------
!     IF (idWrite) WRITE(6,'(a)') 'Finished computing mesh bounding box'
!     !------------------------------------------------------------------
!   END SUBROUTINE COMPUTE_MESH_BOUNDING_BOX

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE MESH_HO_CELLS
