MODULE NACHOS_MESH

  !======================================================================
  ! Module containing the subroutines related to Nachos mesh format (deprecated)
  !======================================================================

  !----------------------------------------------------------------------
  ! Shared modules
  !----------------------------------------------------------------------
  USE ISO_C_BINDING
  !----------------------------------------------------------------------
  IMPLICIT NONE
  !----------------------------------------------------------------------

CONTAINS

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE READ_NACHOS_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE IO_FILES
    USE MESH_ELEMENTS
    USE DOF_PERMUTATIONS
    USE REFERENCE_CELLS
    USE PARALLEL_STRUCTURES
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    CHARACTER(LEN=4, KIND=C_CHAR)        :: type

    REAL(C_DOUBLE), DIMENSION(3)         :: rBuffer

    INTEGER(C_INT)                       :: eltTag, iElt, nVert, nTet, nTri, nUnit
    INTEGER(C_INT), DIMENSION(nvCellMax) :: iBuffer
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Reading and filling Mesh arrays
    !----------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.nmesh')
    READ(nUnit, *) nVert, nTet, nTri
    vertexBlockSize = nVert
    cellBlockSize   = nTet
    faceBlockSize   = nTri
    !
    !---- Vertices
    DO iElt=1,nVert
       READ(nUnit,*) rBuffer(1:3)
       CALL ADD_VERTEX(rBuffer(1:3))
    ENDDO
    !
    !---- Tetrahedra
    DO iElt=1,nTet
       READ(nUnit,*) iBuffer(1:4), eltTag
       type = "TET1"
       CALL ADD_ELEMENT(iBuffer(1:4), eltTag, type)
    ENDDO
    !
    !---- Triangles
    DO iElt=1,nTri
       READ(nUnit,*) eltTag, iBuffer(1:3)
       type = "TRI1"
       CALL ADD_ELEMENT(iBuffer(1:3), eltTag, type)
    ENDDO
    !
    CLOSE(nUnit)
    !----------------------------------------------------------------------
    IF (idWrite) WRITE(6,'(a)') 'Finished mesh reading'
    !----------------------------------------------------------------------
    !---- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE READ_NACHOS_MESH

  !========================================================================
  !========================================================================
  !========================================================================

  SUBROUTINE WRITE_NACHOS_MESH
    !----------------------------------------------------------------------
    !---- Modules
    !----------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_VERTICES
    USE MESH_FACES
    USE MESH_CELLS
    USE MESH_STATS
    USE MESH_TFSF_FACES
    USE IO_FILES
    !----------------------------------------------------------------------
    IMPLICIT NONE
    !----------------------------------------------------------------------
    !---- I/O variables
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Local variables
    !----------------------------------------------------------------------
    INTEGER(C_INT) iVert, iTri, iTet, locVert, nUnit, iCell, iFace
    !----------------------------------------------------------------------
    !---- Testing for bad input
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
    !---- Reset cell tags to initial values depending on zone
    !----------------------------------------------------------------------
    CALL PROCESS_TFSF_ZONES_BEFORE_WRITING
    !----------------------------------------------------------------------
    !---- Write regular file
    !----------------------------------------------------------------------
    CALL GET_NEW_FILE_UNIT(nUnit)
    OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.nmesh')
    !
    WRITE(nUnit, '(3i9)') Mesh%nVert, Mesh%nCellTet, Mesh%nFacB
    !
    !---- Vertices
    DO iVert=1,Mesh%nVert
       WRITE(nUnit, '(3e15.7)') MeshVert(iVert)%coor(:)
    ENDDO
    !
    !---- All tetrahedra
    DO iTet=1,Mesh%nCellTet
       iCell = MeshCellTet(iTet)%cellNum
       WRITE(nUnit, '(5i9)') MeshCell(iCell)%vertex(1:4), MeshCell(iCell)%log
    ENDDO
    !
    !---- All boundary triangles
    DO iTri=1,Mesh%nFacB
       iFace = MeshFacB(iTri)%faceNum
       WRITE(nUnit, '(4i9)') MeshFac(iFace)%log, MeshFac(iFace)%vertex(1:3)
    ENDDO
    !
    CLOSE(nUnit)
    !----------------------------------------------------------------------
    !---- Writing curvilinear dofs
    !----------------------------------------------------------------------
    ! OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.nmesh_curvddl')
    ! !
    ! DO iTet=1,Mesh%nCellTetCurv
    !    DO iVert=1,6
    !       locVert = MeshCellTetCurv(iTet)%curvPtr%hoVerts(iVert)
    !       WRITE(nUnit, '(6e15.7)') MeshVert(locVert)%coor(:)
    !    ENDDO
    ! ENDDO
    ! !
    ! CLOSE(nUnit)
    !----------------------------------------------------------------------
    !---- Evaluating TF/SF bounding box and Writing TF/SF faces
    !----------------------------------------------------------------------
    ! CALL TFSF_BOUNDING_BOX
    ! !
    ! OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.nmesh_tfsf')
    ! !
    ! WRITE(nUnit, '(i7)')     Mesh%nFacTfsf
    ! WRITE(nUnit, '(2e15.7)') Mesh%tfsfMin(1), Mesh%tfsfMax(1)
    ! WRITE(nUnit, '(2e15.7)') Mesh%tfsfMin(2), Mesh%tfsfMax(2)
    ! WRITE(nUnit, '(2e15.7)') Mesh%tfsfMin(3), Mesh%tfsfMax(3)
    ! !
    ! DO iTri=1,Mesh%nFacTfsf
    !    WRITE(nUnit, '(3i7)') MeshFacTfsf(iTri)%ptr%vertex(:)
    ! ENDDO
    ! !
    ! CLOSE(nUnit)
    !----------------------------------------------------------------------
    !---- Writing Poynting faces
    !----------------------------------------------------------------------
    ! OPEN(UNIT=nUnit, FILE=TRIM(Mesh%name)//'.nmesh_poy')
    ! !
    ! WRITE(nUnit,    '(i7)')  Mesh%nFacPoy
    ! !
    ! DO iTri=1,Mesh%nFacPoy
    !    WRITE(nUnit, '(3i7)') MeshFacPoy(iTri)%ptr%vertex(:)
    ! ENDDO
    ! !
    ! CLOSE(nUnit)
    !----------------------------------------------------------------------
    !-- Testing for errors
    !----------------------------------------------------------------------
    !
    !----------------------------------------------------------------------
  END SUBROUTINE WRITE_NACHOS_MESH

  !========================================================================
  !========================================================================
  !========================================================================

END MODULE NACHOS_MESH
