PROGRAM BUILD_INTERFACESTEST
  !------------------------------------------------------------------------
  !---- Test program for interface building
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE MESH_APPS
  USE MESH_DATA
  USE MESH_FACES
  USE MESH_CELLS
  USE MESH_ELEMENTS
  USE MESH_STRUCTURES
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR)              :: readname
  CHARACTER(LEN=1000, KIND=C_CHAR)              :: testName = 'BUILD_INTERFACES'
  CHARACTER(LEN=1000, KIND=C_CHAR)              :: repo     = '../../../tst/build_interfaces/'  

  LOGICAL(C_BOOL)                               :: isIdentical

  INTEGER(C_INT)                                :: testNumber, iFacInter, &
       iFac, iFac1, iFac2

  TYPE(mesh_structure)                          :: localMesh

  TYPE(face),         DIMENSION(:), ALLOCATABLE :: localFace

  TYPE(derived_face), DIMENSION(:), ALLOCATABLE :: localFaceInter
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Test #1 : all tetrahedra
  !------------------------------------------------------------------------
  testNumber  = 1
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  !
  Mesh%name   = 'tst_tet.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL BUILD_INTERFACES_SUB(Mesh%name)
  localMesh   = Mesh
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  ALLOCATE(localFace(SIZE(MeshFac)))
  localFace   = MeshFac
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name   = 'ref_tet.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  CALL BUILD_DERIVED_FACES
  !
  isIdentical = .TRUE.
  IF (Mesh%nFacInter /= localMesh%nFacInter)       isIdentical = .FALSE.
  DO iFacInter=1,Mesh%nFacInter
     iFac = MeshFacInter(iFacInter)%faceNum
     IF (MeshFac(iFac)%log /= localFace(iFac)%log) isIdentical = .FALSE.
  ENDDO
  !
  IF (.NOT. isIdentical) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #2 : simple hybrid case
  !------------------------------------------------------------------------
  testNumber     = 2
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  !
  Mesh%name      = 'tst_hyb.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL BUILD_INTERFACES_SUB(Mesh%name)
  localMesh      = Mesh
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  ALLOCATE(localFace(SIZE(MeshFac)))
  localFace      = MeshFac
  IF (ALLOCATED(localFaceInter)) DEALLOCATE(localFaceInter)
  ALLOCATE(localFaceInter(SIZE(MeshFacInter)))
  localFaceInter = MeshFacInter
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name      = 'ref_hyb.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  CALL BUILD_DERIVED_FACES  
  !
  isIdentical    = .TRUE.
  IF (Mesh%nFacInter /= localMesh%nFacInter)       isIdentical = .FALSE.
  DO iFacInter=1,Mesh%nFacInter
     iFac1 = MeshFacInter(iFacInter)%faceNum
     iFac2 = localFaceInter(iFacInter)%faceNum
     IF (MeshFac(iFac1)%log /= localFace(iFac2)%log) isIdentical = .FALSE.
  ENDDO
  !
  IF (.NOT. isIdentical) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Clean files
  !------------------------------------------------------------------------
  CALL SYSTEM('                    &
       rm tst_tet_interfaces.mesh; &
       rm tst_hyb_interfaces.mesh;')
  !------------------------------------------------------------------------
END PROGRAM BUILD_INTERFACESTEST
