PROGRAM CELLS_ROUTINESTEST
  !------------------------------------------------------------------------
  !---- Test program for various cell-related routines
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE MESH_CELLS
  USE MESH_VERTICES
  USE REFERENCE_CELLS
  USE MESH_ELEMENTS
  USE MESH_FIND_POINT_IN_ELEMENTS
  USE MESH_ELEMENTS_CENTERS
  USE MESH_NORMALS_AND_MEASURES
  USE IO_TESTS
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'CELLS_ROUTINES'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/cells_routines/'  

  INTEGER(C_INT)                   :: iCell, testNumber
  
  LOGICAL(C_BOOL)                  :: isInCell
  
  REAL(C_DOUBLE), DIMENSION(3)     :: pCenter, refCenter, ptIn, ptOut
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)    
  !------------------------------------------------------------------------
  !---- Allocate structure for test
  !------------------------------------------------------------------------
  ALLOCATE(MeshCell(1))
  ALLOCATE(MeshCell(1)%vertex(4))
  ALLOCATE(MeshVert(4))
  !
  iCell                 = 1
  MeshCell(1)%cellType  = "TET1"
  MeshCell(1)%nLinVerts = 4
  MeshCell(1)%vertex(1) = 1
  MeshCell(1)%vertex(2) = 2
  MeshCell(1)%vertex(3) = 3
  MeshCell(1)%vertex(4) = 4
  MeshVert(1)%coor      = (/ 0.0, 0.0, 0.0 /)
  MeshVert(2)%coor      = (/ 1.0, 0.0, 0.0 /)
  MeshVert(3)%coor      = (/ 0.0, 1.0, 0.0 /)
  MeshVert(4)%coor      = (/ 0.0, 0.0, 1.0 /)
  refCenter(:)          = 0.25
  ptIn                  = (/ 0.1, 0.1, 0.1 /)
  ptOut                 = (/ 2.0, 2.0, 2.0 /)
  !------------------------------------------------------------------------
  !---- Test #1 : measure of reference tetrahedron
  !------------------------------------------------------------------------
  testNumber = 1
  CALL COMPUTE_CELL_MEASURE(iCell)
  !
  IF (MeshCell(1)%measure /= Tref_vol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #2 : center of reference tetrahedron
  !------------------------------------------------------------------------
  testNumber = 2
  CALL COMPUTE_CENTER_OF_CELL(iCell, pCenter)
  !
  IF (.NOT. (ALL(pCenter == refCenter))) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #3 : test if point is in reference tetrahedron
  !------------------------------------------------------------------------
  testNumber = 3
  CALL IS_POINT_IN_CELL(ptIn, iCell, isInCell)
  !
  IF (.NOT. (isInCell)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #4 : test if point is in reference tetrahedron
  !------------------------------------------------------------------------
  testNumber = 4
  CALL IS_POINT_IN_CELL(ptOut, iCell, isInCell)
  !
  IF (isInCell) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Allocate structure for test
  !------------------------------------------------------------------------
  DEALLOCATE(MeshCell, MeshVert)
  ALLOCATE  (MeshCell(1))
  ALLOCATE  (MeshCell(1)%vertex(8))
  ALLOCATE  (MeshVert(8))
  !
  iCell                 = 1
  MeshCell(1)%cellType  = "HEX1"
  MeshCell(1)%nLinVerts = 8
  MeshCell(1)%vertex(1) = 1
  MeshCell(1)%vertex(2) = 2
  MeshCell(1)%vertex(3) = 3
  MeshCell(1)%vertex(4) = 4
  MeshCell(1)%vertex(5) = 5
  MeshCell(1)%vertex(6) = 6
  MeshCell(1)%vertex(7) = 7
  MeshCell(1)%vertex(8) = 8  
  MeshVert(1)%coor      = (/ -1.0, -1.0, -1.0 /)
  MeshVert(2)%coor      = (/ 1.0, -1.0, -1.0 /)
  MeshVert(3)%coor      = (/ 1.0, 1.0, -1.0 /)
  MeshVert(4)%coor      = (/ -1.0, 1.0, -1.0 /)
  MeshVert(5)%coor      = (/ -1.0, -1.0, 1.0 /)
  MeshVert(6)%coor      = (/ 1.0, -1.0, 1.0 /)
  MeshVert(7)%coor      = (/ 1.0, 1.0, 1.0 /)
  MeshVert(8)%coor      = (/ -1.0, 1.0, 1.0 /)  
  refCenter(:)          = 0
  ptIn                  = (/ 0.1, 0.1, 0.1 /)
  ptOut                 = (/ 2.0, 2.0, 2.0 /)
  !------------------------------------------------------------------------
  !---- Test #5 : measure of reference hexahedron
  !------------------------------------------------------------------------
  testNumber = 5
  CALL COMPUTE_CELL_MEASURE(iCell)
  !
  IF (MeshCell(1)%measure /= Href_vol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #6 : center of reference hexahedron
  !------------------------------------------------------------------------
  testNumber = 6
  CALL COMPUTE_CENTER_OF_CELL(iCell, pCenter)
  !
  IF (.NOT. (ALL(pCenter == refCenter))) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #7 : test if point is in reference hexahedron
  !------------------------------------------------------------------------
  testNumber = 7
  CALL IS_POINT_IN_CELL(ptIn, iCell, isInCell)
  !
  IF (.NOT. (isInCell)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #8 : test if point is in reference hexahedron
  !------------------------------------------------------------------------
  testNumber = 8
  CALL IS_POINT_IN_CELL(ptOut, iCell, isInCell)
  !
  IF (isInCell) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM CELLS_ROUTINESTEST
