PROGRAM FACES_ROUTINESTEST
  !------------------------------------------------------------------------
  !---- Test program for various faces routines
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE MESH_FACES
  USE MESH_VERTICES
  USE MESH_ELEMENTS
  USE MESH_ELEMENTS_CENTERS
  USE MESH_FIND_POINT_IN_ELEMENTS
  USE MESH_NORMALS_AND_MEASURES
  USE IO_TESTS
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'FACES_ROUTINES'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/faces_routines/'  

  LOGICAL(C_BOOL)                  :: isInFace
  
  INTEGER(C_INT)                   :: iFac, testNumber

  REAL(C_DOUBLE)                   :: measure, refMeasure
  REAL(C_DOUBLE), PARAMETER        :: tolerance = 1.0e-4
  REAL(C_DOUBLE), DIMENSION(3)     :: normal, refNormal, pCenter, &
       refCenter, testVector, ptIn, ptOut
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Allocate structure for test (tri face (0,0,0) - (1,0,0) - (0,1,0))
  !------------------------------------------------------------------------
  ALLOCATE(MeshFac(1))
  ALLOCATE(MeshFac(1)%vertex(3))
  ALLOCATE(MeshVert(3))  
  !
  iFac                 = 1
  MeshFac(1)%faceType  = "TRI1"
  MeshFac(1)%nLinVerts = 3
  MeshFac(1)%vertex(1) = 1
  MeshFac(1)%vertex(2) = 2
  MeshFac(1)%vertex(3) = 3
  MeshVert(1)%coor     = (/ 0.0, 0.0, 0.0 /)
  MeshVert(2)%coor     = (/ 1.0, 0.0, 0.0 /)
  MeshVert(3)%coor     = (/ 0.0, 1.0, 0.0 /)
  refMeasure           = 0.5
  refNormal            = (/ 0.0, 0.0, 1.0 /)
  refCenter            = (/ 1.0/3.0, 1.0/3.0, 0.0 /)
  ptIn                 = (/ 0.1, 0.1, 0.0 /)
  ptOut                = (/ 0.0, 0.0, 1.0 /)
  !------------------------------------------------------------------------
  !---- Test #1 : normal to reference tri face
  !------------------------------------------------------------------------
  testNumber = 1
  CALL COMPUTE_FACE_NORMAL_AND_MEASURE(iFac)
  normal               = MeshFac(1)%normal
  testVector(:)        = ABS(refNormal(:)) - ABS(normal(:))
  measure              = MeshFac(1)%measure
  !
  IF (.NOT. (ALL(testVector < tolerance))) THEN
     WRITE(6,*) 'Normal is wrong'
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !
  IF (ABS(measure - refMeasure) > tolerance) THEN
     WRITE(6,*) 'Measure is wrong'     
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !------------------------------------------------------------------------
  !---- Test #2 : center of reference tri face
  !------------------------------------------------------------------------
  testNumber = 2
  CALL COMPUTE_CENTER_OF_FACE(iFac, pCenter)
  testVector(:)        = ABS(refCenter(:)) - ABS(pCenter(:))
  !
  IF (.NOT. (ALL(testVector < tolerance))) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #3 : test if point is in reference tri face
  !------------------------------------------------------------------------
  testNumber = 3
  CALL IS_POINT_IN_FACE(ptIn, iFac, isInFace)
  !
  IF (.NOT. (isInFace)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #4 : test if point is in reference tri face
  !------------------------------------------------------------------------
  testNumber = 4
  CALL IS_POINT_IN_FACE(ptOut, iFac, isInFace)
  !
  IF (isInFace) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Allocate structure for test (quad face (0,0,0) - (1,0,0) - (1,1,0) - (0,1,0))
  !------------------------------------------------------------------------
  DEALLOCATE(MeshFac)
  ALLOCATE  (MeshFac(1))
  ALLOCATE  (MeshFac(1)%vertex(4))  
  DEALLOCATE(MeshVert)
  ALLOCATE  (MeshVert(4))
  !
  iFac                 = 1
  MeshFac(1)%faceType  = "QUA1"
  MeshFac(1)%nLinVerts = 4
  MeshFac(1)%vertex(1) = 1
  MeshFac(1)%vertex(2) = 2
  MeshFac(1)%vertex(3) = 3
  MeshFac(1)%vertex(4) = 4
  MeshVert(1)%coor     = (/ 0.0, 0.0, 0.0 /)
  MeshVert(2)%coor     = (/ 1.0, 0.0, 0.0 /)
  MeshVert(3)%coor     = (/ 1.0, 1.0, 0.0 /)
  MeshVert(4)%coor     = (/ 0.0, 1.0, 0.0 /)
  refNormal            = (/ 0.0, 0.0, 1.0 /)
  refMeasure           = 1.0
  refCenter            = (/ 1.0/2.0, 1.0/2.0, 0.0 /)
  ptIn                 = (/ 0.1, 0.1, 0.0 /)
  ptOut                = (/ 0.0, 0.0, 1.0 /)
  !------------------------------------------------------------------------
  !---- Test #5 : normal to quad face
  !------------------------------------------------------------------------
  testNumber = 5
  CALL COMPUTE_FACE_NORMAL_AND_MEASURE(iFac)  
  normal               = MeshFac(1)%normal
  testVector(:)        = ABS(refNormal(:)) - ABS(normal(:))
  measure              = MeshFac(1)%measure
  !
  IF (.NOT. (ALL(testVector < tolerance))) THEN
     WRITE(6,*) 'Normal is wrong'     
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !
  IF (ABS(measure - refMeasure) > tolerance) THEN
     WRITE(6,*) 'Measure is wrong'     
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !------------------------------------------------------------------------
  !---- Test #6 : center of quad face 
  !------------------------------------------------------------------------
  testNumber = 6
  CALL COMPUTE_CENTER_OF_FACE(iFac, pCenter)
  testVector(:)        = ABS(refCenter(:)) - ABS(pCenter(:))
  !
  IF (.NOT. (ALL(testVector < tolerance))) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #7 : test if point is in reference quad face
  !------------------------------------------------------------------------
  testNumber = 7
  CALL IS_POINT_IN_FACE(ptIn, iFac, isInFace)
  !
  IF (.NOT. (isInFace)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #8 : test if point is in reference quad face
  !------------------------------------------------------------------------
  testNumber = 8
  CALL IS_POINT_IN_FACE(ptOut, iFac, isInFace)
  !
  IF (isInFace) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM FACES_ROUTINESTEST
