PROGRAM FIT_DISPERSION_DATATEST
  !------------------------------------------------------------------------
  !---- Test program for dispersion data fitting program
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE DISPERSION_FIT
  USE MATERIALS_STRUCTURES
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=100, KIND=C_CHAR)           :: dataFile
  CHARACTER(LEN=1000, KIND=C_CHAR)          :: testName = 'FIT_DISPERSION_DATA'
  CHARACTER(LEN=1000, KIND=C_CHAR)          :: repo     = '../../../tst/fit_dispersion_data/'  

  INTEGER(C_INT)                            :: iSol, testNumber
  
  REAL(C_DOUBLE)                            :: errorR, errorI, correctErrorR, &
       correctErrorI, epsError
  REAL(C_DOUBLE), DIMENSION(:), ALLOCATABLE :: omega, epsR, epsI, sol
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Perform fit of gold data with one second-order pole
  !---- This is just a copy of the calls performed in FIT_DISPERSION_DATA
  !------------------------------------------------------------------------
  testNumber = 1
  dataFile   = 'gold'
  !
  ALLOCATE(Mat(0:1), MatDisp(1))
  ALLOCATE(MatDisp(1)%pole0(0))
  ALLOCATE(MatDisp(1)%pole1(0))
  ALLOCATE(MatDisp(1)%pole2(1))
  !
  CALL INIT_MAT_STRUCT(Mat(0))
  CALL INIT_MAT_STRUCT(Mat(1))
  CALL INIT_MATDISP_STRUCT(MatDisp(1))
  !
  Mat(1)%iDispMat = 1
  MatDisp(1)%iMat = 1
  !
  CALL READ_EXPERIMENTAL_DISPERSION_DATA(dataFile, omega, epsR, epsI)
  !
  CALL SIMULATED_ANNEALING(omega, epsR, epsI, 0, 0, 1, sol)
  !
  iSol                  = 1
  Mat(1)%eps(:)         = sol(iSol)
  iSol                  = iSol + 1
  !
  MatDisp(1)%pole2(1)%c = sol(iSol)
  MatDisp(1)%pole2(1)%d = sol(iSol+1)
  MatDisp(1)%pole2(1)%e = sol(iSol+2)
  MatDisp(1)%pole2(1)%f = sol(iSol+3)
  iSol                  = iSol + 4
  !
  CALL QUALITY_DISPERSION_FIT(omega, epsR, epsI, errorR, errorI)
  !------------------------------------------------------------------------
  !---- Test if L1 errors are correct
  !------------------------------------------------------------------------
  correctErrorR = 0.7362552E+00
  correctErrorI = 0.1658324E+01
  epsError      = 1.0E-5
  print*, errorR, correcterrorR
  !
  IF ((ABS(correctErrorR - errorR) > epsError) .OR. (ABS(correctErrorI - errorI) > epsError)) &
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM FIT_DISPERSION_DATATEST
