PROGRAM MATRICESTEST
  !------------------------------------------------------------------------
  !---- Test program for matrix operations
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE MATRICES_STRUCTURES
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'MATRICES'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/matrices/'
  
  INTEGER(C_INT)                   :: testNumber
  
  REAL(C_DOUBLE)                   :: error, val, det, swap
  REAL(C_DOUBLE), DIMENSION(3)     :: v1, v2, v3, v4
  REAL(C_DOUBLE), DIMENSION(3,3)   :: invertibleMatrix, resMatrix, &
       sampleMatrix, symDefPosMatrix
  REAL(C_DOUBLE), DIMENSION(4,4)   :: det4x4Matrix, convUpperMatrix, upperMatrix
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Initialize test matrices
  !------------------------------------------------------------------------
  DATA invertibleMatrix/   &
       1.0,-1.0, 0.0,      &
       0.0, 5.0, 3.0,      &
       2.0, 0.0,-9.0/
  DATA  symDefPosMatrix/   &
       2.0,-1.0, 0.0,      &
      -1.0, 2.0,-1.0,      &
       0.0,-1.0, 2.0/
  DATA     sampleMatrix/   &
       2.0, 1.0,-3.0,      &
       0.0, 2.0, 3.0,      &
       1.0, 0.0, 1.0/
  DATA     det4x4Matrix/   &
       1.0, 2.0, 3.0, 4.0, &
       1.0, 5.0, 6.0, 7.0, &
       0.0, 4.0, 1.0, 2.0, &
       1.0, 2.0, 4.0, 9.0/
  DATA convUpperMatrix/    &
       1.0, 3.0, 2.0, 1.0, &
       0.0, 0.0, 1.0, 0.0, &
       2.0, 0.0, 4.0, 5.0, &
      -1.0, 5.0,-3.0, 0.0/
  DATA     upperMatrix/    &
       1.0, 0.0, 0.0, 0.0, &
       0.0, 1.0, 0.0, 0.0, &
       2.0, 0.0,-6.0, 0.0, &
      -1.0,-1.0, 8.0, 5.0/
  !------------------------------------------------------------------------
  !---- Test #1 : invertible matrix with Gauss pivot (INVERSE_MATRIX)
  !------------------------------------------------------------------------
  testNumber = 1
  CALL INVERSE_MATRIX('GP',3,invertibleMatrix,resMatrix,error)
  !
  IF (error > 1.0e-10) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #2 : invertible matrix with Cholesky factorization (INVERSE_MATRIX)
  !------------------------------------------------------------------------
  testNumber = 2
  CALL INVERSE_MATRIX('CK',3,symDefPosMatrix,resMatrix,error)
  !
  IF (error > 1.0e-10) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #3 : test determinant of 4x4 matrix (COMPUTE_DETERMINANT)
  !------------------------------------------------------------------------
  testNumber = 3
  CALL COMPUTE_DETERMINANT(det4x4Matrix, 4, det)
  !
  IF (det /= -39.0) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #4 : matrix-vector multiplication (MATVECMUL)
  !------------------------------------------------------------------------
  testNumber = 4
  v1  = (/1.0, 2.0, 3.0/)
  CALL MATVECMUL(sampleMatrix,v1,3,3,v2)
  !
  IF ((v2(1) /= 5.0) .OR. (v2(2) /= 5.0) .OR. (v2(3) /= 6.0)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #5 : vector-vector reduction (VECVECRED)
  !------------------------------------------------------------------------
  testNumber = 5
  v1  = (/1.0, 2.0, 3.0/)
  CALL VECVECRED(v1,v1,3,val)
  !
  IF (val /= 14.0) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #6 : Frobenius norm (FROBENIUS_NORM)
  !------------------------------------------------------------------------
  testNumber = 6
  CALL FROBENIUS_NORM(invertibleMatrix,3,val)
  !
  IF (val /= 11.0) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #7 : Matrix conversion to upper triangular form (CONVERT_TO_UPPER_TRIANGULAR)
  !------------------------------------------------------------------------
  testNumber = 7
  CALL CONVERT_TO_UPPER_TRIANGULAR(convUpperMatrix, 4, swap)
  !
  IF (.NOT. (ALL(upperMatrix == convUpperMatrix))) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM MATRICESTEST
