PROGRAM MESH_PARTITIONNINGTEST
  !------------------------------------------------------------------------
  !---- Test mesh partitionning
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE MESH_APPS
  USE MESH_DATA
  USE MESH_FACES
  USE MESH_CELLS
  USE MESH_ELEMENTS
  USE MESH_STRUCTURES
  USE ISO_C_BINDING  
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR)      :: readname
  CHARACTER(LEN=1000, KIND=C_CHAR)      :: testName = 'MESH_PARTITIONNING'
  CHARACTER(LEN=1000, KIND=C_CHAR)      :: repo     = '../../../tst/mesh_partitionning/'  

  LOGICAL(C_BOOL)                       :: isPartitionned

  INTEGER(C_INT)                        :: testNumber

  TYPE(mesh_structure)                  :: localMesh
  TYPE(face), DIMENSION(:), ALLOCATABLE :: localFace
  TYPE(cell), DIMENSION(:), ALLOCATABLE :: localCell
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Test #1 : Partition cubic cavity tetrahedral mesh in 4 subdomains
  !------------------------------------------------------------------------
  testNumber = 1
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'tetCube.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 4)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_tetCube_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)  
  !------------------------------------------------------------------------
  !---- Test #2 : Partition cubic cavity hybrid mesh in 4 subdomains, with
  !---- a low triangle/quadrangle refinement level
  !------------------------------------------------------------------------
  testNumber = 2
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'hybCube.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 4)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_hybCube_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)  
  !------------------------------------------------------------------------
  !---- Test #3 : Partition a doubly periodic mesh in 7 subdomains
  !---- It tests the proper creation of artificial periodic faces
  !------------------------------------------------------------------------
  testNumber = 3
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'tetPeriodic.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 7)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_tetPeriodic_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)  
  !------------------------------------------------------------------------
  !---- Test #4 : Partition a fully hexahedral cubic cavity in 7 subdomains
  !------------------------------------------------------------------------
  testNumber = 4
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'hexCube.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 7)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_hexCube_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)    
  !------------------------------------------------------------------------
  !---- Test #5 : Partition a complex quad/tri hybrid box in 7 subdomains
  !------------------------------------------------------------------------
  testNumber = 5
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'hybQuadTri.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 7)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_hybQuadTri_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)      
  !------------------------------------------------------------------------
  !---- Test #6 : Partition a complex quad/quad hybrid box in 11 subdomains
  !------------------------------------------------------------------------
  testNumber = 6
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL RESET_MESH_COUNTERS(localMesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  IF (ALLOCATED(localFace)) DEALLOCATE(localFace)
  IF (ALLOCATED(localCell)) DEALLOCATE(localCell)
  !
  Mesh%name = 'hybQuadQuad.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL PARTITION_MESH_SUB(Mesh%name, 11)
  !
  CALL RESET_MESH_COUNTERS(Mesh)
  CALL DEALLOCATE_MESH_STRUCTURES
  Mesh%name  = 'subMesh_hybQuadQuad_1.mesh'
  CALL CHECK_TEST_FILE(Mesh%name, testName, testNumber)
  CALL INITIALIZE_MESH_STRUCTURE(Mesh)
  CALL CHECK_IN_MESH_EXTENSION(Mesh%name)
  CALL READ_MESH
  !
  isPartitionned = .FALSE.
  IF (Mesh%nFac + Mesh%nCell /= 0) isPartitionned = .TRUE.
  !
  IF (.NOT. isPartitionned) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)  
  !------------------------------------------------------------------------
  !---- Clean files
  !------------------------------------------------------------------------
  CALL CLEAN_DIOGENES_SUB
  !------------------------------------------------------------------------
END PROGRAM MESH_PARTITIONNINGTEST
