PROGRAM NACHOS_MESHTEST
  !------------------------------------------------------------------------
  !---- Test program for NACHOS_MESH module
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE MESH_DATA
  USE MESH_STATS
  USE MESH_STRUCTURES
  USE IO_TESTS
  USE ISO_C_BINDING  
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'NACHOS_MESH'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/nachos_mesh/'  

  INTEGER(C_INT)                   :: nVert, nFacPhy, nFacAbs, nFacPer, &
       nFacCurv, nFacTfsf, nTet, nTetCurv, nFacLin, nTetLin, testNumber

  REAL(C_DOUBLE)  :: hmin, hmax, coorMin(3), coorMax(3), tol
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Initialize data for test 1
  !------------------------------------------------------------------------
  Mesh%name      = 'test_1.nmesh'
  nVert          = 18374
  nFacPhy        = 664
  nFacAbs        = 664
  nFacLin        = 664
  nFacTfsf       = 0
  nTet           = 105677
  nTetLin        = 105677
  hmin           = 0.2606260E-08
  hmax           = 0.3427199E-06
  coorMin(1)     =-0.6400000E-06
  coorMin(2)     =-0.6400000E-06
  coorMin(3)     =-0.3500000E-06
  coorMax(1)     = 0.6400000E-06
  coorMax(2)     = 0.6400000E-06
  coorMax(3)     = 0.4950000E-06
  tol            = 1.0e-10
  !------------------------------------------------------------------------
  !---- Test #1 : nanocube mesh
  !------------------------------------------------------------------------
  testNumber = 1
  CALL PREPROCESS_MESH
  !
  IF ( (nVert      /= Mesh%nVert)                  .OR. &
       (nFacPhy    /= Mesh%nFacPhy)                .OR. &
       (nFacAbs    /= Mesh%nFacAbs)                .OR. &
       (nFacLin    /= Mesh%nFacLin)                .OR. &
       (nFacTfsf   /= Mesh%nFacTfsf)               .OR. &
       (nTet       /= Mesh%nCellTet)               .OR. &
       (nTetLin    /= Mesh%nCellTetLin)            .OR. &
       (ABS(coorMin(1) - Mesh%coorMin(1)) > tol)   .OR. &
       (ABS(coorMin(2) - Mesh%coorMin(2)) > tol)   .OR. &
       (ABS(coorMin(3) - Mesh%coorMin(3)) > tol)   .OR. &
       (ABS(coorMax(1) - Mesh%coorMax(1)) > tol)   .OR. &
       (ABS(coorMax(2) - Mesh%coorMax(2)) > tol)   .OR. &
       (ABS(coorMax(3) - Mesh%coorMax(3)) > tol)   .OR. &
       (ABS(hmin - Mesh%hmin) > tol)               .OR. &
       (ABS(hmin - Mesh%hmin) > tol)) THEN
     CALL PRINT_MESH_STATS(Mesh)
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !------------------------------------------------------------------------
END PROGRAM NACHOS_MESHTEST
