PROGRAM QUADRATURESTEST
  !------------------------------------------------------------------------
  !---- Test program for all quadrature and cubature routines
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE QUADRATURES_2D
  USE QUADRATURES_3D
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'QUADRATURES'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/quadratures/'  
  
  INTEGER(C_INT)                   :: iPtQuad, testNumber
  
  REAL(C_DOUBLE)                   :: val, tol, exact
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Initialize data
  !------------------------------------------------------------------------
  tol = 1E-8
  !------------------------------------------------------------------------
  !---- Test #1 : Triangle, f(x,y) = 1 (order 0)
  !------------------------------------------------------------------------
  testNumber = 1
  CALL QUADRATURES_TRIANGLE(1)
  exact = 1.0d0/2.0d0
  !
  val = 0.0d0
  DO iPtQuad=1,SIZE(qw2d)
     val = val + qw2d(iPtQuad)
  ENDDO
  val = val/2.0d0 ! Divide by area of unit triangle
  !
  IF (ABS(val-exact) .GE. tol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #2 : Triangle, f(x,y) = x^2*y^2 (order 4)
  !------------------------------------------------------------------------
  testNumber = 2
  CALL QUADRATURES_TRIANGLE(4)
  exact = 1.0d0/180.0d0
  !
  val = 0.0d0
  DO iPtQuad=1,SIZE(qw2d)
     val = val + qw2d(iPtQuad)*qx2d(iPtQuad)**2*qy2d(iPtQuad)**2
  ENDDO
  val = val/2.0d0 ! Divide by area of unit triangle
  !
  IF (ABS(val-exact) .GE. tol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #3 : Tetrahedron, f(x,y,z) = 1 (order 0)
  !------------------------------------------------------------------------
  testNumber = 3
  CALL QUADRATURES_TETRAHEDRON(1)
  exact = 1.0d0/6.0d0
  !
  val = 0.0d0
  DO iPtQuad=1,SIZE(qw3d)
     val = val + qw3d(iPtQuad)
  ENDDO
  val = val/6.0d0 ! Divide by volume of unit tetrahedron
  !
  IF (ABS(val-exact) .GE. tol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #4 : Tetrahedron, f(x,y,z) = x*y*z (order 3)
  !------------------------------------------------------------------------
  testNumber = 4
  CALL QUADRATURES_TETRAHEDRON(3)
  exact = 1.0d0/720.0d0
  !
  val = 0.0d0
  DO iPtQuad=1,SIZE(qw3d)
     val = val + qw3d(iPtQuad)*qx3d(iPtQuad)*qy3d(iPtQuad)*qz3d(iPtQuad)
  ENDDO
  val = val/6.0d0 ! Divide by volume of unit tetrahedron
  !
  IF (ABS(val-exact) .GE. tol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #5 : Tetrahedron, f(x,y,z) = x**2*y**2*z (order 5)
  !------------------------------------------------------------------------
  testNumber = 5
  CALL QUADRATURES_TETRAHEDRON(5)
  exact = 1.0d0/10080.0d0
  !
  val = 0.0d0
  DO iPtQuad=1,SIZE(qw3d)
     val = val + qw3d(iPtQuad)*qx3d(iPtQuad)**2*qy3d(iPtQuad)**2*qz3d(iPtQuad)
  ENDDO
  val = val/6.0d0 ! Divide by volume of unit tetrahedron
  !
  IF (ABS(val-exact) .GE. tol) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM
