PROGRAM SOLUTION_PROCESSINGTEST
  !------------------------------------------------------------------------
  !---- Test program for SOLUTION_PROCESSING module
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE MESH_DATA
  USE MESH_STATS
  USE MESH_STRUCTURES
  USE IO_TESTS
  USE IO_FILES
  USE ISO_C_BINDING
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'SOLUTION_PROCESSING'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/solution_processing/'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: meshFile, solFile, refMeshFile, refSolFile

  LOGICAL(C_BOOL)                  :: meshFileExists, solFileExists, &
       refMeshFileExists, refSolFileExists

  INTEGER(C_INT)                   :: testNumber, diffMeshSize, diffSolSize

  REAL(C_DOUBLE)                   :: hmin, hmax, coorMin(3), coorMax(3), tol
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Test #1 : solution of cubic cavity, order 2, with plane
  !------------------------------------------------------------------------
  testNumber  = 1
  meshFile    = 'cube_view.mesh'
  solFile     = 'cube_view.sol'
  refMeshFile = 'refCubeMesh'
  refSolFile  = 'refCubeSol'
  CALL SYSTEM('view_data viewConfig;')
  CALL DOES_FILE_EXIST(meshFile,    meshFileExists   )
  CALL DOES_FILE_EXIST(solFile,     solFileExists    )
  CALL DOES_FILE_EXIST(refMeshFile, refMeshFileExists)
  CALL DOES_FILE_EXIST(refSolFile,  refSolFileExists )
  !
  !---- Fail if a file is missing
  IF ( (.NOT. meshFileExists)    .OR. (.NOT. solFileExists)    .OR. &
       (.NOT. refMeshFileExists) .OR. (.NOT. refSolFileExists)) THEN
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !
  CALL SYSTEM('diff '//TRIM(meshFile)//' '//TRIM(refMeshFile)//' > diffMesh')
  CALL SYSTEM('diff '//TRIM(solFile) //' '//TRIM(refSolFile) //' > diffSol' )
  CALL GET_FILE_SIZE_IN_BYTES('diffMesh', diffMeshSize)
  CALL GET_FILE_SIZE_IN_BYTES('diffSol' , diffSolSize )
  !
  !---- Fail if diff is not empty
  IF ((diffMeshSize /= 0) .OR. (diffSolSize /= 0)) THEN
     CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  ENDIF
  !------------------------------------------------------------------------
  CALL CLEAN_DIOGENES_TEST
  !------------------------------------------------------------------------
END PROGRAM SOLUTION_PROCESSINGTEST
