PROGRAM SPECIAL_FUNCTIONSTEST
  !------------------------------------------------------------------------
  !---- Test program for special functions
  !------------------------------------------------------------------------
  !---- Shared modules
  !------------------------------------------------------------------------
  USE IO_TESTS
  USE SPECIAL_FUNCTIONS
  USE MATH_PHYS_CONSTANTS
  USE ISO_C_BINDING  
  !------------------------------------------------------------------------
  IMPLICIT NONE
  !------------------------------------------------------------------------
  !---- Shared variables
  !------------------------------------------------------------------------
  CHARACTER(LEN=1000, KIND=C_CHAR) :: testName = 'SPECIAL_FUNCTIONS'
  CHARACTER(LEN=1000, KIND=C_CHAR) :: repo     = '../../../tst/special_functions/'  
  
  INTEGER(C_INT)                   :: i1, i2, i3, exponent, testNumber
  INTEGER(C_LONG_LONG)             :: iCantor1, iCantor2

  REAL(C_DOUBLE)                   :: val, mantissa, theta, phi
  REAL(C_DOUBLE), DIMENSION(3)     :: u, v, sol, w, z
  !------------------------------------------------------------------------
  !---- Move to test repository
  !------------------------------------------------------------------------
  CALL MOVE_TO_REPO(repo)  
  !------------------------------------------------------------------------
  !---- Test #1 : test the symmetry of the Cantor pairing
  !------------------------------------------------------------------------
  testNumber = 1
  i1 = 1
  i2 = 2
  i3 = 3
  CALL CANTOR_PAIRING(i1,i2,i3,iCantor1)
  CALL CANTOR_PAIRING(i1,i3,i2,iCantor2)
  !
  IF (iCantor1 /= iCantor2) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #2 : test high values for Cantor pairing
  !------------------------------------------------------------------------
  testNumber = 2
  i1 = 1000000
  i2 = 1000001
  i3 = 1000002
  CALL CANTOR_PAIRING(i1,i2,i3,iCantor1)
  !
  IF (iCantor1 < 0) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #3 : test negative input values for Cantor pairing
  !------------------------------------------------------------------------
  testNumber = 3
  i1 = -1
  i2 = -2
  i3 = -3
  CALL CANTOR_PAIRING(i1,i2,i3,iCantor1)
  CALL CANTOR_PAIRING(i1,i3,i2,iCantor2)
  !
  IF (iCantor1 /= iCantor2) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #4 : test cross product
  !------------------------------------------------------------------------
  testNumber = 4
  u   = [1.0, 2.0, 3.0]
  v   = [2.0, 3.0, 1.0]
  sol = [-7.0, 5.0, -1.0]
  CALL CROSS_PROD(u, v, w)
  !
  IF (.NOT. ALL(w .EQ. sol)) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #5 : test mixt product
  !------------------------------------------------------------------------
  testNumber = 5
  u   = [1.0, 2.0, 3.0]
  v   = [2.0, 3.0, 1.0]
  w   = [1.0, 0.0, 1.0]
  val = MIXT_PROD(u, v, w)
  !
  IF (val /= -8.0) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #6 : test mantissa/exponent computation
  !------------------------------------------------------------------------
  testNumber = 6
  val = 1.73e12
  CALL GET_MANTISSA_EXPONENT(val, mantissa, exponent)
  !
  IF (exponent /= 12) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
  !---- Test #7 : test SIN function (test implemented to make sure
  !----           that virtual machines don't do sh*t)
  !------------------------------------------------------------------------
  testNumber = 7
  theta = 180.0
  phi   = 0.0
  CALL DEGREES_TO_RADIANS(theta)
  CALL DEGREES_TO_RADIANS(phi)
  val   = SIN(theta)*COS(phi)
  !
  IF (val > numericalZero) CALL PRINT_TEST_ERROR_MESSAGE(testName, testNumber)
  !------------------------------------------------------------------------
END PROGRAM SPECIAL_FUNCTIONSTEST
