## #################################################
## Module to find libraries for Dense Linear Algebra
##
## BLAS - LAPACK - ScaLAPACK
## #################################################

set(DLA_FOUND "NO")
set(BLAS_FOUND "NO")
set(LAPACK_FOUND "NO")
set(SCALAPACK_FOUND "NO")

if(BLAS_DIR)
  # BLAS_DIR is specified
  set(BLAS_LIBRARY_DIR ${BLAS_DIR}/lib64)
elseif(DEFINED ENV{BLAS_DIR})
  # BLAS_DIR is specified
  set(BLAS_LIBRARY_DIR $ENV{BLAS_DIR}/lib64)
else()
  # Otherwise look for standard places
  find_path(BLAS_LIBRARY_DIR
    NAMES libblas.so
    PATHS
    /opt/BLAS/lib
    /usr/BLAS/lib
    /usr/lib
    ~/lib)
endif()

if(LAPACK_DIR)
  # LAPACK_DIR is specified
  set(LAPACK_LIBRARY_DIR ${LAPACK_DIR}/lib64)
elseif(DEFINED ENV{LAPACK_DIR})
  # LAPACK_DIR is specified
  set(LAPACK_LIBRARY_DIR $ENV{LAPACK_DIR}/lib64)
else()
  # Otherwise look for standard places 
  find_path(LAPACK_LIBRARY_DIR
    NAMES liblapack.so
    PATHS
    /opt/LAPACK/lib
    /usr/LAPACK/lib
    /usr/lib
    ~/lib)
endif()

if(SCALAPACK_DIR)
  # SCALAPACK_DIR is specified
  set(SCALAPACK_LIBRARY_DIR ${SCALAPACK_DIR}/lib)
elseif(DEFINED ENV{SCALAPACK_DIR})
  # SCALAPACK_DIR is specified
  set(SCALAPACK_LIBRARY_DIR $ENV{SCALAPACK_DIR}/lib)
else()
  # Otherwise look for standard places 
  find_path(SCALAPACK_LIBRARY_DIR
    NAMES libscalapack.so
    PATHS
    /opt/SCALAPACK/lib
    /usr/SCALAPACK/lib
    /usr/lib
    ~/lib)
endif()

if(BLAS_LIBRARY_DIR)
  set(BLAS_FOUND YES)

  find_library(BLAS_LIBRARY
    NAMES blas
    PATHS ${BLAS_LIBRARY_DIR}
    NO_DEFAULT_PATH)

  set(BLAS_LIBRARY ${BLAS_LIBRARY})
else()
  if(BLAS_FIND_REQUIRED)
    message(FATAL_ERROR "BLAS not found, please set BLAS_DIR to your BLAS Dense Linear Algebra install directory (../cmake/FindDLA.cmake)")
  endif()
endif()

if(LAPACK_LIBRARY_DIR)
  set(LAPACK_FOUND YES)

  find_library(LAPACK_LIBRARY
    NAMES lapack
    PATHS ${LAPACK_LIBRARY_DIR}
    NO_DEFAULT_PATH)

  set(LAPACK_LIBRARY ${LAPACK_LIBRARY})
else()
  if(LAPACK_FIND_REQUIRED)
    message(FATAL_ERROR "LAPACK not found, please set LAPACK_DIR to your Dense Linear Algebra install directory (../cmake/FindDLA.cmake)")
  endif()
endif()

if(SCALAPACK_LIBRARY_DIR)
  set(SCALAPACK_FOUND YES)

  find_library(SCALAPACK_LIBRARY 
    NAMES scalapack
    PATHS ${SCALAPACK_LIBRARY_DIR} 
    NO_DEFAULT_PATH)

  set(SCALAPACK_LIBRARY ${SCALAPACK_LIBRARY})
else()
  if(SCALAPACK_FIND_REQUIRED)
    message(FATAL_ERROR "SCALAPACK not found, please set SCALAPACK_DIR to your Dense Linear Algebra install directory (../cmake/FindDLA.cmake)")
  endif()
endif()

if(BLAS_LIBRARY_DIR AND LAPACK_LIBRARY_DIR AND SCALAPACK_LIBRARY_DIR)
  set(DLA_FOUND YES)
  set(DLA_LIBRARIES ${BLAS_LIBRARY} ${LAPACK_LIBRARY} ${SCALAPACK_LIBRARY})
endif()
