## ########################################
## Module to find Intel Math Kernel Library
## ########################################

set(MKL_FOUND "NO")

if(MKL_DIR)
  # MKL_DIR is specified
  set(MKL_LIBRARY_DIR ${MKL_DIR}/lib/intel64)
elseif(DEFINED ENV{MKL_DIR})
  # MKL_DIR is specified
  set(MKL_LIBRARY_DIR $ENV{MKL_DIR}/lib/intel64)
else()
  # Otherwise look for specific places
  find_path(MKL_LIBRARY_DIR
    NAMES libmkl_sequential.so
    PATHS
    /opt/MKL/lib
    /usr/MKL/lib
    /usr/lib
    ~/lib)
endif()

if(MKL_LIBRARY_DIR)
  set(MKL_FOUND YES)

  find_library(MKL_LIBRARY_1 
    NAMES mkl_scalapack_lp64
    PATHS ${MKL_LIBRARY_DIR} 
    NO_DEFAULT_PATH)
  find_library(MKL_LIBRARY_2 
    NAMES mkl_blacs_intelmpi_lp64
    PATHS ${MKL_LIBRARY_DIR} 
    NO_DEFAULT_PATH)
  find_library(MKL_LIBRARY_3 
    NAMES mkl_intel_lp64
    PATHS ${MKL_LIBRARY_DIR} 
    NO_DEFAULT_PATH)
  find_library(MKL_LIBRARY_4 
    NAMES mkl_core
    PATHS ${MKL_LIBRARY_DIR} 
    NO_DEFAULT_PATH)
  find_library(MKL_LIBRARY_5 
    NAMES mkl_intel_thread
    PATHS ${MKL_LIBRARY_DIR} 
    NO_DEFAULT_PATH)
 
  set(MKL_LIBRARIES ${MKL_LIBRARY_1} ${MKL_LIBRARY_2} ${MKL_LIBRARY_3} ${MKL_LIBRARY_4} ${MKL_LIBRARY_5})
else()
  if(MKL_FIND_REQUIRED)
    message(FATAL_ERROR "Intel Math Kernel Library not found, please set MKL_DIR to your MKL install directory")
  endif()
endif()
