MODULE HDG_MATERIALS

  !====================================================================
  ! Definition and data structure associated to materials in hdgm 
  ! (frequency domain)
  !
  ! Warning: 
  ! differs from data structure associated to materials in dgtd method 
  ! (cf module materials.f90 in common library)
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Data structure associated to a material
  !  
  ! eps        ==> relative dielectric permittivity (dimensionless)
  ! mu         ==> relative magnetic permeability (dimensionless)
  ! z          ==> relative impedance (dimensionless)
  ! rho        ==> density
  ! sigma      ==> conductivity
  ! tau        ==> stabilization parameter in hdgm
  ! matNum     ==> number of the material as given in dataHorse file
  !--------------------------------------------------------------------
  TYPE hdg_material
     REAL(C_DOUBLE)  :: eps, mu, sigma, rho, tau, z

     INTEGER(C_INT)  :: matNum
  END TYPE hdg_material
  !--------------------------------------------------------------------
  TYPE(hdg_material), DIMENSION(:), ALLOCATABLE, TARGET :: hdgMat
  !------------------------------------------------------------------------

END MODULE HDG_MATERIALS
