MODULE SCHWARZ_STRUCTURES

  !====================================================================
  ! Initialize all the necessary data structures for Schwarz algorithm
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE SCHWARZ_DATA_STRUCTURES_INIT
    !------------------------------------------------------------------
    !---- Inititalization of data structures for HDGM combined to a 
    !---- Scharwz algorithm
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Array of indices of interfaces (i.e. artificial faces)
    !------------------------------------------------------------------
    CALL SCHWARZ_INDEX_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- EM field on interfaces
    !------------------------------------------------------------------
    CALL SCHWARZ_EM_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- RHS on interfaces
    !------------------------------------------------------------------
    CALL SCHWARZ_RHS_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE SCHWARZ_DATA_STRUCTURES_INIT

  !====================================================================
  !====================================================================
  !====================================================================
  
  SUBROUTINE SCHWARZ_INDEX_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- Inititalization of the array of interfaces indices
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_FACES
    USE MESH_FIND_FACES
    USE SCHWARZ_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: faceNum, iFac
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    ALLOCATE(indArt(Mesh%nFac))
    DO iFac=1,Mesh%nFac
       indArt(iFac) = -1
    ENDDO
    DO iFac=1,Mesh%nFac
       IF (MeshFac(iFac)%isArtificial) THEN
          faceNum = iFac
          CALL FIND_DERIVED_FACE_IN_LIST_FROM_VERTICES(MeshFac(faceNum)%vertex,          &
               MeshFacArt, Mesh%nFacArt, faceNum)
          indArt(iFac) = faceNum
       ENDIF
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE SCHWARZ_INDEX_INTERFACE_INIT

  !====================================================================
  !====================================================================
  !====================================================================
  
  SUBROUTINE SCHWARZ_EM_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- Inititalization of EM on interfaces i.e. on artificial boundary 
    !---- faces
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE BASES
    USE BASES_LAGRANGE
    USE MESH_CELLS
    USE MESH_DATA
    USE MESH_FIND_FACES
    USE MESH_FACES
    USE MESH_VERTICES
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    USE REFERENCE_CELLS
    USE SCHWARZ_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: cellNum, faceNum, faceNumArt, faceNumHyb, i,     &
         iFac, iNeighb, iVert, k, nbFacNeighbMax, order, orderF

    INTEGER(C_INT), DIMENSION(:)  , ALLOCATABLE :: nDoFsRecv, nDoFsSend
    INTEGER(C_INT), DIMENSION(:)  , ALLOCATABLE :: recvDoFs,  sendDoFs
    INTEGER(C_INT), DIMENSION(:)  , ALLOCATABLE :: recvOrder, sendOrder

    REAL(C_DOUBLE), DIMENSION(3,8)              :: coorHex
    REAL(C_DOUBLE), DIMENSION(3,4)              :: coorTet
    REAL(C_DOUBLE), DIMENSION(24)               :: recvCoorHex, sendCoorHex

    COMPLEX(C_DOUBLE_COMPLEX) :: zero
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Dynamic allocation and initialization of EM on interfaces for 
    !---- communications in Schwarz algorithm
    !------------------------------------------------------------------
    zero = (0.d0,0.d0)
    !
    ALLOCATE(EMex(Mesh%nFacArt))
    !
    nbFacNeighbMax = 0
    DO iNeighb=1,nbNeighb
       nbFacNeighbMax = MAX(nbFacNeighbMax,neighb(iNeighb)%nbFac)
    ENDDO
    ALLOCATE(recvDoFs(nbFacNeighbMax),  sendDoFs(nbFacNeighbMax))
    ALLOCATE(recvOrder(nbFacNeighbMax), sendOrder(nbFacNeighbMax))
    DO iNeighb=1,nbNeighb
       DO iFac=1,neighb(iNeighb)%nbFac
          faceNum         = neighb(iNeighb)%faces(iFac)
          cellNum         = MeshFac(faceNum)%neighb(1)
          sendOrder(iFac) = hdgEM(cellNum)%order
          sendDoFs(iFac)  = hdgEM(cellNum)%nDofS
       ENDDO
       !
       !---- Send/Recv interpolation orders and number of surfacic DoFs 
       !---- of neighboring cells on interfaces
       CALL MPI_SENDRECV(sendOrder, neighb(iNeighb)%nbFac, MPI_INTEGER,                  &
            neighb(ineighb)%id, 100, recvOrder, neighb(iNeighb)%nbFac,                   &
            MPI_INTEGER, neighb(iNeighb)%id, 100, defaultComm, status,                   &
            mpiError)
       CALL MPI_SENDRECV(sendDoFs, neighb(iNeighb)%nbFac, MPI_INTEGER,                   &
            neighb(ineighb)%id, 200, recvDoFs, neighb(iNeighb)%nbFac,                    &
            MPI_INTEGER, neighb(iNeighb)%id, 200, defaultComm, status,                   &
            mpiError)
       !
       DO iFac=1,neighb(iNeighb)%nbFac
          faceNum                     = neighb(iNeighb)%faces(iFac)
          faceNumArt                  = indArt(faceNum)
          EMex(faceNumArt)%order      = recvOrder(iFac)
          EMex(faceNumArt)%nDoFs      = recvDoFs(iFac) 
          ALLOCATE(EMex(faceNumArt)%E%vec(EMex(faceNumArt)%nDoFs,3))
          ALLOCATE(EMex(faceNumArt)%H%vec(EMex(faceNumArt)%nDoFs,3))
          EMex(faceNumArt)%E%vec(:,:) = zero
          EMex(faceNumArt)%H%vec(:,:) = zero
       ENDDO
    ENDDO
    DEALLOCATE(recvDoFs,  sendDoFs)
    DEALLOCATE(recvOrder, sendOrder)
    !
    !---- Dispatch surfacic matrix
    DO iNeighb=1,nbNeighb
       DO iFac=1,neighb(iNeighb)%nbFac
          faceNum    = neighb(iNeighb)%faces(iFac)
          orderF     = Lambda(faceNum)%order
          faceNumArt = indArt(faceNum)
          order      = EMex(faceNumArt)%order
          cellNum    = MeshFac(faceNum)%neighb(1)
          IF (MeshFac(faceNum)%isHybrid) THEN
             IF (MeshCell(cellNum)%cellType == "HEX1") THEN
                EMex(faceNumArt)%srfMtx%ptr => mSurfTet(order,orderF)
                !
                k = 0
                DO iVert=1,8
                   DO i=1,3
                      sendCoorHex(i+k) = MeshVert(MeshCell(cellNum)%vertex(iVert))%coor(i)
                   ENDDO
                   k = k + 3
                ENDDO
                CALL MPI_SEND(sendCoorHex, 24, MPI_DOUBLE_PRECISION, neighb(iNeighb)%id, &
                     300, defaultComm, mpiError)
             ELSE
                CALL MPI_RECV(recvCoorHex, 24, MPI_DOUBLE_PRECISION, neighb(iNeighb)%id, &
                     300, defaultComm, status, mpiError)
                k = 0
                DO iVert=1,8
                   DO i=1,3
                      coorHex(i,iVert) = recvCoorHex(i+k)
                   ENDDO
                   k = k + 3
                ENDDO
                DO iVert=1,4
                   coorTet(:,iVert) = MeshVert(MeshCell(cellNum)%vertex(iVert))%coor(:)
                ENDDO
                faceNumHyb = faceNum
                CALL FIND_DERIVED_FACE_IN_LIST_FROM_VERTICES(MeshFac(faceNumHyb)%vertex, &
                     MeshFacHyb, Mesh%nFacHyb, faceNumHyb)
                CALL BASES_LAGRANGE_MATRICES_HYBRID_HDG_F(faceNumHyb,                    &
                     EMex(faceNumArt)%order, coorHex,                                    &
                     MeshCell(cellNum)%measure, coorTet, orderF)
                 EMex(faceNumArt)%srfMtx%ptr => mSurfHybF(faceNumHyb)
             ENDIF
          ELSE
             IF (MeshCell(cellNum)%cellType == "TET1") THEN
                EMex(faceNumArt)%srfMtx%ptr => mSurfTet(order,orderF)
             ELSE 
                EMex(faceNumArt)%srfMtx%ptr => mSurfHex(order,orderF)
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !------------------------------------------------------------------
    !---- Compute the number of DoFs for communications
    !------------------------------------------------------------------
    nDoFsInterface = 0
    DO iFac=1,Mesh%nFacArt
       faceNum        = MeshFacArt(iFac)%faceNum
       nDoFsInterface = nDoFsInterface + 2*Lambda(faceNum)%nDof
    ENDDO
    !
    ALLOCATE(nDoFsRecv(nbNeighb), nDoFsSend(nbNeighb))
    ALLOCATE(nDoFsComm(nbNeighb))
    nDoFsSend    = 0
    nDoFsRecv    = 0
    nDoFsComm    = 0
    nDoFsCommMax = 0
    DO iNeighb=1,nbNeighb
       DO iFac=1,neighb(iNeighb)%nbFac
          faceNum            = neighb(iNeighb)%faces(iFac)
          cellNum            = MeshFac(faceNum)%neighb(1)
          nDoFsSend(iNeighb) = nDoFsSend(iNeighb) + hdgEM(cellNum)%nDofS
          faceNumArt         = indArt(faceNum)
          nDoFsRecv(iNeighb) = nDoFsRecv(iNeighb) + EMex(faceNumArt)%nDoFs
       ENDDO
       nDoFsComm(iNeighb)    = MAX(nDoFsSend(iNeighb),nDoFsRecv(iNeighb))
       nDoFsCommMax          = MAX(nDoFsCommMax,      nDoFsComm(iNeighb))
    ENDDO
    DEALLOCATE(nDoFsRecv, nDoFsSend)
    !------------------------------------------------------------------
    !---- Dynamic allocation and initialization of EM on interfaces for 
    !---- send/recv MPI communications in Schwarz algorithm
    !------------------------------------------------------------------
    ALLOCATE(EMrecv(6*nDoFsCommMax), EMsend(6*nDoFsCommMax))
    EMrecv = zero
    EMsend = zero
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE SCHWARZ_EM_INTERFACE_INIT

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE SCHWARZ_RHS_INTERFACE_INIT
    !------------------------------------------------------------------
    !---- Inititalization of RHS on interfaces for hdgm combined to a 
    !---- Scharwz algorithm
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE MESH_DATA
    USE MESH_FACES
    USE NUMERICAL_TYPES
    USE SCHWARZ_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: faceNum, faceNumArt, iFac, nDoFs

    COMPLEX(C_DOUBLE_COMPLEX) :: zero
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Dynamic allocation and initialization of RHS on interfaces i.e.
    !---- on artificial boundary faces
    !------------------------------------------------------------------
    zero = (0.d0, 0.d0)
    !
    ALLOCATE(rhsSchArt(Mesh%nFacArt))
    DO iFac=1,Mesh%nFacArt
       faceNum                        = MeshFacArt(iFac)%faceNum
       nDoFs                          = Lambda(faceNum)%nDof
       faceNumArt                     = indArt(faceNum)
       ALLOCATE(rhsSchArt(faceNumArt)%vec(nDoFs,2))
       rhsSchArt(faceNumArt)%vec(:,:) = zero
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE SCHWARZ_RHS_INTERFACE_INIT

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE SCHWARZ_DATA_STRUCTURES_END
    !------------------------------------------------------------------
    !---- Deallocate all data structures defined in this module
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE MESH_DATA
    USE SCHWARZ_TYPES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: iFac
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Deallocate all data structures defined for Schwarz algorithm
    !------------------------------------------------------------------
    !
    !---- Deallocate DofS on interfaces for MPI communications
    DEALLOCATE(nDoFsComm)
    !
    !---- Deallocate RHS on interfaces
    DO iFac=1,Mesh%nFacArt
       DEALLOCATE(rhsSchArt(iFac)%vec)
    ENDDO
    DEALLOCATE(rhsSchArt)
    !
    !---- Deallocate EM field on interfaces
    DO iFac=1,Mesh%nFacArt
       DEALLOCATE(EMex(iFac)%E%vec)
       DEALLOCATE(EMex(iFac)%H%vec)
    ENDDO
    DEALLOCATE(EMex, EMrecv, EMsend)
    !
    !---- Deallocate the array of interface indices
    DEALLOCATE(indArt)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE SCHWARZ_DATA_STRUCTURES_END

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE SCHWARZ_STRUCTURES
