MODULE SCHWARZ_TYPES

  !====================================================================
  ! All the types used in the HDGM combined to a Schwarz algorithm 
  ! routines
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  USE MATRICES_STRUCTURES
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Numerical element
  !--------------------------------------------------------------------
  ! E, H                         ==> EM fields
  ! nDofS                        ==> surfacic number of degrees of freedom
  ! order                        ==> polynomial order for field representation
  ! srfMtx(:)                    ==> Surface matrices of the element
  !--------------------------------------------------------------------
  TYPE numerical_element_comm
     TYPE(complex_vector_field) :: E, H

     INTEGER(C_INT)             :: nDoFs, order

     TYPE(rect_matrix_ptr)      :: srfMtx

  END TYPE numerical_element_comm

  !--------------------------------------------------------------------
  ! Discretized electromagnetic fields and related FE informations
  !--------------------------------------------------------------------
  TYPE(numerical_element_comm),  DIMENSION(:), ALLOCATABLE, TARGET :: EMex
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Numerical element for MPI communications
  !--------------------------------------------------------------------
  COMPLEX(C_DOUBLE_COMPLEX), DIMENSION(:), ALLOCATABLE :: EMrecv, EMsend
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Number of DoFs for MPI communications
  !--------------------------------------------------------------------
  INTEGER(C_INT)                            :: nDoFsCommMax, nDoFsInterface

  INTEGER(C_INT), DIMENSION(:), ALLOCATABLE :: nDoFsComm
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! RHS on interfaces for local linear systems (in terms of DoFs of the 
  ! hybrid variable) in HDGM combined to a Schwarz algorithm
  !--------------------------------------------------------------------
  TYPE(complex_vector_field), DIMENSION(:), ALLOCATABLE :: rhsSchArt
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Array of indices for interfaces (i.e. artificial faces)
  !--------------------------------------------------------------------
  INTEGER(C_INT), DIMENSION(:), ALLOCATABLE :: indArt
  !--------------------------------------------------------------------
END MODULE SCHWARZ_TYPES
