MODULE MATERIAL_STRUCTURES

  !====================================================================
  ! Computes all the necessary material structures
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE PREPROCESS_MATERIALS
    !------------------------------------------------------------------
    !---- Read matFile and print a status in output file
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Shared modules
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Read DataMaterials and print sumUp
    !------------------------------------------------------------------
    CALL READ_DATAMATERIALS
    CALL PRINT_MATERIAL_STATS
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE PREPROCESS_MATERIALS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE READ_DATAMATERIALS
    !------------------------------------------------------------------
    !---- Read DataMaterials
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE HDG_MATERIALS
    USE MATH_PHYS_CONSTANTS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    CHARACTER(LEN=100, KIND=C_CHAR) :: readChain
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: nMat, iMat
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Read DataMaterials
    !------------------------------------------------------------------
    OPEN(UNIT=dataUnit, FILE=dataFile, STATUS="UNKNOWN")
    !
    !---- Go to materials description part
    DO WHILE (INDEX(readChain,'#DataMaterials')==0)
       READ(dataUnit,'(a)') readChain
    ENDDO
    !
    !---- Number of materials
    READ(dataUnit, *) nMat
    !
    !---- Allocate the material structure
    ALLOCATE(hdgMat(nMat))
    !
    !---- Read DataMaterials
    DO iMat=1,nMat
       READ(dataUnit,*) hdgMat(iMat)%matNum, hdgMat(iMat)%eps, hdgMat(iMat)%mu, &
            hdgMat(iMat)%sigma, hdgMat(iMat)%rho, hdgMat(iMat)%tau
    ENDDO
    !
    CLOSE(dataUnit)
    !
    !---- Impedance, Sigma has to be redimensioned in meters^-1
    DO iMat=1,nMat
       hdgMat(iMat)%z = SQRT(hdgMat(iMat)%mu / hdgMat(iMat)%eps)
       hdgMat(iMat)%sigma = z0*hdgMat(iMat)%sigma
    ENDDO
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE READ_DATAMATERIALS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE PRINT_MATERIAL_STATS
    !------------------------------------------------------------------
    !---- Module
    !------------------------------------------------------------------
    USE HDG_MATERIALS
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: iMat
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Print statistics
    !------------------------------------------------------------------
    IF (idWrite) THEN
       WRITE(6, 10)
       WRITE(6, '(a)')           ' Material characteristics          '
       WRITE(6, 10)
       !
       IF (SIZE(hdgMat) == 0) THEN
          WRITE(6, '(a)')        ' No materials found in dataHorse'
       ELSE
          WRITE(6, '(a,i7,a)')   ' Found ',SIZE(hdgMat),' materials while reading dataHorse'
          DO iMat=1,SIZE(hdgMat)
             WRITE(6, '(a,i7)')     '                 Material number : ',iMat
             WRITE(6, '(a,3f11.3)') '                 Epsilon         : ',hdgMat(iMat)%eps
             WRITE(6, '(a,3f11.3)') '                 Mu              : ',hdgMat(iMat)%mu
             WRITE(6, '(a,3f11.3)') '                 Sigma           : ',hdgMat(iMat)%sigma
             WRITE(6, '(a,3f11.3)') '                 Rho             : ',hdgMat(iMat)%rho
             WRITE(6, '(a,3f11.3)') '                 Tau             : ',hdgMat(iMat)%tau
             !
             IF (iMat /= SIZE(hdgMat)) THEN
                WRITE(6, '(a)')     '                                   '
             ENDIF
          ENDDO
       ENDIF
       WRITE(6, 10)
    ENDIF
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
10  FORMAT(60('-'))
    !------------------------------------------------------------------
  END SUBROUTINE PRINT_MATERIAL_STATS

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE MATERIAL_STRUCTURES
