MODULE WRITE_FILES

  !====================================================================
  ! Everything for output files
  !====================================================================

  !--------------------------------------------------------------------
  ! Shared modules
  !--------------------------------------------------------------------
  USE ISO_C_BINDING
  !--------------------------------------------------------------------
  IMPLICIT NONE
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  ! Stuff related to EM field output files
  !
  ! WRITE_CELLS_FIELD => procedure pointer to subroutine writing EM field in cells
  ! WRITE_FACES_FIELD => procedure pointer to subroutine writing EM field on faces
  !--------------------------------------------------------------------
  PROCEDURE(), POINTER :: WRITE_CELLS_FIELD => NULL()
  PROCEDURE(), POINTER :: WRITE_FACES_FIELD => NULL()
  !--------------------------------------------------------------------

CONTAINS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE PREPROCESS_WRITE_FIELDS
    !------------------------------------------------------------------
    !---- Initializes writing DoFs of EM field on output files
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Make WRITE_CELLS_FIELD and WRITE_FACES_FIELD points to the 
    !---- right routineS
    !------------------------------------------------------------------
    IF (cellField) WRITE_CELLS_FIELD => WRITE_EM_DOFS_CELLS
    IF (faceField) WRITE_FACES_FIELD => WRITE_EM_DOFS_FACES
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE PREPROCESS_WRITE_FIELDS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE POSTPROCESS_WRITE_FIELDS
    !------------------------------------------------------------------
    !---- Finalizes writing of DoFs of EM field on output files
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------ 
    IF (cellField) CALL WRITE_CELLS_FIELD
    IF (faceField) CALL WRITE_FACES_FIELD
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE POSTPROCESS_WRITE_FIELDS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_EM_DOFS_CELLS
    !------------------------------------------------------------------
    !---- Write DoFs of EM field in cells 
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)  :: writeUnit

    LOGICAL(C_BOOL) :: ImE, ImH, ReE, ReH
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    IF (idWrite) THEN
       WRITE(6, 10)
       WRITE(6, '(a)') " Output: DoFs of EM field in mesh cells "
       WRITE(6, 10)
    ENDIF
    !------------------------------------------------------------------
    !---- Some variables for initialization
    !------------------------------------------------------------------
    writeUnit = 1
    ReE       = .TRUE.
    ImE       = .TRUE.
    ReH       = .TRUE.
    ImH       = .TRUE.
    !------------------------------------------------------------------
    !---- Writes coordinates of DoFs in cells
    !------------------------------------------------------------------
    CALL WRITE_COORD_DOFS_CELLS(writeUnit)
    !------------------------------------------------------------------
    !---- Writes  DoFs of E field in cells
    !------------------------------------------------------------------    
    CALL WRITE_FIELD_DOFS_CELLS(writeUnit,"E",ReE,ImE)
    !------------------------------------------------------------------
    !---- Writes  DoFs of H field in cells
    !------------------------------------------------------------------    
    CALL WRITE_FIELD_DOFS_CELLS(writeUnit,"H",ReH,ImH)
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
10  FORMAT(60('-'))    
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_EM_DOFS_CELLS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_EM_DOFS_FACES
    !------------------------------------------------------------------
    !---- Write DoFs of EM field on faces
    !
    !---- IN
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: writeUnit

    LOGICAL(C_BOOL) :: ImE, ImH, ReE, ReH
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    IF (idWrite) THEN
       WRITE(6, 10)
       WRITE(6, '(a)') " Output: DoFs of EM field on mesh faces "
       WRITE(6, 10)
    ENDIF
    !------------------------------------------------------------------
    !---- Some variables for initialization
    !------------------------------------------------------------------
    writeUnit = 1
    ReE       = .TRUE.
    ImE       = .TRUE.
    ReH       = .TRUE.
    ImH       = .TRUE.
    !------------------------------------------------------------------
    !---- Writes coordinates of DoFs on faces
    !------------------------------------------------------------------
    CALL WRITE_COORD_DOFS_FACES(writeUnit)
    !------------------------------------------------------------------
    !---- Writes  DoFs of E field on faces
    !------------------------------------------------------------------    
    CALL WRITE_FIELD_DOFS_FACES(writeUnit,"E",ReE,ImE)
    !------------------------------------------------------------------
    !---- Writes  DoFs of H field on faces
    !------------------------------------------------------------------    
    CALL WRITE_FIELD_DOFS_FACES(writeUnit,"H",ReH,ImH)

    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
10  FORMAT(60('-'))
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_EM_DOFS_FACES

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_COORD_DOFS_CELLS(nUnit)
    !------------------------------------------------------------------
    !---- Write coordinates of Dofs of mesh cells on file 
    !
    !---- IN
    !---- nUnit : unit number for writing file
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE IO_FILES
    USE MESH_CELLS
    USE MESH_DATA
    USE MESH_ELEMENTS
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT), INTENT(IN) :: nUnit
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: iCell, iDof, nDofV, order, subMesh

    CHARACTER(LEN=8, KIND=C_CHAR) :: csubMesh
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Write DoFs coordinates of mesh cells
    !------------------------------------------------------------------
    subMesh = idProc + 1
    CALL INT_TO_CHAR(subMesh,csubMesh)
    OPEN(UNIT=nUnit, FILE="Coord_DoFs_Cell-subMesh"//TRIM(csubMesh),   &
         STATUS="UNKNOWN")
    WRITE(nUnit, '(i0)') Mesh%nCell
    DO iCell=1,Mesh%nCell
       order = hdgEM(iCell)%order
       nDofV = hdgEM(iCell)%nDofV
       CALL GET_CELL_DOF(iCell, order, nDofV, dofXV(1:nDofV),          &
            dofYV(1:nDofV), dofZV(1:nDofV))
       DO iDof=1,nDofV
          WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,        &
               dofXV(iDof), dofYV(iDof), dofZV(iDof)
       ENDDO
    ENDDO
    CLOSE(nUnit)
    !
    !---- Print information of output file
    IF (idWrite) THEN
       WRITE(6, '(a)') " Writing of DoFs coordinates                "
       WRITE(6, '(a)') "   --> Filenames : Coord_DoFs_Cell-subMesh* "
    ENDIF
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_COORD_DOFS_CELLS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_COORD_DOFS_MAT(nUnit)
    !------------------------------------------------------------------
    !---- Write coordinates of Dofs of materials on file
    !---- (usefull for SAR in tissues visu.)
    !
    !---- IN
    !---- nUnit : unit number for writing file
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE IO_FILES
    USE MESH_CELLS
    USE MESH_DATA
    USE MESH_ELEMENTS
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT), INTENT(IN) :: nUnit
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: iCell, iDof, ncellMat, nDofV, order, subMesh

    CHARACTER(LEN=8, KIND=C_CHAR) :: csubMesh
    !------------------------------------------------------------------
    !---- Testing for bad input 
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Count the number of cells for materials
    !------------------------------------------------------------------
    ncellMat = 0
    DO iCell=1,Mesh%nCell
       IF (MeshCell(iCell)%log > 0) ncellMat = ncellMat + 1
    ENDDO
    !------------------------------------------------------------------
    !---- Write coordinates of DoFs of materials
    !------------------------------------------------------------------       
    subMesh = idProc + 1
    CALL INT_TO_CHAR(subMesh,csubMesh)
    OPEN(UNIT=nUnit, FILE="Coord_DoFs_Mat-subMesh"//TRIM(csubMesh),    &
         STATUS="UNKNOWN")
    WRITE(nUnit, '(i0)') ncellMat
    do_cell : DO iCell=1,Mesh%nCell
       IF (MeshCell(iCell)%log == 0) CYCLE do_cell
       order = hdgEM(iCell)%order
       nDofV = hdgEM(iCell)%nDofV
       CALL GET_CELL_DOF(iCell, order, nDofV, dofXV(1:nDofV),          &
            dofYV(1:nDofV), dofZV(1:nDofV))
       DO iDof=1,nDofV
          WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,        &
               dofXV(iDof), dofYV(iDof), dofZV(iDof)
       ENDDO
    ENDDO do_cell
    CLOSE(nUnit)
    !
    !---- Print information of output file
    IF (idWrite) THEN
       WRITE(6, '(a)') " Writing of DoFs coordinates of materials  "
       WRITE(6, '(a)') "   --> Filenames : Coord_DoFs_Mat-subMesh* "
    ENDIF
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_COORD_DOFS_MAT

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_FIELD_DOFS_CELLS(nUnit, field, ReField, ImField) 
    !------------------------------------------------------------------
    !---- Write DoFs of a field on mesh cells 
    !
    !---- IN
    !---- nUnit     : unit number for writing files
    !---- field     : field of interest
    !---- ReField   : writing(=TRUE)/not writing(= FALSE) real part of field
    !---- ImField   : writing(=TRUE)/not writing(= FALSE) imaginary part of field
    !---- field     : field of interest
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE IO_FILES
    USE MATH_PHYS_CONSTANTS
    USE MATRICES_STRUCTURES
    USE MESH_CELLS
    USE MESH_DATA
    USE MESH_FACES
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT),                INTENT(IN) :: nUnit

    LOGICAL(C_BOOL), OPTIONAL,     INTENT(IN) :: ImField, ReField

    CHARACTER(LEN=*, KIND=C_CHAR), INTENT(IN) :: field
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT)  :: iCell, i, iDof, nDofV, subMesh

    REAL(C_DOUBLE)  :: coefSAR
    
    LOGICAL(C_BOOL) :: ImFieldLoc, ReFieldLoc

    CHARACTER(LEN=8, KIND=C_CHAR) :: csubMesh
    !------------------------------------------------------------------
    !---- Testing for bad input + writing files
    !------------------------------------------------------------------
    ReFieldLoc = .TRUE.
    ImFieldLoc = .TRUE.
    IF (PRESENT(ReField)) ReFieldLoc = ReField
    IF (PRESENT(ImField)) ImFieldLoc = ImField
    SELECT CASE(TRIM(field))
    CASE("E")
       subMesh = idProc + 1
       CALL INT_TO_CHAR(subMesh,csubMesh)
       IF (ReFieldLoc) THEN
          !
          !---- We write DoFs of the real part of the electric field 
          !---- on mesh cells
          OPEN(UNIT=nUnit, FILE="ReE_DoFs_Cell-subMesh"//TRIM(csubMesh),&
               STATUS="UNKNOWN")
          DO iCell=1,Mesh%nCell
             nDofV = hdgEM(iCell)%nDofV
             DO iDof=1,nDofV
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,&
                     (REAL(hdgEM(iCell)%E%vec(iDof,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Re(E) DoFs                    "
             WRITE(6, '(a)') "   --> Filenames : ReE_DoFs_Cell-subMesh* "
          ENDIF
       ENDIF
       IF (ImFieldLoc) THEN
          !
          !---- We write DoFs of the imaginary part of the electric field 
          !---- on mesh cells
          OPEN(UNIT=nUnit, FILE="ImE_DoFs_Cell-subMesh"//TRIM(csubMesh),&
               STATUS="UNKNOWN")
          DO iCell=1,Mesh%nCell
             nDofV = hdgEM(iCell)%nDofV
             DO iDof=1,nDofV
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,&
                     (AIMAG(hdgEM(iCell)%E%vec(iDof,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Im(E) DoFs                    "
             WRITE(6, '(a)') "   --> Filenames : ImE_DoFs_Cell-subMesh* "
          ENDIF
       ENDIF
    CASE("H")
       subMesh = idProc + 1
       CALL INT_TO_CHAR(subMesh,csubMesh)
       IF (ReFieldLoc) THEN
          !
          !---- We write DoFs of the real part of the magnetic field 
          !---- on mesh cells
          OPEN(UNIT=nUnit, FILE="ReH_DoFs_Cell-subMesh"//TRIM(csubMesh),&
               STATUS="UNKNOWN")
          DO iCell=1,Mesh%nCell
             nDofV = hdgEM(iCell)%nDofV
             DO iDof=1,nDofV
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,&
                     (REAL(hdgEM(iCell)%H%vec(iDof,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Re(H) DoFs                    "
             WRITE(6, '(a)') "   --> Filenames : ReH_DoFs_Cell-subMesh* "
          ENDIF
       ENDIF
       IF (ImFieldLoc) THEN
          !
          !---- We write DoFs of the imaginary part of the magnetic field 
          !---- on mesh cells
          OPEN(UNIT=nUnit, FILE="ImH_DoFs_Cell-subMesh"//TRIM(csubMesh),&
               STATUS="UNKNOWN")
          DO iCell=1,Mesh%nCell
             nDofV = hdgEM(iCell)%nDofV
             DO iDof=1,nDofV
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshCell(iCell)%log,&
                     (AIMAG(hdgEM(iCell)%H%vec(iDof,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Im(H) DoFs                    "
             WRITE(6, '(a)') "   --> Filenames : ImH_DoFs_Cell-subMesh* "
          ENDIF
       ENDIF
    CASE("SAR")
       subMesh = idProc + 1
       CALL INT_TO_CHAR(subMesh,csubMesh)
       !
       !---- We write SAR DoFs for mesh cells materials (tissues)
       OPEN(UNIT=nUnit, FILE="SAR_DoFs_Cell-subMesh"//TRIM(csubMesh),  &
            STATUS="UNKNOWN")
       do_sar : DO iCell=1,Mesh%nCell
          IF (MeshCell(iCell)%log <= 0) CYCLE do_sar
          nDofV    = hdgEM(iCell)%nDofV
          coefSAR  = hdgEM(iCell)%sigma / (hdgEM(iCell)%rho*z0)
          DO iDof=1,nDofV
             WRITE(nUnit, '(i0,e15.6)') MeshCell(iCell)%log,            &
                  coefSAR*REAL(                                            &
                  hdgEM(iCell)%E%vec(iDof,1)*CONJG(hdgEM(iCell)%E%vec(iDof,1))   &
                  + hdgEM(iCell)%E%vec(iDof,2)*CONJG(hdgEM(iCell)%E%vec(iDof,2)) &
                  + hdgEM(iCell)%E%vec(iDof,3)*CONJG(hdgEM(iCell)%E%vec(iDof,3)))
          ENDDO
       ENDDO do_sar
       CLOSE(nUnit)
       IF (idWrite) THEN
          WRITE(6, '(a)') " Writing of SAR DoFs for mesh cells materials "
          WRITE(6, '(a)') "   --> Filenames : SAR_DoFs_Cell-subMesh*     "
       ENDIF
    CASE DEFAULT
       !
       !---- Bad input
       IF (idWrite) THEN
          WRITE(6, '(a,a)') " Unknown type of field for writing output files: ", field
          WRITE(6, '(a)')   " Bad input in WRITE_FIELD_DOFS_CELLS routine     "
          WRITE(6, '(a)')   " Available type of fields are:                   "
          WRITE(6, '(a)')   " - field = E for electric field                  "
          WRITE(6, '(a)')   " - field = H for magnetic field                  "
          WRITE(6, '(a)')   " - field = SAR for SAR values                    "
       ENDIF
       RETURN
    END SELECT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_FIELD_DOFS_CELLS

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_COORD_DOFS_FACES(nUnit)
    !------------------------------------------------------------------
    !---- Write coordinates of Dofs of mesh faces on file 
    !
    !---- IN
    !---- nUnit : unit number for writing file
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE IO_FILES
    USE MESH_DATA
    USE MESH_ELEMENTS
    USE MESH_FACES
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT), INTENT(IN) :: nUnit
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: iCell, iDof, iFac, nDofS, nDofV, order, subMesh

    CHARACTER(LEN=8, KIND=C_CHAR) :: csubMesh
    !------------------------------------------------------------------
    !---- Testing for bad input
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !---- Write DoFs coordinates of mesh faces 
    !------------------------------------------------------------------
    subMesh = idProc + 1
    CALL INT_TO_CHAR(subMesh,csubMesh)
    OPEN(UNIT=nUnit, FILE="Coord_DoFs_Fac-subMesh"//TRIM(csubMesh),    &
         STATUS="UNKNOWN")
    WRITE(nUnit, '(i0)') Mesh%nFac
    DO iFac=1,Mesh%nFac
       iCell = MeshFac(iFac)%neighb(1)
       order = hdgEM(iCell)%order
       nDofS = hdgEM(iCell)%nDofS
       nDofV = hdgEM(iCell)%nDofV
       CALL GET_FACE_DOF(iFac, order, nDofS, nDofV, dofXS(1:nDofS),    &
            dofYS(1:nDofS), dofZS(1:nDofS))
       DO iDof=1,nDofS
          WRITE(nUnit, '(i0,3(e15.6,1x))') MeshFac(iFac)%log,          &
               dofXS(iDof), dofYS(iDof), dofZS(iDof)
       ENDDO
    ENDDO
    CLOSE(nUnit)
    !------------------------------------------------------------------
    !---- Print information of output files
    !------------------------------------------------------------------    
    IF (idWrite) THEN
       WRITE(6, '(a)') " Writing of DoFs coordinates                "
       WRITE(6, '(a)') "   --> Filenames : Coord_DoFs_Fac-subMesh*  "
    ENDIF
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_COORD_DOFS_FACES

  !====================================================================
  !====================================================================
  !====================================================================

  SUBROUTINE WRITE_FIELD_DOFS_FACES(nUnit, field, ReField, ImField) 
    !------------------------------------------------------------------
    !---- Write DoFs of a field on mesh faces 
    !
    !---- IN
    !---- nUnit    : unit number for writing file
    !---- field    : field of interest
    !---- ReField  : writing(=TRUE)/not writing(= FALSE) real part of field
    !---- ImField  : writing(=TRUE)/not writing(= FALSE) imaginary part of field
    !
    !---- OUT
    !
    !------------------------------------------------------------------
    !---- Modules
    !------------------------------------------------------------------
    USE INPUT_DATA
    USE IO_FILES
    USE MESH_CELLS
    USE MESH_DATA
    USE MESH_FACES
    USE NUMERICAL_TYPES
    USE PARALLEL_STRUCTURES
    USE REFERENCE_CELLS
    !------------------------------------------------------------------
    !---- I/O variables
    !------------------------------------------------------------------
    INTEGER(C_INT),                INTENT(IN) :: nUnit

    LOGICAL(C_BOOL),               INTENT(IN) :: ReField, ImField

    CHARACTER(LEN=1, KIND=C_CHAR), INTENT(IN) :: field
    !------------------------------------------------------------------
    !---- Local variables
    !------------------------------------------------------------------
    INTEGER(C_INT) :: i, iCell, iDof, iDofk, iFac, k, nDofS, order,    &
         Perm, subMesh

    CHARACTER(LEN=8, KIND=C_CHAR) :: csubMesh
    !------------------------------------------------------------------
    !---- Testing for bad input + writing files
    !------------------------------------------------------------------
    SELECT CASE(TRIM(field))
    CASE("E")
       subMesh = idProc + 1
       CALL INT_TO_CHAR(subMesh,csubMesh)
       IF (ReField) THEN
          !
          !---- We write DoFs of the real part of the electric field 
          !---- on file (for mesh faces with log = facLog)
          OPEN(UNIT=nUnit, FILE="ReE_DoFs_Fac-subMesh"//TRIM(csubMesh), &
               STATUS="UNKNOWN")
          DO iFac=1,Mesh%nFac
             iCell = MeshFac(iFac)%neighb(1)
             order = hdgEM(iCell)%order
             nDofS = hdgEM(iCell)%nDofS
             k     = MeshFac(iFac)%numLocal(1)
             Perm  = MeshCell(iCell)%perm(1,k)
             DO iDof=1,nDofS
                iDofk = hdgEM(iCell)%iPermDof%ptr%tab(iDof,Perm)
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshFac(iFac)%log,     &
                     (REAL(hdgEM(iCell)%E%vec(iDofk,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Re(E) DoFs                   "
             WRITE(6, '(a)') "   --> Filenames : ReE_DoFs_Fac-subMesh* "
          ENDIF
       ENDIF
       !
       IF (ImField) THEN
          !
          !---- We write DoFs of the imaginary part of the electric field 
          !---- on file (for mesh faces with log = facLog)
          OPEN(UNIT=nUnit, FILE="ImE_DoFs_Fac-subMesh"//TRIM(csubMesh), &
               STATUS="UNKNOWN")
          DO iFac=1,Mesh%nFac
             iCell = MeshFac(iFac)%neighb(1)
             order = hdgEM(iCell)%order
             nDofS = hdgEM(iCell)%nDofS
             k     = MeshFac(iFac)%numLocal(1)
             Perm  = MeshCell(iCell)%perm(1,k)
             DO iDof=1,nDofS
                iDofk = hdgEM(iCell)%iPermDof%ptr%tab(iDof,Perm)
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshFac(iFac)%log,     &
                     (AIMAG(hdgEM(iCell)%E%vec(iDofk,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Im(E) DoFs                   "
             WRITE(6, '(a)') "   --> Filenames : ImE_DoFs_Fac-subMesh* "
          ENDIF
       ENDIF
    CASE("H")
       subMesh = idProc + 1
       CALL INT_TO_CHAR(subMesh,csubMesh)
       IF (ReField) THEN
          !
          !---- We write DoFs of the real part of the magnetic field 
          !---- on file (for mesh faces with log = facLog)
          OPEN(UNIT=nUnit, FILE="ReH_DoFs_Fac-subMesh"//TRIM(csubMesh), &
               STATUS="UNKNOWN")
          DO iFac=1,Mesh%nFac
             iCell = MeshFac(iFac)%neighb(1)
             order = hdgEM(iCell)%order
             nDofS = hdgEM(iCell)%nDofS
             k     = MeshFac(iFac)%numLocal(1)
             Perm  = MeshCell(iCell)%perm(1,k)
             DO iDof=1,nDofS
                iDofk = hdgEM(iCell)%iPermDof%ptr%tab(iDof,Perm)
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshFac(iFac)%log,     &
                     (REAL(hdgEM(iCell)%H%vec(iDofk,i)), i=1,3)
             ENDDO
          ENDDO
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Re(H) DoFs                   "
             WRITE(6, '(a)') "   --> Filenames : ReH_DoFs_Fac-subMesh* "
          ENDIF
          CLOSE(nUnit)
       ENDIF
       !
       IF (ImField) THEN
          !
          !---- We write DoFs of the imaginary part of the magnetic field 
          !---- on file (for mesh faces with log = facLog)
          OPEN(UNIT=nUnit, FILE="ImH_DoFs_Fac-subMesh"//TRIM(csubMesh), &
               STATUS="UNKNOWN")
          DO iFac=1,Mesh%nFac
             iCell = MeshFac(iFac)%neighb(1)
             order = hdgEM(iCell)%order
             nDofS = hdgEM(iCell)%nDofS
             k     = MeshFac(iFac)%numLocal(1)
             Perm  = MeshCell(iCell)%perm(1,k)
             DO iDof=1,nDofS
                iDofk = hdgEM(iCell)%iPermDof%ptr%tab(iDof,Perm)
                WRITE(nUnit, '(i0,3(e15.6,1x))') MeshFac(iFac)%log,     &
                     (AIMAG(hdgEM(iCell)%H%vec(iDofk,i)), i=1,3)
             ENDDO
          ENDDO
          CLOSE(nUnit)
          IF (idWrite) THEN
             WRITE(6, '(a)') " Writing of Im(H) DoFs                   "
             WRITE(6, '(a)') "   --> Filenames : ImH_DoFs_Fac-subMesh* "
          ENDIF
       ENDIF
    CASE DEFAULT
       IF (idWrite) THEN
          WRITE(6, '(a,a)') " Unknown type of field for writing output files: ", field
          WRITE(6, '(a)')   " Bad input in WRITE_FIELD_DOFS_FACES routine     "
          WRITE(6, '(a)')   " Available type of fields are:                   "
          WRITE(6, '(a)')   " - field = E for electric field                  "
          WRITE(6, '(a)')   " - field = H for magnetic field                  "
       ENDIF
       RETURN
    END SELECT
    !------------------------------------------------------------------
    !---- Testing for errors
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
    !
    !------------------------------------------------------------------
  END SUBROUTINE WRITE_FIELD_DOFS_FACES

  !====================================================================
  !====================================================================
  !====================================================================

END MODULE WRITE_FILES
